@@ -1,6 +1,6 @@
 // Copyright (C) 2004 John Farrell
 // Copyright (C) 2004, 2005, 2006, 2007 Ulf Lorenz
-// Copyright (C) 2008, 2009, 2010, 2014, 2015 Ben Asselstine
+// Copyright (C) 2008, 2009, 2010, 2014, 2015, 2017 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -45,9 +45,10 @@
 #include "QEnemyArmies.h"
 #include "QEnemyArmytype.h"
 #include "rnd.h"
+#include "reward.h"
 
-#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::flush<<std::endl;}
-//#define debug(x)
+//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::flush<<std::endl;}
+#define debug(x)
 
 AI_Allocation* AI_Allocation::s_instance = 0;
 
@@ -105,7 +106,8 @@ int AI_Allocation::allocateStackToCapacityBuilding(Threat *threat, City *first_c
   bool killed = false;
   //only take what we need.
   std::list<guint32> armies = attacker->determineStrongArmies(3.0);
-  if (armies.size() > 0 && armies.size() != attacker->size())
+  if (armies.size() > 0 && armies.size() != attacker->size() &&
+      !attacker->fliesWithItemAndNonFlyersOverWaterOrMountains())
     {
       Stack *stack = d_owner->stackSplitArmies(attacker, armies);
       moved = moveStack(stack, dest, killed);
@@ -170,8 +172,7 @@ int AI_Allocation::continueQuests()
   if (GameScenarioOptions::s_play_with_quests == GameParameters::NO_QUESTING)
     return count;
 
-  std::vector<Quest*> quests =
-    QuestsManager::getInstance()->getPlayerQuests(d_owner);
+  auto quests = QuestsManager::getInstance()->getPlayerQuests(d_owner);
   for (std::vector<Quest*>::iterator i = quests.begin(); i != quests.end(); i++)
     {
       Quest *quest = *i;
@@ -246,15 +247,12 @@ int AI_Allocation::attackNearbyEnemies()
 {
   bool moved;
   int count = 0;
-  Citylist *cl = Citylist::getInstance();
-  for (Citylist::iterator i = cl->begin(); i != cl->end(); i++)
+  for (auto city: *Citylist::getInstance())
     {
-      sbusy.emit();
       if (d_owner->abortRequested())
         return count;
       if (d_stacks->size() == 0)
         break;
-      City *city = *i;
       if (city->getOwner() == d_owner || city->isBurnt() == true)
         continue;
       std::list<Vector<int> > p = GameMap::getNearbyPoints(city->getPos(), 2);
@@ -285,6 +283,7 @@ int AI_Allocation::attackNearbyEnemies()
             }
           //break;
         }
+      sbusy.emit();
     }
   //return count;
   //attack nearby stacks in the field.
@@ -292,7 +291,6 @@ int AI_Allocation::attackNearbyEnemies()
   std::list<Vector<int> > pos = sl->getPositions();
   for (std::list<Vector<int> >::iterator i = pos.begin(); i != pos.end(); i++)
     {
-      sbusy.emit();
       if (d_owner->abortRequested())
         return count;
       Stack *s = GameMap::getFriendlyStack(*i);
@@ -328,6 +326,7 @@ int AI_Allocation::attackNearbyEnemies()
           else
             break;
         }
+      sbusy.emit();
     }
   //don't leave heroes sitting around in cities.
   pos = sl->getPositions();
@@ -344,7 +343,7 @@ int AI_Allocation::attackNearbyEnemies()
         continue;
       if (s->getMoves() < s->getMaxMoves())
         continue;
-      City *target = cl->getClosestEnemyCity(s);
+      City *target = Citylist::getInstance()->getClosestEnemyCity(s);
       if (target)
         {
           bool killed = false;
@@ -357,6 +356,7 @@ int AI_Allocation::attackNearbyEnemies()
                 deleteStack(s);
             }
         }
+      sbusy.emit();
     }
    //fixme: this should probably be commented out in favour of emptyOutCities
   //don't leave stacks of eight lying around in cities.
@@ -380,7 +380,7 @@ int AI_Allocation::attackNearbyEnemies()
       if (city->isBurnt() == false && 
           city->getDefenders().size() > MAX_STACK_SIZE * 2)
         continue;
-      City *target = cl->getClosestEnemyCity(s);
+      City *target = Citylist::getInstance()->getClosestEnemyCity(s);
       if (target)
         {
           bool killed = false;
@@ -393,6 +393,7 @@ int AI_Allocation::attackNearbyEnemies()
                 deleteStack(s);
             }
         }
+      sbusy.emit();
     }
   return count;
 }
@@ -400,14 +401,10 @@ int AI_Allocation::attackNearbyEnemies()
 bool AI_Allocation::emptyOutCities()
 {
   //everybody out on the dancefloor.
-  Citylist *cl = Citylist::getInstance();
-  for (Citylist::iterator it = cl->begin(); it != cl->end(); it++)
+  for (auto c: *Citylist::getInstance())
     {
-      
-      sbusy.emit();
       if (d_owner->abortRequested())
         return false;
-      City *c = *it;
       if (c->getOwner() != d_owner || c->isBurnt() == true)
         continue;
       bool bail = false;
@@ -422,7 +419,8 @@ bool AI_Allocation::emptyOutCities()
               if ((s->getMoves() > 3 && s->size() >= 4 &&
                    (num_defenders - s->size()) >= 3) || (Rnd::rand() % 10) == 0)
                 {
-                  City *target = cl->getNearestEnemyCity(s->getPos());
+                  City *target = 
+                    Citylist::getInstance()->getNearestEnemyCity(s->getPos());
                   if (target)
                     {
                       bool killed = false;
@@ -441,6 +439,7 @@ bool AI_Allocation::emptyOutCities()
           if (bail)
             break;
         }
+      sbusy.emit();
     }
   return true;
 }
@@ -564,8 +563,8 @@ int AI_Allocation::oldPickupItems()
   for (std::vector<Vector<int> >::iterator i = items.begin(); i != items.end();
        i++)
     {
-      std::list<Stack*> stks = GameMap::getNearbyFriendlyStacks(*i, 8);
-      for (std::list<Stack*>::iterator j = stks.begin(); j != stks.end(); j++)
+      std::vector<Stack*> stks = GameMap::getNearbyFriendlyStacks(*i, 8);
+      for (std::vector<Stack*>::iterator j = stks.begin(); j != stks.end(); j++)
         {
           Stack *s = *j;
           if (s->hasHero() == false)
@@ -621,15 +620,18 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
   int attack_moved = 0, defensive_moved = 0, capacity_moved = 0, offensive_moved = 0, default_moved= 0;
   int temple_alloc = 0, ruin_alloc = 0, pickup_alloc = 0, attack_alloc = 0, quest_alloc = 0, immediate_alloc = 0, defensive_alloc = 0, capacity_alloc = 0, offensive_alloc = 0, default_alloc= 0;
   int moved;
+  (void) attack_moved;
+  (void) defensive_moved;
+  (void) capacity_moved;
+  (void) offensive_moved;
+  (void) default_moved;
   // move stacks
   d_stacks = new StackReflist(d_owner->getStacklist(), true);
 
-  int total = d_stacks->size();
   debug("Player " << d_owner->getName() << " starts with " << d_stacks->size() << " stacks to do something with");
 
   int count = 0;
 
-  sbusy.emit();
   if (d_owner->abortRequested())
     return count;
 
@@ -640,7 +642,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
   debug("Player " << d_owner->getName() << " still has " << d_stacks->size() << " stacks after allocating stacks to fulfilling quests");
   debug("Player " << d_owner->getName() << " moved " << moved << " stacks in quest mode.");
 
-  sbusy.emit();
   if (d_owner->abortRequested())
     return count;
 
@@ -651,7 +652,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
   debug("Player " << d_owner->getName() << " still has " << d_stacks->size() << " stacks after allocating stacks to visiting temples");
   debug("Player " << d_owner->getName() << " moved " << moved << " stacks in temple-visiting mode.");
 
-  sbusy.emit();
   if (d_owner->abortRequested())
     return count;
   //move hero stacks to ruins for searching.
@@ -661,7 +661,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
   debug("Player " << d_owner->getName() << " still has " << d_stacks->size() << " stacks after allocating stacks to visiting ruins");
   debug("Player " << d_owner->getName() << " moved " << moved << " stacks in ruin-visiting mode.");
 
-  sbusy.emit();
   if (d_owner->abortRequested())
     return count;
   //if we're near a bag of stuff, go pick it up.
@@ -671,7 +670,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
   debug("Player " << d_owner->getName() << " still has " << d_stacks->size() << " stacks after allocating stacks to picking up items");
   debug("Player " << d_owner->getName() << " moved " << moved << " stacks in pickup-items mode.");
 
-  sbusy.emit();
   if (d_owner->abortRequested())
     return count;
   // if a stack has a path for an enemy city and is outside of a city, then keep going.
@@ -682,7 +680,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
   debug("Player " << d_owner->getName() << " still has " << d_stacks->size() << " stacks after allocating stacks to continuing attacks");
   debug("Player " << d_owner->getName() << " moved " << moved << " stacks in continuing-attacks mode.");
 
-  sbusy.emit();
   if (d_owner->abortRequested())
     return count;
   // if a stack is 2 tiles away from another enemy city, then attack it.
@@ -692,7 +689,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
   debug("Player " << d_owner->getName() << " still has " << d_stacks->size() << " stacks after allocating stacks to attacking nearby stacks");
   debug("Player " << d_owner->getName() << " moved " << moved << " stacks in attack-nearby-stacks mode.");
 
-  sbusy.emit();
   if (d_owner->abortRequested())
     return count;
   //if (take_neutrals)
@@ -706,7 +702,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
       count+=moved;
     }
 
-  sbusy.emit();
   if (d_owner->abortRequested())
     return count;
   defensive_alloc = d_stacks->size();
@@ -723,7 +718,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
       return count;
     }
 
-  sbusy.emit();
   if (d_owner->abortRequested())
     return count;
   offensive_alloc = d_stacks->size();
@@ -740,7 +734,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
       return count;
     }
       
-  sbusy.emit();
   if (d_owner->abortRequested())
     return count;
   default_alloc = d_stacks->size();
@@ -750,7 +743,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
   debug("Player " << d_owner->getName() << " moved " << moved << " stacks in Default stack movements.");
   count+= moved;
 
-  sbusy.emit();
   if (d_owner->abortRequested())
     return count;
   //empty out the cities damnit.
@@ -791,6 +783,8 @@ int AI_Allocation::allocateDefensiveStacksToCity(City *city)
       Stack *defender = *it;
       if (defender->getParked() == true)
         continue;
+      if (defender->fliesWithItemAndNonFlyersOverWaterOrMountains())
+        continue;
       //shuffleStacksWithinCity(city, defender, Vector<int>(0,0));
       float stackStrength = d_analysis->assessStackStrength(defender);
       debug("Player " << d_owner->getName() << " assigns some or all of stack " << defender->getId() << " with strength " << stackStrength
@@ -851,6 +845,8 @@ int AI_Allocation::allocateDefensiveStacksToCity(City *city)
         Vector<int> dest = getFreeSpotInCity(city, s->size());
       if (dest == Vector<int>(-1,-1))
         break;
+      if (s->fliesWithItemAndNonFlyersOverWaterOrMountains())
+        break;
       float stackStrength = d_analysis->assessStackStrength(s);
       totalDefenderStrength += stackStrength;
 
@@ -895,7 +891,6 @@ int AI_Allocation::allocateDefensiveStacks(Citylist *cities)
       if (!city->isFriend(d_owner) || city->isBurnt())
 	continue;
       count += allocateDefensiveStacksToCity(city);
-      sbusy.emit();
       if (d_owner->abortRequested())
         return count;
 
@@ -992,7 +987,6 @@ int AI_Allocation::allocateStacksToThreats()
 
       if (d_stacks->size() == 0)
         break;
-      sbusy.emit();
       if (d_owner->abortRequested())
         return count;
 
@@ -1013,10 +1007,10 @@ Vector<int> AI_Allocation::getFreeOtherSpotInCity(City *city, Stack *stack)
           continue;
 	if (GameMap::canAddArmies(pos, stack->size()) == false)
 	  continue;
-        std::list<Stack*> f = GameMap::getFriendlyStacks(pos);
+        std::vector<Stack*> f = GameMap::getFriendlyStacks(pos);
         if (f.size() > 0)
           {
-            for (std::list<Stack*>::iterator k = f.begin(); k != f.end(); k++)
+            for (std::vector<Stack*>::iterator k = f.begin(); k != f.end(); k++)
               {
                 if ((*k)->size() > size)
                   {
@@ -1164,12 +1158,10 @@ Stack *AI_Allocation::findBestAttackerFor(Threat *threat, guint32 &city_defender
 int AI_Allocation::defaultStackMovements()
 {
   int count = 0;
-  Citylist *allCities = Citylist::getInstance();
   debug("Default movement for " <<d_stacks->size() <<" stacks");
 
   while (d_stacks->size() > 0)
     {
-      sbusy.emit();
       if (d_owner->abortRequested())
         return count;
       Stack* s = d_stacks->front();
@@ -1190,7 +1182,8 @@ int AI_Allocation::defaultStackMovements()
       if (leave == true)
         {
           bool moved = false;
-          City* enemyCity = allCities->getNearestEnemyCity(s->getPos());
+          City* enemyCity = 
+            Citylist::getInstance()->getNearestEnemyCity(s->getPos());
           if (enemyCity)
             {
               int mp = s->getPath()->calculate(s, enemyCity->getNearestPos(s->getPos()));
@@ -1212,7 +1205,8 @@ int AI_Allocation::defaultStackMovements()
             }
           else
             {
-              enemyCity = allCities->getNearestForeignCity(s->getPos());
+              enemyCity = 
+                Citylist::getInstance()->getNearestForeignCity(s->getPos());
               if (enemyCity)
                 {
                   s->getOwner()->proposeDiplomacy(Player::PROPOSE_WAR,
@@ -1263,20 +1257,19 @@ int AI_Allocation::defaultStackMovements()
           if (moved)
             count++;
         }
+      sbusy.emit();
     }
   return count;
 }
 
 bool AI_Allocation::stackReinforce(Stack *s)
 {
-  Citylist *allCities = Citylist::getInstance();
   float mostNeeded = -1000.0;
   City *cityNeeds = 0;
   int moves = 1000;
   Vector<int> target_tile = Vector<int>(-1,-1);
-  for (Citylist::iterator it = allCities->begin(); it != allCities->end(); ++it)
+  for (auto city: *Citylist::getInstance())
     {
-      City *city = (*it);
       if (city->getOwner() != d_owner)
         continue;
       if (city->isBurnt() == true)
@@ -1321,14 +1314,12 @@ bool AI_Allocation::stackReinforce(Stack *s)
   }
 
   //okay, no city needed us, just try to reinforce our nearest city
-  City *target = allCities->getNearestFriendlyCity(s->getPos());
+  City *target = Citylist::getInstance()->getNearestFriendlyCity(s->getPos());
   if (!target) // no friendly city?
     return false;
   //are we already there?
   if (target->contains(s->getPos()))
-    {
-      return false;
-    }
+    return false;
   else
     {
       Vector<int> dest = getFreeSpotInCity(target, s->size());
@@ -1348,7 +1339,8 @@ void AI_Allocation::searchRuin(Stack *stack, Ruin *ruin)
   if (reward && ruin->isSearched() == true && stack_died == false)
     {
       StackReflist *stacks = new StackReflist();
-      d_owner->giveReward(stack, reward, stacks);
+      d_owner->giveReward(stack, reward, stacks, false);
+      delete reward;
       delete stacks;
     }
   // what to do if the ruin search fails?
@@ -1368,7 +1360,7 @@ bool AI_Allocation::shuffleStacksWithinCity(City *city, Stack *stack,
   if (stack->getPos() == target)
     {
       debug("stack " << stack->getId() <<" at ("<<stack->getPos().x<<","<<stack->getPos().y<<") already in preferred position.");
-      std::list<Stack*> f = GameMap::getFriendlyStacks(target);
+      std::vector<Stack*> f = GameMap::getFriendlyStacks(target);
       if (f.size() > 1)
         {
           groupStacks(stack);
@@ -1378,12 +1370,12 @@ bool AI_Allocation::shuffleStacksWithinCity(City *city, Stack *stack,
       return false;
     }
 
-  std::list<Stack*> f = GameMap::getFriendlyStacks(target);
+  std::vector<Stack*> f = GameMap::getFriendlyStacks(target);
   if (f.size() > 1)
     {
       printf("i am stack %d at %d,%d\n", stack->getId(), stack->getPos().x, stack->getPos().y);
       printf("crap.  there are %lu stacks at %d,%d\n", f.size(), target.x, target.y);
-      for (std::list<Stack*>::iterator it = f.begin(); it != f.end(); it++)
+      for (std::vector<Stack*>::iterator it = f.begin(); it != f.end(); it++)
         {
           Stack *n = *it;
           if (n)
@@ -1446,7 +1438,6 @@ bool AI_Allocation::shuffleStack(Stack *stack, Vector<int> dest, bool split_if_n
     s->getPath()->setMovesExhaustedAtPoint(1);
   else
     s->getPath()->setMovesExhaustedAtPoint(0);
-  Vector<int> src = s->getPos();
   bool moved;
 
   if (split_if_necessary)
@@ -1479,11 +1470,9 @@ bool AI_Allocation::shuffleStack(Stack *stack, Vector<int> dest, bool split_if_n
 
 bool AI_Allocation::moveStack(Stack *stack, bool &stack_died)
 {
-  guint32 stack_id = stack->getId();
   Stack *s = stack;
   assert (s != NULL);
   d_owner->getStacklist()->setActivestack(s);
-  Vector<int> src = s->getPos();
   bool moved;
 
   //printf("going in, size of path for stack: %d\n", s->getPath()->size());
@@ -1542,7 +1531,7 @@ bool AI_Allocation::groupStacks(Stack *stack)
   Stack *s = stack;
   debug("groupStacks on stack id " << stack->getId() << " at pos (" << s->getPos().x <<"," <<s->getPos().y<<")");
   //which friendly stacks are on that tile that aren't us?
-  std::list<Stack*> stks = GameMap::getFriendlyStacks(s->getPos());
+  std::vector<Stack*> stks = GameMap::getFriendlyStacks(s->getPos());
   if (stks.size() <= 1)
     {
       if (stks.front()->getId() != stack->getId())
