@@ -1,7 +1,7 @@
 // Copyright (C) 2003 Michael Bartl
 // Copyright (C) 2003, 2004, 2005, 2006 Ulf Lorenz
 // Copyright (C) 2003, 2006 Andrea Paternesi
-// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2014, 2015 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2014, 2015, 2020 Ben Asselstine
 // Copyright (C) 2008 Janek Kozicki
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -19,6 +19,7 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
+#pragma once
 #ifndef GAMEMAP_H
 #define GAMEMAP_H
 
@@ -41,6 +42,7 @@ class MapGenerator;
 class XML_Helper;
 class Port;
 class Road;
+class Stone;
 class City;
 class Temple;
 class Bridge;
@@ -134,9 +136,12 @@ class GameMap: public sigc::trackable
           */
         void setTileset(Glib::ustring tileset);
 
-        //! Return the width of a tile on the BigMap in pixels.
+        //! Return the width of a tile on the BigMap in pixels after scaling.
         guint32 getTileSize() const;
 
+        //! Return the width of a tile on the BigMap in pixels before scaling.
+        guint32 getUnscaledTileSize() const;
+
         //! Return the id of the current Tileset.  Returns zero if a valid tileset hasn't been set yet.
         guint32 getTilesetId() const;
 
@@ -184,7 +189,8 @@ class GameMap: public sigc::trackable
           * \note This method deletes the old tile if one is already present.
           * \note A new TileStyle is automatically assigned to the Maptile.
           */
-	void setTile(int x, int y, Maptile *tile);
+	//void setTile(int x, int y, Maptile *tile);
+	void setTileIndex(int x, int y, guint32 new_index);
 
         /** Change a Maptile on the map.
           * 
@@ -194,7 +200,8 @@ class GameMap: public sigc::trackable
           * \note This method deletes the old tile if one is already present.
           * \note A new TileStyle is automatically assigned to the Maptile.
           */
-        void setTile(Vector<int> p, Maptile *t) {return setTile(p.x, p.y, t);}
+        //void setTile(Vector<int> p, Maptile *t) {return setTile(p.x, p.y, t);}
+        void setTileIndex(Vector<int> p, guint32 new_index) {return setTileIndex(p.x, p.y, new_index);}
 
         /** Return a pointer to the City at the given position on the map.
           * 
@@ -439,9 +446,9 @@ class GameMap: public sigc::trackable
          * @param player Stacks owned by this Player will be returned.
          *
          * @return Returns an empty list if no stacks are found, otherwise a 
-         * list of pointers to Stack objects is returned.
+         * vector of pointers to Stack objects is returned.
          */
-	static std::list<Stack*> getFriendlyStacks(Vector<int> pos, Player *player = NULL);
+	static std::vector<Stack*> getFriendlyStacks(Vector<int> pos, Player *player = NULL);
 
         /** Return a pointer to a Stack at the given position not belonging to the active player.
          *
@@ -460,9 +467,9 @@ class GameMap: public sigc::trackable
          * @param player Stacks not owned by this Player will be returned.
          *
          * @return Returns an empty list if no stacks are found, otherwise a 
-         * list of pointers to Stack objects is returned.
+         * vector of pointers to Stack objects is returned.
          */
-	static std::list<Stack*> getEnemyStacks(Vector<int> pos, Player *player = NULL);
+	static std::vector<Stack*> getEnemyStacks(Vector<int> pos, Player *player = NULL);
 
         /** Returns all stacks not belonging to the active player at the given positions.
          *
@@ -472,7 +479,7 @@ class GameMap: public sigc::trackable
          * @return Returns an empty list if no stacks are found, otherwise a 
          * list of pointers to Stack objects is returned.
          */
-	static std::list<Stack*> getEnemyStacks(std::list<Vector<int> > posns);
+	static std::vector<Stack*> getEnemyStacks(std::list<Vector<int> > posns);
 
         /** Returns all stacks belonging to the active player that are within a given distance from a given position.
          *
@@ -485,9 +492,9 @@ class GameMap: public sigc::trackable
          * \note The returned list of Stack objects is sorted by their distance
          * away from pos.
          * @return Returns an empty list if no stacks are found, otherwise a 
-         * list of pointers to Stack objects is returned.
+         * vector of pointers to Stack objects is returned.
          */
-	static std::list<Stack*> getNearbyFriendlyStacks(Vector<int> pos, int dist);
+	static std::vector<Stack*> getNearbyFriendlyStacks(Vector<int> pos, int dist);
 
         /** Returns all stacks not belonging to the active player that are within a given distance from a given position.
          *
@@ -503,7 +510,7 @@ class GameMap: public sigc::trackable
          * @return Returns an empty list if no stacks are found, otherwise a 
          * list of pointers to Stack objects is returned.
          */
-	static std::list<Stack*> getNearbyEnemyStacks(Vector<int> pos, int dist);
+	static std::vector<Stack*> getNearbyEnemyStacks(Vector<int> pos, int dist);
 
         /** Returns a list of positions that are within a given distance from a given position.
          *
@@ -513,7 +520,7 @@ class GameMap: public sigc::trackable
          * \note When dist is 1, the size of the box is 3x3.
          * \note The returned list of positions is sorted by their distance away from pos (the center).
          *
-         * @return Returns a list of valid map positions.
+         * @return Returns a vector of valid map positions.
          */
         static std::list<Vector<int> > getNearbyPoints(Vector<int> pos, int dist);
 
@@ -541,6 +548,9 @@ class GameMap: public sigc::trackable
          */
 	static MapBackpack *getBackpack(Vector<int> pos);
 
+        //! Return how many bags of stuff there are on the map.
+        static guint32 countBags ();
+
         /** Check if the given Stack is able to search the Maptile it is on.
          *
          * @param stack A pointer to the stack to check if it can search.
@@ -605,7 +615,7 @@ class GameMap: public sigc::trackable
          * @return Returns NULL when the given position is out of range.  
          * Otherwise a pointer to a Maptile object is returned.
          */
-        inline Maptile* getTile(int x, int y) const { return d_map[y*s_width + x]; };
+        inline Maptile* getTile(int x, int y) const { return &d_map[y*s_width + x]; };
 
         /** Return a pointer to the Maptile object at the given position.
          *
@@ -948,6 +958,14 @@ class GameMap: public sigc::trackable
 	bool canPutBuilding(Maptile::Building bldg, guint32 size, Vector<int> to, bool making_islands = true);
 
 
+        /** Check if we can put some items here.
+         *
+         * @param pos  where on the map we're dropping the bag
+         *
+         * @return Returns True if we can drop it, otherwise false.
+         */
+        bool canDropBag (Vector<int> pos);
+
         /** Check if a stack of the given size, owned by the given player, can be added to the given position.
          * @param size The number of army units to check if we can add.
          * @param p Only stacks owned by this player are considered.
@@ -974,7 +992,7 @@ class GameMap: public sigc::trackable
 
         /** Move a bag of stuff from one position on the map, to another.
          *
-         * @param from The source position of a bag of stuff.
+         * @param bag The bag of stuff we're moving.
          * @param to The destination position.
          *
          * \note Every square has a Backpack object, but only some of them
@@ -983,7 +1001,7 @@ class GameMap: public sigc::trackable
          * If there are items in the Backpack located at the source position, 
          * they are removed and added to the destination position.
          */
-	void moveBackpack(Vector<int> from, Vector<int> to);
+	void moveBackpack(MapBackpack *bag, Vector<int> to);
 
         /** Returns the size of the building at the given position on the map.
          *
@@ -1145,6 +1163,11 @@ class GameMap: public sigc::trackable
          */
 	bool removeTemple(Vector<int> pos);
 
+	static Stone* getStone(Vector<int> pos);
+	bool putStone(Stone *t);
+        bool putNewStone(Vector<int> tile);
+	bool removeStone(Vector<int> pos);
+
         /** Add a given Road tile to the map.
          *
          * @param t A pointer to the Road tile to add to the map.
@@ -1438,12 +1461,13 @@ class GameMap: public sigc::trackable
 	void removeBuilding(LocationBox *b);
 
 	void updateShips(Vector<int> pos);
+        void updateTowers (Vector<int> pos);
 
         static void changeFootprintToSmallerCityset(Location *location, Maptile::Building building_type, guint32 old_tile_width);
 
         static void relocateLocation(Location *location, Maptile::Building building_type, guint32 tile_width);
 
-	static std::list<Stack*> getNearbyStacks(Vector<int> pos, int dist, bool friendly);
+	static std::vector<Stack*> getNearbyStacks(Vector<int> pos, int dist, bool friendly);
 
 	static bool offmap(int x, int y);
 
@@ -1461,7 +1485,7 @@ class GameMap: public sigc::trackable
         Glib::ustring d_shieldset; //the basename, not the friendly name.
         Glib::ustring d_cityset; //the basename, not the friendly name.
 
-        Maptile** d_map;
+        Maptile* d_map;
 };
 
 #endif
