@@ -1,4 +1,4 @@
-// Copyright (C) 2009, 2010, 2011, 2012, 2014, 2015 Ben Asselstine
+// Copyright (C) 2009-2012, 2014, 2015, 2017, 2020 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -14,6 +14,8 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
+
+#pragma once
 #ifndef PIXMASK_H
 #define PIXMASK_H
 
@@ -51,7 +53,7 @@ class PixMask
      void draw_pixbuf(Glib::RefPtr<Gdk::Pixbuf> pixbuf, int src_x, int src_y, int dest_x, int dest_y, int width, int height);
 
      //! scale a pixmask in place (alters pixmask)
-     static void scale(PixMask*& pixmask, int xsize, int ysize, Gdk::InterpType intper = Gdk::INTERP_NEAREST);
+     static void scale(PixMask*& pixmask, int xsize, int ysize, Gdk::InterpType intper = Gdk::INTERP_BILINEAR);
 
      //! draw this pixmask onto a pixmap.
      void blit(Cairo::RefPtr<Cairo::Surface> pixmap, int dest_x, int dest_y);
@@ -60,6 +62,8 @@ class PixMask
       //blit a tile's worth of imagery from this pixmask to a pixmap.
      void blit(Vector<int> tile, int ts, Cairo::RefPtr<Cairo::Surface> pixmap, Vector<int> dest = Vector<int>(0,0));
 
+     Vector<int> get_dim() const;
+     Vector<int> get_unscaled_dim() const;
      //! Destructor.
     ~PixMask();
  protected:
