@@ -20,21 +20,24 @@
 #include "stack.h"
 #include "armysetlist.h"
 #include "tileset.h"
+#include "path.h"
 #include "army.h"
 
 #define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::flush<<std::endl;}
 //#define debug(x)
 
-RoadPathCalculator::RoadPathCalculator(Vector<int> starting_point)
+RoadPathCalculator::RoadPathCalculator(Vector<int> starting_point, bool fly)
 {
   stack = new Stack(NULL, starting_point);
 
-  ArmyProto *basearmy = ArmyProto::createScout();
+  ArmyProto *basearmy =
+    fly ? ArmyProto::createBat () : ArmyProto::createScout();
   Army *a = Army::createNonUniqueArmy(*basearmy);
   delete basearmy;
   stack->add(a);
   path_calculator = new PathCalculator(stack, false);
 }
+
 RoadPathCalculator::RoadPathCalculator(const RoadPathCalculator &r)
 {
   stack = new Stack(*r.stack);
@@ -46,8 +49,37 @@ RoadPathCalculator::~RoadPathCalculator()
   delete stack;
   delete path_calculator;
 }
+
+guint32 RoadPathCalculator::calculate_moves(Vector<int> dest)
+{
+  guint32 moves = 0, turns = 0, left = 0;
+  Path *p = path_calculator->calculate(dest, moves, turns, left, false);
+  delete p;
+  return moves;
+}
+
 Path* RoadPathCalculator::calculate(Vector<int> dest)
 {
   guint32 moves = 0, turns = 0, left = 0;
   return path_calculator->calculate(dest, moves, turns, left, false);
 }
+
+Path* RoadPathCalculator::calculate(Vector<int> dest, guint32 &moves)
+{
+  guint32 turns = 0, left = 0;
+  return path_calculator->calculate(dest, moves, turns, left, false);
+}
+
+void RoadPathCalculator::regenerate()
+{
+  if (path_calculator)
+    delete path_calculator;
+  path_calculator = new PathCalculator(stack, false);
+}
+
+Vector<int> RoadPathCalculator::getPos() const
+{
+  if (stack)
+    return stack->getPos();
+  return Vector<int>(-1,-1);
+}
