@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009, 2014, 2015 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2014, 2015, 2020 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -33,8 +33,6 @@ ArmyMap::ArmyMap()
 
 void ArmyMap::draw_stacks()
 {
-  ImageCache *gc = ImageCache::getInstance();
-  
   // Draw stacks as tiny shields
   for (Playerlist::iterator pit = Playerlist::getInstance()->begin();
        pit != Playerlist::getInstance()->end(); pit++)
@@ -55,8 +53,9 @@ void ArmyMap::draw_stacks()
           if (Playerlist::getViewingplayer()->getFogMap()->isFogged (pos) == true)
             continue;
 
-          PixMask *tmp = gc->getShieldPic(1, (*it)->getOwner());
-          tmp = tmp->copy();
+          PixMask *tmp = 
+            ImageCache::getInstance()->getShieldPic(1, (*it)->getOwner(),
+                                                    true, 0)->copy();
           PixMask::scale(tmp, tmp->get_width()/2, tmp->get_height()/2);
 
           pos = mapToSurface(pos);
