@@ -2,7 +2,8 @@
 // Copyright (C) 2003, 2004, 2005, 2006, 2007 Ulf Lorenz
 // Copyright (C) 2004, 2005 Bryan Duff
 // Copyright (C) 2004, 2005, 2006 Andrea Paternesi
-// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2014, 2015 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2014, 2015, 2016, 2017,
+// 2020 Ben Asselstine
 // Copyright (C) 2007 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -37,6 +38,7 @@
 #include "port.h"
 #include "bridge.h"
 #include "road.h"
+#include "stone.h"
 #include "playerlist.h"
 #include "File.h"
 #include "stacktile.h"
@@ -46,18 +48,24 @@
 #include "FogMap.h"
 #include "MapBackpack.h"
 #include "GameScenarioOptions.h"
+#include "tileset.h"
 
 #include <iostream>
 //#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<std::flush;}
 #define debug(x)
 
-BigMap::BigMap()
-    : d_renderer(0), buffer(0), magnified_buffer(0), magnification_factor(1.0)
+bool BigMap::s_show_hidden_ruins;
+
+BigMap::BigMap(bool headless)
+    : d_headless (headless), d_renderer(0), buffer(0), d_fighting(LocationBox(Vector<int>(-1,-1)))
 {
     // note: we are not fully initialized before set_view is called
     view.x = view.y = 0;
     view.w = 0;
     view.h = 0;
+    deltax = 0;
+    deltay = 0;
+    view_pos = Vector<int>(0,0);
     input_locked = false;
     d_grid_toggled = false;
 
@@ -67,12 +75,9 @@ BigMap::BigMap()
 
 BigMap::~BigMap()
 {
-    if (buffer == true)
+    if (buffer)
       buffer.clear();
 
-    if (magnified_buffer == true)
-      magnified_buffer.clear();
-
     delete d_renderer;
 }
 
@@ -96,7 +101,7 @@ void BigMap::set_view(Rectangle new_view)
 	if (view_pos != new_view_pos)
 	{
 	    view_pos = new_view_pos;
-	    draw(Playerlist::getViewingplayer());
+	    draw();
 	}
 	
 	return;
@@ -112,7 +117,7 @@ void BigMap::set_view(Rectangle new_view)
     // this represents a 1 tile border around the outside of the picture.
     // it gets rid of the black border.
 
-    if (buffer == true)
+    if (buffer)
       buffer.clear();
     
     buffer_view.dim = view.dim + Vector<int>(2, 2);
@@ -122,7 +127,7 @@ void BigMap::set_view(Rectangle new_view)
     buffer_gc = Cairo::Context::create(buffer);
 
     //now create the part that will go out to the gtk::image
-    if (outgoing == true)
+    if (outgoing)
       outgoing.clear();
     outgoing = Cairo::Surface::create(buffer, Cairo::CONTENT_COLOR_ALPHA, image.get_width(), image.get_height());
 
@@ -147,12 +152,12 @@ void BigMap::clip_viewable_buffer(Cairo::RefPtr<Cairo::Surface> pixmap, Vector<i
   return;
 }
 
-void BigMap::draw(Player *player, bool redraw_buffer)
+void BigMap::draw(bool redraw_buffer)
 {
     // no size and buffer yet, return
-    if (!buffer)
+    if (!buffer || d_headless)
         return;
-    Playerlist::getInstance()->setViewingplayer(player);
+    //Playerlist::getInstance()->setViewingplayer(player);
 
     int tilesize = GameMap::getInstance()->getTileSize();
 
@@ -168,21 +173,10 @@ void BigMap::draw(Player *player, bool redraw_buffer)
       draw_buffer();
 
     // blit the visible part of buffer to the screen
-    Vector<int> p = view_pos - (buffer_view.pos * tilesize * magnification_factor);
+    Vector<int> p = view_pos - (buffer_view.pos * tilesize);
     outgoing.clear();
     outgoing = Cairo::Surface::create(buffer, Cairo::CONTENT_COLOR_ALPHA, image.get_width(), image.get_height());
-    //Glib::RefPtr<Gdk::Pixmap> outgoing;
-    if (magnification_factor != 1.0)
-      {
-	if (magnified_buffer == true)
-	  magnified_buffer.clear();
-	magnified_buffer = magnify(buffer);
-	clip_viewable_buffer(magnified_buffer, p, outgoing);
-      }
-    else
-      {
-	clip_viewable_buffer(buffer, p, outgoing);
-      }
+    clip_viewable_buffer(buffer, p, outgoing);
 
     if (blank_screen)
       {
@@ -202,23 +196,17 @@ void BigMap::screen_size_changed(Gtk::Allocation box)
 
     Rectangle new_view = view;
     
-    new_view.w = box.get_width() / (ts * magnification_factor) + 0;
-    new_view.h = box.get_height() / (ts * magnification_factor) + 0;
+    new_view.w = box.get_width() / ts;
+    new_view.h = box.get_height() / ts;
 
     if (new_view.w <= GameMap::getWidth() && new_view.h <= GameMap::getHeight()
 	&& new_view.w >= 0 && new_view.h >= 0)
       {
 	new_view.pos = clip(Vector<int>(0,0), new_view.pos,
 			    GameMap::get_dim() - new_view.dim);
-
-	if (image.get_width() != box.get_width() ||
-	    image.get_height() != box.get_height() ||
-	    new_view != view)
-	  {
-	    image = box;
-	    set_view(new_view);
-	    view_changed.emit(view);
-	  }
+	image = box;
+	set_view(new_view);
+	view_changed.emit(view);
       }
     image = box;
 }
@@ -229,8 +217,8 @@ Vector<int> BigMap::get_view_pos_from_view()
     int ts = GameMap::getInstance()->getTileSize();
 
     // clip to make sure we don't see a black border at the bottom and right
-    return clip(Vector<int>(0, 0), view.pos * ts * magnification_factor,
-		GameMap::get_dim() * ts * magnification_factor - screen_dim);
+    return clip(Vector<int>(0, 0), view.pos * ts,
+		GameMap::get_dim() * ts - screen_dim);
 }
 
 Vector<int> BigMap::tile_to_buffer_pos(Vector<int> tile)
@@ -242,13 +230,13 @@ Vector<int> BigMap::tile_to_buffer_pos(Vector<int> tile)
 Vector<int> BigMap::mouse_pos_to_tile(Vector<int> pos)
 {
     int ts = GameMap::getInstance()->getTileSize();
-    return (view_pos + pos) / (ts * magnification_factor);
+    return (view_pos + pos) / ts;
 }
 
 Vector<int> BigMap::mouse_pos_to_tile_offset(Vector<int> pos)
 {
     int ts = GameMap::getInstance()->getTileSize();
-    return (view_pos + pos) % (int)rint(ts * magnification_factor);
+    return (view_pos + pos) % ts;
 }
 
 MapTipPosition BigMap::map_tip_position(Vector<int> tile)
@@ -261,8 +249,8 @@ MapTipPosition BigMap::map_tip_position(Rectangle tile_area)
     // convert area to pixels on the screen
     int tilesize = GameMap::getInstance()->getTileSize();
 
-    Rectangle area(tile_area.pos * tilesize * magnification_factor - view_pos,
-		   tile_area.dim * tilesize * magnification_factor);
+    Rectangle area(tile_area.pos * tilesize - view_pos,
+		   tile_area.dim * tilesize);
 
     // calculate screen edge distances
     int left, right, top, bottom;
@@ -316,10 +304,9 @@ void BigMap::draw_stack(Stack *s, Cairo::RefPtr<Cairo::Surface> surface)
 {
   //this routine is for drawing the active stack.
   //for all other stacks see ImageCache::draw_tile_pic
-  GameMap *gm = GameMap::getInstance();
-  ImageCache *gc = ImageCache::getInstance();
   Vector<int> p = s->getPos();
   Player *player = s->getOwner();
+  int tilesize = GameMap::getInstance()->getTileSize();
 
   // check if the object lies in the viewed part of the map
   // otherwise we shouldn't draw it
@@ -339,7 +326,10 @@ void BigMap::draw_stack(Stack *s, Cairo::RefPtr<Cairo::Surface> surface)
       //we don't show the army or the flag if we're in fortified tent.
       if (s->hasShip())
 	{
-	  gc->getShipPic(player)->blit(surface, p);
+          PixMask *ship = ImageCache::getInstance()->getShipPic(player)->copy();
+          ship->scale (ship, tilesize, tilesize);
+          ship->blit(surface, p);
+          delete ship;
 	}
       else
 	{
@@ -349,11 +339,16 @@ void BigMap::draw_stack(Stack *s, Cairo::RefPtr<Cairo::Surface> surface)
 	      if (player->getStacklist()->getActivestack() != s &&
 		  player == Playerlist::getActiveplayer())
 		show_army = false;
-	      Maptile *tile = gm->getTile(s->getPos());
+	      Maptile *tile = GameMap::getInstance()->getTile(s->getPos());
 	      if (tile->getBuilding() != Maptile::CITY &&
 		  tile->getBuilding() != Maptile::RUIN &&
 		  tile->getBuilding() != Maptile::TEMPLE)
-		gc->getTowerPic(player)->blit(surface, p);
+                {
+                  PixMask *tower = ImageCache::getInstance()->getTowerPic(player)->copy();
+                  tower->scale (tower, tilesize, tilesize);
+                  tower->blit(surface, p);
+                  delete tower;
+                }
 	      else
 		show_army = true;
 	    }
@@ -361,7 +356,10 @@ void BigMap::draw_stack(Stack *s, Cairo::RefPtr<Cairo::Surface> surface)
 	  if (show_army == true)
 	    {
 	      Army *a = *s->begin();
-	      gc->getArmyPic(a)->blit(surface, p);
+	      PixMask *armypic = ImageCache::getInstance()->getArmyPic(a)->copy();
+              armypic->scale (armypic, tilesize, tilesize);
+              armypic->blit(surface, p);
+              delete armypic;
 	    }
 	}
 
@@ -380,7 +378,13 @@ void BigMap::draw_stack(Stack *s, Cairo::RefPtr<Cairo::Surface> surface)
           if (stacksize > MAX_STACK_SIZE)
             stacksize = MAX_STACK_SIZE;
           if (stacksize > 0)
-            gc->getFlagPic(stacksize, player)->blit(surface, p);
+            {
+              PixMask *flag = ImageCache::getInstance()->getFlagPic(stacksize, player)->copy();
+              flag->scale (flag, tilesize, tilesize);
+              flag->blit(surface, p);
+              delete flag;
+
+            }
         }
     }
 }
@@ -388,27 +392,25 @@ void BigMap::draw_stack(Stack *s, Cairo::RefPtr<Cairo::Surface> surface)
 void BigMap::draw_buffer()
 {
   draw_buffer (buffer_view, buffer);
-    
-  //the idea here is that we want to show what happens when an AI-owned
-  //stack moves through our area.  in lieu of that, we just block everything
-  //if we're a computer player.
-  if (Playerlist::getViewingplayer()->getType() != Player::HUMAN &&
-      GameScenarioOptions::s_hidden_map == true)
+  // if we're hidden map this is hosed.
+  after_draw();
+  if (blank_screen == false)
     {
-      int width = image.get_width();
-      int height = image.get_height();
-      buffer_gc->set_source_rgba(FOG_COLOUR.get_red(), FOG_COLOUR.get_green(), FOG_COLOUR.get_blue(), FOG_COLOUR.get_alpha());
-      buffer_gc->rectangle(0, 0, width, height);
-      buffer_gc->fill();
+      ImageCache *gc = ImageCache::getInstance();
+      int tilesize = GameMap::getInstance()->getTileSize();
+      if (d_fighting.getPos() != Vector<int>(-1,-1))
+        {
+          Vector<int> p = tile_to_buffer_pos(d_fighting.getPos());
+          PixMask *tmp = gc->getExplosionPic()->copy();
+          if (d_fighting.getSize() > 1)
+            {
+              PixMask::scale(tmp, d_fighting.getSize() * tilesize,
+                             d_fighting.getSize() * tilesize);
+            }
+          tmp->blit(buffer, p);
+          delete tmp;
+        }
     }
-  else
-    after_draw();
-
-}
-
-bool BigMap::saveUnderlyingMapAsBitmap(Glib::ustring filename)
-{
-  return d_renderer->saveAsBitmap(filename);
 }
 
 bool BigMap::saveAsBitmap(Glib::ustring filename)
@@ -433,10 +435,13 @@ void BigMap::draw_buffer_tile(Vector<int> tile, Cairo::RefPtr<Cairo::Surface> su
   guint32 tilesize = GameMap::getInstance()->getTileSize();
   Player *viewing = Playerlist::getViewingplayer();
   ImageCache *gc = ImageCache::getInstance();
-  GameMap *gm = GameMap::getInstance();
-  int tile_style_id = gm->getTile(tile)->getTileStyle()->getId();
+  int tile_style_id = GameMap::getInstance()->getTile(tile)->getTileStyle()->getId();
   int fog_type_id = 0;
-  fog_type_id = viewing->getFogMap()->getShadeTile(tile);
+  if (Playerlist::getViewingplayer()->getType() != Player::HUMAN &&
+      GameScenarioOptions::s_hidden_map == true)
+    fog_type_id = FogMap::ALL;
+  else
+    fog_type_id = viewing->getFogMap()->getShadeTile(tile);
 
   bool has_bag = false;
   bool has_standard = false;
@@ -446,10 +451,11 @@ void BigMap::draw_buffer_tile(Vector<int> tile, Cairo::RefPtr<Cairo::Surface> su
   int army_type_id = -1;
   bool has_ship = false;
   bool has_tower = false;
-  Maptile::Building building_type = gm->getTile(tile)->getBuilding();
+  auto building_type = GameMap::getInstance()->getTile(tile)->getBuilding();
   Vector<int> building_tile = Vector<int>(-1,-1);
   int building_subtype = -1;
   int building_player_id = -1;
+  int stone_type = -1;
 
   if (fog_type_id == FogMap::ALL)
     {
@@ -459,7 +465,7 @@ void BigMap::draw_buffer_tile(Vector<int> tile, Cairo::RefPtr<Cairo::Surface> su
 		       player_standard_id, stack_size, stack_player_id, 
 		       army_type_id, has_tower, has_ship, building_type, 
 		       building_subtype, building_tile, building_player_id, 
-		       tilesize, d_grid_toggled);
+		       tilesize, d_grid_toggled, stone_type);
       pixmask->blit(surface, tile_to_buffer_pos(tile));
       return;
     }
@@ -490,7 +496,7 @@ void BigMap::draw_buffer_tile(Vector<int> tile, Cairo::RefPtr<Cairo::Surface> su
 	  if (Playerlist::getActiveplayer()->getActivestack() != stack)
 	    {
 	      stack_player_id = stack->getOwner()->getId();
-	      Maptile *m = gm->getTile(tile);
+	      Maptile *m = GameMap::getInstance()->getTile(tile);
 	      if (stack->getFortified() == true &&
 		  m->getBuilding() != Maptile::CITY &&
 		  m->getBuilding() != Maptile::RUIN &&
@@ -546,6 +552,11 @@ void BigMap::draw_buffer_tile(Vector<int> tile, Cairo::RefPtr<Cairo::Surface> su
 		  building_tile = tile - ruin->getPos();
 		  building_subtype = ruin->getType();
 		}
+              else if (s_show_hidden_ruins)
+                {
+		  building_tile = tile - ruin->getPos();
+		  building_subtype = ruin->getType();
+                }
 	      else
 		building_type = Maptile::NONE;
 	    }
@@ -568,6 +579,17 @@ void BigMap::draw_buffer_tile(Vector<int> tile, Cairo::RefPtr<Cairo::Surface> su
 	      Road *road = GameMap::getRoad(tile);
 	      building_tile = tile - road->getPos();
 	      building_subtype = road->getType();
+              Stone *stone = GameMap::getStone(tile);
+              if (stone)
+                stone_type = stone->getType();
+	    }
+	  break;
+	case Maptile::STONE:
+	    {
+              Stone *stone = GameMap::getStone(tile);
+	      building_tile = tile - stone->getPos();
+	      building_subtype = stone->getType();
+              stone_type = stone->getType();
 	    }
 	  break;
 	case Maptile::PORT:
@@ -587,12 +609,14 @@ void BigMap::draw_buffer_tile(Vector<int> tile, Cairo::RefPtr<Cairo::Surface> su
 	  break;
 	}
     }
+  if (GameMap::getTileset()->getStonesFilename().empty() == true)
+    stone_type = -1;
   PixMask *pixmask = 
     gc->getTilePic(tile_style_id, fog_type_id, has_bag, has_standard, 
 		   player_standard_id, stack_size, stack_player_id, 
 		   army_type_id, has_tower, has_ship, building_type, 
 		   building_subtype, building_tile, building_player_id, 
-		   tilesize, d_grid_toggled);
+		   tilesize, d_grid_toggled, stone_type);
   pixmask->blit(surface, tile_to_buffer_pos(tile));
 }
 
@@ -609,29 +633,10 @@ void BigMap::draw_buffer(Rectangle map_view, Cairo::RefPtr<Cairo::Surface> surfa
   draw_buffer_tiles(map_view, surface);
 }
 
-//here we want to magnify the entire buffer, not a subset
-Cairo::RefPtr<Cairo::Surface> BigMap::magnify(Cairo::RefPtr<Cairo::Surface> orig)
-{
-  //magnify the buffer into a buffer of the correct size
-
-  int width = image.get_width();
-  int height = image.get_height();
-  if (width == 0 || height == 0)
-    return orig;
-  Cairo::RefPtr<Cairo::Surface> result = 
-    Cairo::Surface::create(orig, Cairo::CONTENT_COLOR_ALPHA,
-                           width * magnification_factor,
-                           height * magnification_factor);
-  Glib::RefPtr<Gdk::Pixbuf> unzoomed_buffer;
-  unzoomed_buffer = Gdk::Pixbuf::create(orig, 0, 0, width, height);
-
-  return result;
-}
-
 void BigMap::toggle_grid()
 {
   d_grid_toggled = !d_grid_toggled;
-  draw(Playerlist::getViewingplayer(), true);
+  draw(true);
 }
 
 void BigMap::blank(bool on)
@@ -645,39 +650,60 @@ bool BigMap::scroll(GdkEventScroll *event)
   if (input_locked)
     return true;
   Rectangle n = view;
-  int amt = 1;
-  int max_height = GameMap::getHeight() * GameMap::getInstance()->getTileSize();
-  int max_width = GameMap::getWidth() * GameMap::getInstance()->getTileSize();
   switch (event->direction)
     {
-    case GDK_SCROLL_UP:
-      if (n.y > amt)
-        n.y -= amt;
-      else
-        n.y = 0;
+    case GDK_SCROLL_SMOOTH:
+        {
+          double dx, dy;
+          if (gdk_event_get_scroll_deltas ((GdkEvent*)event, &dx, &dy))
+            {
+              if (event->state & GDK_SHIFT_MASK)
+                {
+                  dx*=-1;
+                  dy*=-1;
+                }
+              if (event->state & GDK_CONTROL_MASK)
+                {
+                  double swap;
+                  swap = dx;
+                  dx = dy;
+                  dy = swap;
+                }
+              deltax += dx;
+              deltay += dy;
+              if (deltax <= -1.0 || deltax >= 1.0)
+                {
+                  n.x += deltax;
+                  deltax += (int)deltax * -1;
+                }
+              if (deltay <= -1.0 || deltay >= 1.0)
+                {
+                  n.y += deltay;
+                  deltay += (int)deltay * -1;
+                }
+            }
+        }
       break;
-      break;
-    case GDK_SCROLL_DOWN:
-      if (n.y + amt < max_height)
-        n.y += amt;
-      else
-        n.y = max_height;
-      break;
-    case GDK_SCROLL_RIGHT:
-      if (n.x + amt < max_width)
-        n.x += amt;
-      else
-        n.x = max_width;
-      break;
-    case GDK_SCROLL_LEFT:
-      if (n.x > amt)
-        n.x -= amt;
-      else
-        n.x = 0;
     default:
       break;
     }
+  if (n.x < 0)
+    n.x = 0;
+  else if (n.x > GameMap::getWidth()-1)
+    n.x = GameMap::getWidth()-1;
+  if (n.y < 0)
+    n.y = 0;
+  else if (n.y > GameMap::getHeight()-1)
+    n.y = GameMap::getHeight()-1;
   set_view(n);
   view_changed.emit(view);
   return true;
 }
+
+double BigMap::get_default_zoom_scale (int screen_height)
+{
+  //aiming for 40 pixels on a 768 height
+  double target_ts = (double) screen_height * 0.05208333;
+  guint32 ts = GameMap::getInstance ()->getUnscaledTileSize ();
+  return (double)target_ts / (double) ts;
+}
