@@ -2,7 +2,8 @@
 // Copyright (C) 2003, 2004, 2005 Ulf Lorenz
 // Copyright (C) 2004, 2005 Bryan Duff
 // Copyright (C) 2004, 2005, 2006 Andrea Paternesi
-// Copyright (C) 2006, 2007, 2008, 2009, 2012, 2014, 2015 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2012, 2014, 2015, 2017,
+// 2020 Ben Asselstine
 // Copyright (C) 2007 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -20,6 +21,7 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
+#pragma once
 #ifndef BIGMAP_H
 #define BIGMAP_H
 
@@ -32,6 +34,7 @@
 #include "map-tip-position.h"
 #include "rectangle.h"
 #include "PixMask.h"
+#include "LocationBox.h"
 
 class Player;
 class Stack;
@@ -51,11 +54,15 @@ class Location;
 class BigMap: public sigc::trackable
 {
  public:
-    BigMap();
+    static bool s_show_hidden_ruins;
+
+    BigMap(bool headless);
     virtual ~BigMap();
 
     // draw everything
-    void draw(Player *player, bool redraw_buffer = true);
+    void draw(bool redraw_buffer = true);
+
+    bool get_toggled () const {return d_grid_toggled;}
 
     // view the rectangle, measured in tiles
     void set_view(Rectangle rect);
@@ -77,17 +84,25 @@ class BigMap: public sigc::trackable
     sigc::signal<void, Cairo::RefPtr<Cairo::Surface> > map_changed;
     void blank(bool on);
 
+    //the game object sets this when the active stack is fighting so we can 
+    //draw a fight graphic, or not
+    void setFighting(LocationBox ruckus) {d_fighting = ruckus;};
+
     //! Save the whole map as one big image (bmp file).
     bool saveAsBitmap(Glib::ustring filename);
 
-    //! Save the whole map, but not the game objects on top of it.
-    bool saveUnderlyingMapAsBitmap(Glib::ustring filename);
-
     void toggle_grid();
     bool scroll(GdkEventScroll *event);
     
     Cairo::RefPtr<Cairo::Surface> get_surface() const {return outgoing;}
+
+    //! Calculate the zoom factor.
+    /**
+     */
+    static double get_default_zoom_scale (int screen_height);
+
  protected:
+    bool d_headless;
     MapRenderer* d_renderer;
 
     Rectangle view;		// approximate view of screen, in tiles
@@ -96,8 +111,6 @@ class BigMap: public sigc::trackable
     Cairo::RefPtr<Cairo::Surface> buffer;	// the buffer we draw things in
     Cairo::RefPtr<Cairo::Surface> outgoing; //goes out to the gtk::image
     Cairo::RefPtr<Cairo::Context> buffer_gc;
-    Cairo::RefPtr<Cairo::Surface> magnified_buffer;	// the zoomed buffer;
-    double magnification_factor; //how much we're zoomed
     Rectangle buffer_view;	// current view of the buffer, in tiles
 
     bool input_locked;
@@ -105,6 +118,8 @@ class BigMap: public sigc::trackable
 
     bool d_grid_toggled;
     Gtk::Allocation image;
+    double deltax; //for smooth scrolling
+    double deltay;
 
     // helpers
     Vector<int> mouse_pos_to_tile(Vector<int> pos);
@@ -119,12 +134,12 @@ class BigMap: public sigc::trackable
 
  protected:
     void draw_stack(Stack *s, Cairo::RefPtr<Cairo::Surface> surface);
+    LocationBox d_fighting;
  private:
     void draw_buffer(Rectangle map_view, Cairo::RefPtr<Cairo::Surface> surface);
     void draw_buffer_tiles(Rectangle map_view, Cairo::RefPtr<Cairo::Surface> surface);
 
     void draw_buffer_tile(Vector<int> tile, Cairo::RefPtr<Cairo::Surface> surface);
-    Cairo::RefPtr<Cairo::Surface> magnify(Cairo::RefPtr<Cairo::Surface> orig);
     void clip_viewable_buffer(Cairo::RefPtr<Cairo::Surface> pixmap, Vector<int> pos, Cairo::RefPtr<Cairo::Surface> out);
 };
 
