@@ -3,7 +3,8 @@
 // Copyright (C) 2003, 2004, 2005, 2006 Andrea Paternesi
 // Copyright (C) 2004 David Sterba
 // Copyright (C) 2005 Bryan Duff
-// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2014, 2015 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2014, 2015, 2016,
+// 2020 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -22,6 +23,7 @@
 
 //This file contains the various macros used within lordsawar.
 
+#pragma once
 #ifndef DEFINITIONS_H
 #define DEFINITIONS_H
 
@@ -29,12 +31,12 @@
 #include <glibmm.h>
 #include <libintl.h>
 
-#define LORDSAWAR_SAVEGAME_VERSION "0.2.1"
+#define LORDSAWAR_SAVEGAME_VERSION "0.3.2"
 #define LORDSAWAR_TILESET_VERSION "0.2.1"
 #define LORDSAWAR_ARMYSET_VERSION "0.3.0"
 #define LORDSAWAR_CITYSET_VERSION "0.2.1"
 #define LORDSAWAR_SHIELDSET_VERSION "0.2.1"
-#define LORDSAWAR_CONFIG_VERSION "0.2.1"
+#define LORDSAWAR_CONFIG_VERSION "0.2.2"
 #define LORDSAWAR_ITEMS_VERSION "0.2.1"
 #define LORDSAWAR_RECENTLY_PLAYED_VERSION "0.2.1"
 #define LORDSAWAR_RECENTLY_EDITED_VERSION "0.2.1"
@@ -50,15 +52,17 @@
 //within the code, and sometimes you have to tweak them a little bit.
 const unsigned int TIMER_BIGMAP_SELECTOR = 150; //milliseconds
 const unsigned int TIMER_SMALLMAP_REFRESH = 8000; //microseconds
+const unsigned int TIMER_BIGMAP_EXPLOSION_DELAY = 1500000; //microseconds
 const unsigned int CITY_LEVELS = 4;
 const unsigned int MAX_PLAYERS = 8;
 const unsigned int TEMPLE_TYPES = 1;
 const unsigned int RUIN_TYPES = 3;
 const unsigned int DIPLOMACY_TYPES = 3;
 const unsigned int ROAD_TYPES = 15;
+const unsigned int STONE_TYPES = 89;
 const unsigned int FOG_TYPES = 15;
 const unsigned int BRIDGE_TYPES = 4;
-const unsigned int CURSOR_TYPES = 12;
+const unsigned int CURSOR_TYPES = 13;
 const unsigned int DEFAULT_TILESTYLE_TYPES = 18;
 const unsigned int MAX_CITIES_VECTORED_TO_ONE_CITY = 4;
 const unsigned int MAX_TURNS_FOR_VECTORING = 2;
@@ -80,17 +84,11 @@ const unsigned int MAP_SIZE_SMALL_HEIGHT = 105;
 const unsigned int MAP_SIZE_NORMAL_WIDTH = 112;
 const unsigned int MAP_SIZE_NORMAL_HEIGHT = 156;
 
-const unsigned int SMALL_PRODUCTION_SHIELD_WIDTH = 10;
-const unsigned int SMALL_PRODUCTION_SHIELD_HEIGHT = 10;
-const unsigned int MEDIUM_PRODUCTION_SHIELD_WIDTH = 11;
-const unsigned int MEDIUM_PRODUCTION_SHIELD_HEIGHT = 14;
 const unsigned int PRODUCTION_SHIELD_TYPES = 8;
-const unsigned int MOVE_BONUS_WIDTH = 32;
-const unsigned int MOVE_BONUS_HEIGHT = 20;
 const unsigned int MOVE_BONUS_TYPES = 6;
 const unsigned int MEDAL_TYPES = 3;
 const unsigned int NUM_WAYPOINTS = 2;
-const unsigned int NUM_GAME_BUTTON_IMAGES = 11;
+const unsigned int NUM_GAME_BUTTON_IMAGES = 12;
 const unsigned int NUM_ARROW_IMAGES = 8;
 
 const int MAX_GOLD_TO_CARRY_OVER_TO_NEXT_SCENARIO = 5000;
@@ -150,8 +148,10 @@ const Glib::ustring YELLOW_COLOUR = "#FCFCECEC2020";
 const Glib::ustring ORANGE_COLOUR = "#FCFCA0A00000";
 const Glib::ustring WHITE_COLOUR = "#FFFFFFFFFFFF";
 const Glib::ustring BLACK_COLOUR = "#000000000000";
-const Glib::ustring DARK_GREY_COLOUR = "#515151515151";
-const Glib::ustring LIGHT_GREY_COLOUR = "#929292929292";
+//const Glib::ustring DARK_GREY_COLOUR = "#515151515151";
+//const Glib::ustring LIGHT_GREY_COLOUR = "#929292929292";
+const Glib::ustring DARK_GREY_COLOUR = "#292929292929";
+const Glib::ustring LIGHT_GREY_COLOUR = "#393f3f393f3f";
 const Gdk::RGBA SEND_VECTORED_UNIT_LINE_COLOUR(YELLOW_COLOUR);
 const Gdk::RGBA RECEIVE_VECTORED_UNIT_LINE_COLOUR(ORANGE_COLOUR);
 const Gdk::RGBA SELECTOR_BOX_COLOUR(WHITE_COLOUR);
@@ -164,11 +164,73 @@ const Gdk::RGBA VECTORMAP_ACTIVE_BOX_COLOUR(WHITE_COLOUR);
 const Gdk::RGBA SELECTED_CITY_BOX_COLOUR(WHITE_COLOUR);
 const Gdk::RGBA BEVELED_CIRCLE_DARK(DARK_GREY_COLOUR);
 const Gdk::RGBA BEVELED_CIRCLE_LIGHT(LIGHT_GREY_COLOUR);
+const Gdk::RGBA ACTIVE_RUIN_BOX(YELLOW_COLOUR);
+
 #ifdef GDK_WINDOWING_WIN32
 const int SPEED_DELAY = 0;
+const double ZOOM_STEP = 0.1;
 #else
-const int SPEED_DELAY = 3000;
+const int SPEED_DELAY = 300000;
+const double ZOOM_STEP = 0.1;
 #endif
 
+const Glib::ustring DEFAULT_CONFIG_FILENAME = "lordsawarrc";
+
+//1 in x chance of standing stone being on a road tile for random map.
+const unsigned int ROAD_STONE_CHANCE = 150;
+
+//how tall an army icon appears on a dialog relative to default font height
+const double DIALOG_ARMY_PIC_FONTSIZE_MULTIPLE = 3.636363634;
+//scale for the small shield pictures that appear on dialogs
+const double DIALOG_SMALL_SHIELD_PIC_FONTSIZE_MULTIPLE = 0.727272727;
+//scale for the medium shield pictures that appear on dialogs
+const double DIALOG_MEDIUM_SHIELD_PIC_FONTSIZE_MULTIPLE = 1.872727273; //was 1.2
+//scale for the large shield pictures
+const double DIALOG_LARGE_SHIELD_PIC_FONTSIZE_MULTIPLE = 3.272727273;
+//scale for the smaller diplomacy pictures
+const double DIALOG_DIPLOMACY_TYPE_0_PIC_FONTSIZE_MULTIPLE = 3.0;
+//scale for the larger diplomacy pictures
+const double DIALOG_DIPLOMACY_TYPE_1_PIC_FONTSIZE_MULTIPLE = 5.0;
+//scale for the mouse cursor graphics
+const double DIALOG_CURSOR_PIC_FONTSIZE_MULTIPLE = 2.25;
+//scale for the new hero level picture
+const double DIALOG_NEW_LEVEL_PIC_FONTSIZE_MULTIPLE = 23;
+//scale for the tartan progress images
+const double DIALOG_TARTAN_PIC_FONTSIZE_MULTIPLE = 3.73;
+//scale for the movement bonus pictures
+const double DIALOG_MOVE_BONUS_PIC_FONTSIZE_MULTIPLE = 2;
+//scale for the large medal awarded picture
+const double DIALOG_MEDAL_PIC_FONTSIZE_MULTIPLE = 19;
+//scale for the cities/treasury/income/upkeep pictures on the main game screen
+const double DIALOG_STATUS_PIC_FONTSIZE_MULTIPLE = 2.333;
+//scale for the tiny defense icon that appears on a city info tip
+const double DIALOG_DEFENSE_PIC_FONTSIZE_MULTIPLE = 1.333;
+//scale for the images in the buttons on the main game screen
+const double DIALOG_GAME_BUTTON_PIC_FONTSIZE_MULTIPLE = 3;
+//scale for the new turn (ship) picture
+const double DIALOG_NEXT_TURN_PIC_FONT_SIZE_MULTIPLE = 23;
+//scale for the male/female hero picture
+const double DIALOG_NEW_HERO_PIC_FONT_SIZE_MULTIPLE = 23;
+//scale for city defeated picture
+const double DIALOG_CONQUERED_CITY_PIC_FONT_SIZE_MULTIPLE = 23;
+//scale for the game winning cheese picture
+const double DIALOG_WINNING_PIC_FONT_SIZE_MULTIPLE = 41;
+//scale for the ruin success/defeated picture
+const double DIALOG_RUIN_PIC_FONT_SIZE_MULTIPLE = 23;
+//scale for the parley accepted/refuse picture
+const double DIALOG_PARLEY_PIC_FONT_SIZE_MULTIPLE = 41;
+//scale for the underline beneath the shields in the turn indicator
+const double TURN_INDICATOR_FONT_SIZE_MULTIPLE = 0.3;
+//scale for tile graphics appearing on dialogs in the scenario builder
+const double EDITOR_DIALOG_TILE_PIC_FONTSIZE_MULTIPLE = 5.3;
+//scale for the warlord commentator picture
+const double DIALOG_COMMENTATOR_PIC_FONT_SIZE_MULTIPLE = 41;
+
+//do dialog pics too
+
+//for mingw:
+#ifndef M_PI
+ # define M_PI 3.14159265358979323846 /* pi */
+#endif
 #endif // DEFINITIONS_H
 
