@@ -1,4 +1,4 @@
-// Copyright (C) 2010, 2014 Ben Asselstine
+// Copyright (C) 2010, 2014, 2017, 2020 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "cityset.h"
 #include "RoadPathCalculator.h"
 #include "path.h"
+#include "font-size.h"
 
 #include "editablesmallmap.h"
 
@@ -126,7 +127,7 @@ void EditableSmallMap::change_map(Vector<int> tile)
       break;
     }
   if (redraw)
-    draw(Playerlist::getViewingplayer());
+    draw();
   return;
 }
 
@@ -155,7 +156,7 @@ void EditableSmallMap::set_pointer(Pointer p, int size, Tile::Type t)
   pointer_size = size;
 
   if (redraw)
-    draw(Playerlist::getViewingplayer());
+    draw();
 }
 
 Glib::RefPtr<Gdk::Pixbuf> EditableSmallMap::get_cursor(Vector<int> & hotspot) const
@@ -164,39 +165,51 @@ Glib::RefPtr<Gdk::Pixbuf> EditableSmallMap::get_cursor(Vector<int> & hotspot) co
   switch (pointer)
     {
     case POINTER:
-      cursor = ImageCache::getInstance()->getCursorPic(ImageCache::POINTER)->to_pixbuf();
+      cursor = ImageCache::getInstance()->getCursorPic
+        (ImageCache::POINTER,
+         FontSize::getInstance ()->get_height ())->to_pixbuf();
       hotspot = Vector<int>(cursor->get_width() / 2, cursor->get_height() / 2);
       break;
     case PICK_NEW_ROAD_START: 
     case PICK_NEW_ROAD_FINISH: 
-      cursor = ImageCache::getInstance()->getCursorPic(ImageCache::TARGET)->to_pixbuf();
+      cursor = ImageCache::getInstance()->getCursorPic
+        (ImageCache::TARGET,
+         FontSize::getInstance ()->get_height ())->to_pixbuf();
       hotspot = Vector<int>(cursor->get_width() / 2, cursor->get_height() / 2);
       break;
     case ERASE: 
-      cursor = ImageCache::getInstance()->getCursorPic(ImageCache::TARGET)->to_pixbuf();
+      cursor = ImageCache::getInstance()->getCursorPic
+        (ImageCache::TARGET,
+         FontSize::getInstance ()->get_height ())->to_pixbuf();
       hotspot = Vector<int>(cursor->get_width() / 2, cursor->get_height() / 2);
       break;
     case TERRAIN:
         {
-          cursor = ImageCache::getInstance()->getCursorPic(ImageCache::TARGET)->to_pixbuf();
+          cursor = ImageCache::getInstance()->getCursorPic
+            (ImageCache::TARGET,
+             FontSize::getInstance ()->get_height ())->to_pixbuf();
           hotspot = 
             Vector<int>(cursor->get_width() / 2, cursor->get_height() / 2);
         }
       break;
     case CITY:
-      cursor = ImageCache::getInstance()->getShieldPic(0, Playerlist::getInstance()->getNeutral())->to_pixbuf();
+      cursor = ImageCache::getInstance()->getShieldPic(0, Playerlist::getInstance()->getNeutral(), true, 0)->to_pixbuf();
       hotspot = Vector<int>(cursor->get_width() / 2, cursor->get_height() / 2);
       break;
     case RUIN:
         {
-          cursor = ImageCache::getInstance()->getCursorPic(ImageCache::TARGET)->to_pixbuf();
+          cursor = ImageCache::getInstance()->getCursorPic
+            (ImageCache::TARGET,
+             FontSize::getInstance ()->get_height ())->to_pixbuf();
           hotspot = 
             Vector<int>(cursor->get_width() / 2, cursor->get_height() / 2);
         }
       break;
     case TEMPLE: 
         {
-          cursor = ImageCache::getInstance()->getCursorPic(ImageCache::TARGET)->to_pixbuf();
+          cursor = ImageCache::getInstance()->getCursorPic
+            (ImageCache::TARGET,
+             FontSize::getInstance ()->get_height ())->to_pixbuf();
           hotspot = 
             Vector<int>(cursor->get_width() / 2, cursor->get_height() / 2);
         }
@@ -270,7 +283,7 @@ bool EditableSmallMap::create_road()
     }
   Rectangle r = Rectangle(0,0,GameMap::getWidth(), GameMap::getHeight());
   redraw_tiles(r);
-  draw(Playerlist::getViewingplayer());
+  draw();
   map_edited.emit();
   return success;
 }
@@ -279,6 +292,6 @@ void EditableSmallMap::clear_road()
 {
   road_start = Vector<int>(-1,-1);
   road_finish = Vector<int>(-1,-1);
-  draw(Playerlist::getViewingplayer());
+  draw();
   check_road();
 }
