@@ -15,6 +15,7 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
+#pragma once
 #ifndef EDITABLESMALLMAP_H
 #define EDITABLESMALLMAP_H
 
@@ -47,7 +48,7 @@ public:
     EditableSmallMap();
 
     //! Destructor.
-    ~EditableSmallMap() {};
+    ~EditableSmallMap() {}
 
 
     // Get Methods
