@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2010, 2014, 2015 Ben Asselstine
+//  Copyright (C) 2007-2010, 2014, 2015, 2017, 2020 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -40,6 +40,8 @@
 #include "port.h"
 #include "roadlist.h"
 #include "road.h"
+#include "stonelist.h"
+#include "stone.h"
 #include "playerlist.h"
 #include "defs.h"
 #include "File.h"
@@ -54,21 +56,23 @@
 #include "backpack-editor-dialog.h"
 #include "citysetlist.h"
 #include "cityset.h"
-
+#include "tileset.h"
+#include "font-size.h"
 
 EditorBigMap::EditorBigMap()
+ : BigMap(false)
 {
-    mouse_pos = Vector<int>(-1, -1);
-    prev_mouse_pos = Vector<int>(0, 0);
+  mouse_pos = Vector<int>(-1, -1);
+  prev_mouse_pos = Vector<int>(0, 0);
 
-    moving_objects_from = Vector<int>(-1,-1);
-    mouse_state = NONE;
-    input_locked = false;
-    pointer = POINTER;
-    pointer_size = 1;
-    pointer_terrain = Tile::GRASS;
-    pointer_tile_style_id = -1;
-    show_tile_types_instead_of_tile_styles = false;
+  moving_objects_from = Vector<int>(-1,-1);
+  mouse_state = NONE;
+  input_locked = false;
+  pointer = POINTER;
+  pointer_size = 1;
+  pointer_terrain = Tile::GRASS;
+  pointer_tile_style_id = -1;
+  moving_bag = NULL;
 }
 
 void EditorBigMap::set_pointer(Pointer p, int size, Tile::Type t, 
@@ -85,7 +89,7 @@ void EditorBigMap::set_pointer(Pointer p, int size, Tile::Type t,
 
     moving_objects_from = Vector<int>(-1,-1);
     if (redraw)
-      draw(Playerlist::getViewingplayer());
+      draw();
     
 }
 
@@ -110,6 +114,7 @@ void EditorBigMap::mouse_button_event(MouseButtonEvent e)
   else if (e.button == MouseButtonEvent::RIGHT_BUTTON
            && e.state == MouseButtonEvent::PRESSED)
     bring_up_details();
+  return;
 }
 
 void EditorBigMap::mouse_motion_event(MouseMotionEvent e)
@@ -164,7 +169,7 @@ void EditorBigMap::mouse_motion_event(MouseMotionEvent e)
 	    redraw_buffer = true;
 	}
 
-	draw(Playerlist::getViewingplayer(),redraw_buffer);
+	draw(redraw_buffer);
 	redraw = false;
 	mouse_state = DRAGGING;
     }
@@ -174,7 +179,7 @@ void EditorBigMap::mouse_motion_event(MouseMotionEvent e)
       mouse_state = MOVE_DRAGGING;
 
     if (redraw && pointer != POINTER)
-	draw(Playerlist::getViewingplayer());
+	draw();
     
     prev_mouse_pos = mouse_pos;
 }
@@ -183,7 +188,7 @@ void EditorBigMap::mouse_leave_event()
 {
     mouse_pos.x = mouse_pos.y = -10000;
     mouse_on_tile.emit(Vector<int>(-100, -100));
-    draw(Playerlist::getViewingplayer());
+    draw();
 }
 
 std::vector<Vector<int> > EditorBigMap::get_screen_tiles()
@@ -311,7 +316,11 @@ void EditorBigMap::change_map_under_cursor()
           if (GameMap::getInstance()->getBuilding(tile) != Maptile::NONE ||
               GameMap::getStack(tile) != NULL ||
               GameMap::getBackpack(tile)->empty() == false)
-            moving_objects_from = tile;
+            {
+              moving_objects_from = tile;
+              if (GameMap::getBackpack(tile)->empty() == false)
+                moving_bag = GameMap::getBackpack (tile);
+            }
         }
       else
         {
@@ -325,12 +334,11 @@ void EditorBigMap::change_map_under_cursor()
               Stack *s = gm->getStack(from);
               if (!s)
                 s = gm->getStack(from);
-              std::list<Stack *> enemy_stacks =
-                gm->getEnemyStacks(tile, s->getOwner());
+              auto enemy_stacks = gm->getEnemyStacks(tile, s->getOwner());
               if (gm->canPutStack(s->size(), s->getOwner(), tile) == true &&
                   enemy_stacks.empty() == true)
                 {
-                  std::list<Stack*> friendly_stacks = gm->getFriendlyStacks(tile, s->getOwner());
+                  std::vector<Stack*> friendly_stacks = gm->getFriendlyStacks(tile, s->getOwner());
                   if (friendly_stacks.empty() == true)
                     gm->moveStack(s, tile);
                   else
@@ -347,21 +355,40 @@ void EditorBigMap::change_map_under_cursor()
                       //also we need to clear the active stack to have it show.
                       gm->getStack(tile)->getOwner()->setActivestack(0);
                     }
+                  changed_tiles = Rectangle (s->getPos ());
                 }
             }
           else if (gm->getBackpack(from)->empty() == false)
-            gm->moveBackpack(from, tile);
+            {
+              if (gm->canDropBag (tile))
+                {
+                  if (moving_bag->getPos () != tile)
+                    {
+                      gm->moveBackpack(moving_bag, tile);
+                      changed_tiles = Rectangle (tile);
+                    }
+                  moving_bag = NULL;
+                }
+              else
+                break;
+            }
           else if (gm->getBuilding(from) != Maptile::NONE)
             {
               guint32 s = gm->getBuildingSize(from);
               if (gm->canPutBuilding
                   (gm->getBuilding(from), s, tile, false) == true)
-                gm->moveBuilding(from, tile);
+                {
+                  gm->moveBuilding(from, tile);
+                  changed_tiles = Rectangle (tile);
+                }
               else
                 {
                   if (gm->getLocation(from)->contains(tile) ||
                       LocationBox(tile, s).contains(from))
-                    gm->moveBuilding(from, tile);
+                    {
+                      gm->moveBuilding(from, tile);
+                      changed_tiles = Rectangle (tile);
+                    }
                 }
             }
           moving_objects_from = Vector<int>(-1,-1);
@@ -370,10 +397,7 @@ void EditorBigMap::change_map_under_cursor()
     case ERASE:
       // check if there is a building or a stack there and remove it
       if (GameMap::getInstance()->eraseTile(tile))
-        {
-          changed_tiles.pos = tile;
-          changed_tiles.dim = Vector<int>(1, 1);
-        }
+        changed_tiles = Rectangle (tile);
       break;
 
     case STACK:
@@ -399,6 +423,7 @@ void EditorBigMap::change_map_under_cursor()
           GameMap::getInstance()->putStack(s);
           //if we're on a city, change the allegiance of the stack
           //and it's armies to that of the city
+          changed_tiles = Rectangle (s->getPos ());
           if (GameMap::getInstance()->getBuilding(s->getPos()) == Maptile::CITY)
             {
               City *c = GameMap::getCity(s->getPos());
@@ -425,7 +450,10 @@ void EditorBigMap::change_map_under_cursor()
             objects_selected.emit(seq);
         }
       else
-        GameMap::getInstance()->putNewCity(tile);
+        {
+          GameMap::getInstance()->putNewCity(tile);
+          changed_tiles = Rectangle (tile);
+        }
       break;
 
     case RUIN:
@@ -440,7 +468,10 @@ void EditorBigMap::change_map_under_cursor()
             objects_selected.emit(seq);
         }
       else
-        GameMap::getInstance()->putNewRuin(tile);
+        {
+          GameMap::getInstance()->putNewRuin(tile);
+          changed_tiles = Rectangle (tile);
+        }
       break;
 
     case TEMPLE:
@@ -455,7 +486,10 @@ void EditorBigMap::change_map_under_cursor()
             objects_selected.emit(seq);
         }
       else
-        GameMap::getInstance()->putNewTemple(tile);
+        {
+          GameMap::getInstance()->putNewTemple(tile);
+          changed_tiles = Rectangle (tile);
+        }
       break;
 
     case SIGNPOST:
@@ -478,6 +512,7 @@ void EditorBigMap::change_map_under_cursor()
                 break;
               Signpost *s = new Signpost(tile);
               GameMap::getInstance()->putSignpost(s);
+              changed_tiles = Rectangle (tile);
             }
           break;
         }
@@ -490,6 +525,7 @@ void EditorBigMap::change_map_under_cursor()
             break;
           Port *p = new Port(tile);
           GameMap::getInstance()->putPort(p);
+          changed_tiles = Rectangle (tile);
           break;
         }
 
@@ -500,6 +536,7 @@ void EditorBigMap::change_map_under_cursor()
               GameMap::getInstance()->removeBridge(tile);
               Bridge *b = new Bridge(tile, tile_to_bridge_type (tile));
               GameMap::getInstance()->putBridge(b);
+              changed_tiles = Rectangle (tile);
               break;
             }
           bool bridge_placeable = GameMap::getInstance()->canPutBuilding
@@ -508,23 +545,45 @@ void EditorBigMap::change_map_under_cursor()
             break;
           Bridge *b = new Bridge(tile, tile_to_bridge_type (tile));
           GameMap::getInstance()->putBridge(b);
+          changed_tiles = Rectangle (tile);
           break;
         }
 
     case ROAD:
         {
-          if (GameMap::getRoad(tile) != NULL)
-            GameMap::getInstance()->removeRoad(tile);
+          Maptile::Building bldg =
+            GameMap::getInstance()->getTile(tile)->getBuilding();
+          switch (bldg)
+            {
+            case Maptile::ROAD:
+            case Maptile::STONE:
+            case Maptile::NONE:
+                {
+                  bool had_stone = GameMap::getStone(tile) != NULL;
+                  if (GameMap::getRoad(tile) != NULL)
+                    GameMap::getInstance()->removeRoad(tile);
 
-          if (GameMap::getInstance()->getLocation(tile))
-            break;
+                  int type = CreateScenario::calculateRoadType(tile);
+                  Road *r = new Road(tile, type);
+                  GameMap::getInstance()->putRoad(r);
+                  if (had_stone)
+                    {
+                      Stone *s = new Stone (tile, Road::Type(r->getType()));
+                      GameMap::getInstance()->putStone(s);
+                    }
 
-          int type = CreateScenario::calculateRoadType(tile);
-          Road *r = new Road(tile, type);
-          GameMap::getInstance()->putRoad(r);
-
-          changed_tiles.pos -= Vector<int>(1, 1);
-          changed_tiles.dim = Vector<int>(3, 3);
+                  changed_tiles.pos -= Vector<int>(1, 1);
+                  changed_tiles.dim = Vector<int>(3, 3);
+                }
+              break;
+            case Maptile::CITY:
+            case Maptile::RUIN:
+            case Maptile::TEMPLE:
+            case Maptile::SIGNPOST:
+            case Maptile::PORT:
+            case Maptile::BRIDGE:
+              break;
+            }
           break;
         }
     case BAG:
@@ -532,12 +591,40 @@ void EditorBigMap::change_map_under_cursor()
         bag_selected.emit(tile);
       break;
 
+    case FIGHT:
+        {
+          Stack *s = GameMap::getStack(tile);
+          if (s)
+            stack_selected_for_battle_calculator.emit(s);
+        }
+      break;
+    case STONE:
+        {
+          if (GameMap::getStone(tile) != NULL)
+            {
+              map_selection_seq seq;
+              seq.push_back(GameMap::getStone(tile));
+              objects_selected.emit(seq);
+            }
+          else
+            {
+              int type = Stone::ROAD_E_AND_W_STONE_N;
+              Road *r = GameMap::getRoad(tile);
+              if (r)
+                type = Stone::getRandomType(Road::Type(r->getType()));
+              Stone *s = new Stone(tile, type);
+              GameMap::getInstance()->putStone(s);
+              changed_tiles = Rectangle (tile);
+            }
+        }
+      break;
     }
 
   if (changed_tiles.w > 0 && changed_tiles.h > 0)
     map_tiles_changed.emit(changed_tiles);
 
-  draw(Playerlist::getViewingplayer());
+  draw();
+  return ;
 }
 
 void EditorBigMap::bring_up_details()
@@ -560,6 +647,8 @@ void EditorBigMap::bring_up_details()
   MapBackpack *b = GameMap::getInstance()->getTile(tile)->getBackpack();
   if (b->empty() == false)
     seq.push_back(b);
+  if (Stone * st = GameMap::getStone(tile))
+    seq.push_back(st);
 
   if (!seq.empty())
     objects_selected.emit(seq);
@@ -569,15 +658,14 @@ void EditorBigMap::smooth_view()
 {
   GameMap::getInstance()->applyTileStyles(view.y, view.x, view.y+view.h, 
 					  view.x+view.w, true);
-  Roadlist::iterator i = Roadlist::getInstance()->begin();
-  for (; i != Roadlist::getInstance()->end(); i++)
-    (*i)->setType(CreateScenario::calculateRoadType((*i)->getPos()));
-  draw(Playerlist::getViewingplayer());
+  CreateScenario::updateRoadsBridgesAndStones ();
+  draw();
 }
 
 void EditorBigMap::display_moving_building(Vector<int> src, Vector<int> dest)
 {
   PixMask *pic = NULL;
+  double scale = GameMap::getCityset()->get_scale ();
   switch (GameMap::getInstance()->getBuilding(src))
     {
     case Maptile::CITY:
@@ -595,6 +683,10 @@ void EditorBigMap::display_moving_building(Vector<int> src, Vector<int> dest)
     case Maptile::ROAD:
       pic = ImageCache::getInstance()->getRoadPic (GameMap::getRoad (src));
       break;
+    case Maptile::STONE:
+      pic = GameMap::getTileset()->getStoneImage (GameMap::getStone (src)->getType());
+      scale = GameMap::getTileset()->get_scale ();
+      break;
     case Maptile::PORT:
       pic = ImageCache::getInstance()->getPortPic ();
       break;
@@ -605,201 +697,268 @@ void EditorBigMap::display_moving_building(Vector<int> src, Vector<int> dest)
       break;
     }
   if (pic)
-    pic->blit(buffer, dest);
+    {
+      PixMask *p = pic->copy ();
+      double new_height = p->get_unscaled_height () * scale;
+      int new_width =
+        ImageCache::calculate_width_from_adjusted_height (p, new_height);
+      PixMask::scale (p, new_width, new_height);
+      p->blit(buffer, dest);
+      delete p;
+    }
+  if (GameMap::getInstance()->getBuilding(src) == Maptile::ROAD)
+    {
+      if (GameMap::getStone(src))
+        {
+          PixMask *stone = GameMap::getTileset()->getStoneImage (GameMap::getStone (src)->getType());
+          if (stone)
+            {
+              PixMask *p = stone->copy ();
+              double new_height = p->get_unscaled_height () * scale;
+              int new_width =
+                ImageCache::calculate_width_from_adjusted_height
+                (p, new_height);
+              PixMask::scale (p, new_width, new_height);
+              p->blit(buffer, dest);
+              delete p;
+            }
+        }
+    }
+}
+
+void EditorBigMap::blit (PixMask *src, Cairo::RefPtr<Cairo::Surface> dest, Vector<int> pos, double scale)
+{
+  PixMask *p = src->copy ();
+  double new_height = p->get_unscaled_height () * scale;
+  int new_width =
+    ImageCache::calculate_width_from_adjusted_height (p, new_height);
+  PixMask::scale (p, new_width, new_height);
+  p->blit(dest, pos);
+  delete p;
 }
 
 void EditorBigMap::after_draw()
 {
-    int tilesize = GameMap::getInstance()->getTileSize();
-    std::vector<Vector<int> > tiles;
+  guint32 army_set_id = Playerlist::getActiveplayer()->getArmyset();
+  Armyset *armyset = Armysetlist::getInstance()->get(army_set_id);
 
-    if (show_tile_types_instead_of_tile_styles)
-      {
-        buffer_gc->reset_clip();
-	tiles = get_screen_tiles();
-	for (std::vector<Vector<int> >::iterator i = tiles.begin(),
-	     end = tiles.end(); i != end; ++i)
-	  {
-	    Vector<int> pos = tile_to_buffer_pos(*i);
-	    buffer_gc->set_source_rgba(GameMap::getInstance()->getTile(*i)->getColor().get_red(),
-                                       GameMap::getInstance()->getTile(*i)->getColor().get_green(),
-                                       GameMap::getInstance()->getTile(*i)->getColor().get_blue(),
-                                       GameMap::getInstance()->getTile(*i)->getColor().get_alpha());
-	    int x = pos.x;
-	    int y = pos.y;
-	    int ts = tilesize;
-            buffer_gc->rectangle(x, y, ts, ts);
-            buffer_gc->clip();
-            buffer_gc->paint();
-            buffer_gc->reset_clip();
-	  }
-      }
+  int tilesize = GameMap::getInstance()->getTileSize();
+  std::vector<Vector<int> > tiles;
 
-    if (mouse_pos == Vector<int>(-1,-1))
-      return;
-
-    // we need to draw a drawing cursor on the map
-    tiles = get_cursor_tiles();
-    // draw each tile
-	
-    Gdk::RGBA terrain_box_color = Gdk::RGBA();
-    terrain_box_color.set_rgba(200.0/255.0, 200.0/255.0, 200.0/255.0);
-    Gdk::RGBA erase_box_color = Gdk::RGBA();
-    erase_box_color.set_rgba(200.0/255.0, 50.0/255.0, 50.0/255.0);
-    Gdk::RGBA move_box_color = Gdk::RGBA();
-    move_box_color.set_rgba(50.0/255.0, 200.0/255.0, 50.0/255.0);
-    Gdk::RGBA moving_box_color = Gdk::RGBA();
-    moving_box_color.set_rgba(250.0/255.0, 250.0/255.0, 0.0/255.0);
-    for (std::vector<Vector<int> >::iterator i = tiles.begin(),
-	     end = tiles.end(); i != end; ++i)
-      {
-	Vector<int> pos = tile_to_buffer_pos(*i);
-
-	PixMask *pic;
-
-
-	switch (pointer)
-	  {
-	  case POINTER:
-	    break;
-
-	  case TERRAIN:
-	    buffer_gc->set_source_rgb(terrain_box_color.get_red(),
-                                       terrain_box_color.get_green(),
-                                       terrain_box_color.get_blue());
-            buffer_gc->move_to(pos.x+1, pos.y+1);
-            buffer_gc->rel_line_to(tilesize-2, 0);
-            buffer_gc->rel_line_to(0, tilesize-2);
-            buffer_gc->rel_line_to(-tilesize +2, 0);
-            buffer_gc->rel_line_to(0, -tilesize+2);
-            buffer_gc->set_line_width(1.0);
-            buffer_gc->stroke();
-	    break;
-
-	  case ERASE:
-	    buffer_gc->set_source_rgb(erase_box_color.get_red(),
-                                       erase_box_color.get_green(),
-                                       erase_box_color.get_blue());
-            buffer_gc->move_to(pos.x+1, pos.y+1);
-            buffer_gc->rel_line_to(tilesize-2, 0);
-            buffer_gc->rel_line_to(0, tilesize-2);
-            buffer_gc->rel_line_to(-tilesize +2, 0);
-            buffer_gc->rel_line_to(0, -tilesize+2);
-            buffer_gc->set_line_width(1.0);
-            buffer_gc->stroke();
-	    break;
-
-	  case MOVE:
-	    if (moving_objects_from != Vector<int>(-1,-1))
-              {
-                Vector<int> tile = *i;
-                buffer_gc->set_source_rgb(moving_box_color.get_red(),
-                                          moving_box_color.get_green(),
-                                          moving_box_color.get_blue());
-                GameMap *gm = GameMap::getInstance();
-                Vector<int> from = moving_objects_from;
-                if (gm->getStack(from) != NULL)
-                  {
-                    Stack *s = gm->getStack(from);
-                    if (!s)
-                      s = gm->getStack(from);
-                    std::list<Stack *> enemy_stacks = 
-                      gm->getEnemyStacks(tile, s->getOwner());
-                    if (gm->canPutStack(s->size(), s->getOwner(), tile) == true &&
-                        enemy_stacks.empty() == true)
-                      {
-                        Playerlist *plist = Playerlist::getInstance();
-                        pic = ImageCache::getInstance()->getArmyPic
-                          (plist->getActiveplayer()->getArmyset(), 0,
-                           plist->getActiveplayer(), NULL);
-                        pic->blit(buffer, pos);
-                        pic = ImageCache::getInstance()->getFlagPic
-                          (gm->countArmyUnits(s->getPos()),
-                           plist->getActiveplayer());
-                        pic->blit(buffer, pos);
-                      }
-                  }
-                else if (gm->getBackpack(from)->empty() == false)
-                  {
-                    pic = ImageCache::getInstance()->getBagPic();
-                    pic->blit(buffer, pos);
-                  }
-                else if (gm->getBuilding(from) != Maptile::NONE)
-                  {
-                    guint32 s = gm->getBuildingSize(from);
-                    bool same = false;
-                    if (gm->getLocation(from)->contains(tile) ||
-                        LocationBox(tile, s).contains(from))
-                      same = true;
-                    if (gm->canPutBuilding
-                        (gm->getBuilding(from), s, tile, false) == true ||
-                        same)
-                      display_moving_building (from, pos);
-                  }
-              }
-	    else
-              buffer_gc->set_source_rgb(move_box_color.get_red(),
-                                        move_box_color.get_green(),
-                                        move_box_color.get_blue());
-            buffer_gc->move_to(pos.x+1, pos.y+1);
-            buffer_gc->rel_line_to(tilesize-2, 0);
-            buffer_gc->rel_line_to(0, tilesize-2);
-            buffer_gc->rel_line_to(-tilesize +2, 0);
-            buffer_gc->rel_line_to(0, -tilesize+2);
-            buffer_gc->set_line_width(1.0);
-            buffer_gc->stroke();
-	    break;
-
-	  case STACK:
-            pic = ImageCache::getInstance()->getArmyPic
-	       (Playerlist::getInstance()->getActiveplayer()->getArmyset(), 0,
-                Playerlist::getInstance()->getActiveplayer(), NULL);
-	    pic->blit(buffer, pos);
-	    break;
-
-	  case CITY:
-	    pic = ImageCache::getInstance()->getCityPic(0, Playerlist::getInstance()->getActiveplayer(), GameMap::getInstance()->getCitysetId());
-	    pic->blit(buffer, pos);
-	    break;
-
-	  case RUIN:
-	    pic = ImageCache::getInstance()->getRuinPic(0, GameMap::getInstance()->getCitysetId());
-	    pic->blit(buffer, pos);
-	    break;
-
-	  case TEMPLE:
-	    pic = ImageCache::getInstance()->getTemplePic(0, GameMap::getInstance()->getCitysetId());
-	    pic->blit(buffer, pos);
-	    break;
-
-	  case SIGNPOST:
-	    pic = ImageCache::getInstance()->getSignpostPic();
-	    pic->blit(buffer, pos);
-	    break;
-
-	  case ROAD:
-              {
-                Road *r = GameMap::getRoad(*i);
-                if (r)
-                  pic = ImageCache::getInstance()->getRoadPic(r->getType());
-                else
-                  pic = ImageCache::getInstance()->getRoadPic(CreateScenario::calculateRoadType(*i));
-                pic->blit(buffer, pos);
-              }
-	    break;
-	  case PORT:
-	    pic = ImageCache::getInstance()->getPortPic();
-	    pic->blit(buffer, pos);
-	    break;
-	  case BRIDGE:
-	    pic = ImageCache::getInstance()->getBridgePic(tile_to_bridge_type(*i));
-	    pic->blit(buffer, pos);
-	    break;
-	  case BAG:
-	    pic = ImageCache::getInstance()->getBagPic();
-	    pic->blit(buffer, pos);
-	    break;
-	  }
-      }
+  if (mouse_pos == Vector<int>(-1,-1))
     return;
+
+  // we need to draw a drawing cursor on the map
+  tiles = get_cursor_tiles();
+  // draw each tile
+
+  Gdk::RGBA terrain_box_color = Gdk::RGBA();
+  terrain_box_color.set_rgba(200.0/255.0, 200.0/255.0, 200.0/255.0);
+  Gdk::RGBA erase_box_color = Gdk::RGBA();
+  erase_box_color.set_rgba(1.0, 1.0, 1.0);
+  Gdk::RGBA move_box_color = Gdk::RGBA();
+  move_box_color.set_rgba(50.0/255.0, 200.0/255.0, 50.0/255.0);
+  Gdk::RGBA moving_box_color = Gdk::RGBA();
+  moving_box_color.set_rgba(250.0/255.0, 250.0/255.0, 0.0/255.0);
+  for (std::vector<Vector<int> >::iterator i = tiles.begin(),
+       end = tiles.end(); i != end; ++i)
+    {
+      Vector<int> pos = tile_to_buffer_pos(*i);
+
+      PixMask *pic;
+
+
+      switch (pointer)
+        {
+        case POINTER:
+          break;
+
+        case TERRAIN:
+          buffer_gc->set_source_rgb(terrain_box_color.get_red(),
+                                    terrain_box_color.get_green(),
+                                    terrain_box_color.get_blue());
+          buffer_gc->move_to(pos.x+1, pos.y+1);
+          buffer_gc->rel_line_to(tilesize-2, 0);
+          buffer_gc->rel_line_to(0, tilesize-2);
+          buffer_gc->rel_line_to(-tilesize +2, 0);
+          buffer_gc->rel_line_to(0, -tilesize+2);
+          buffer_gc->set_line_width(1.0);
+          buffer_gc->unset_dash ();
+          buffer_gc->stroke();
+          break;
+
+        case ERASE:
+          buffer_gc->set_source_rgb(erase_box_color.get_red(),
+                                    erase_box_color.get_green(),
+                                    erase_box_color.get_blue());
+          buffer_gc->move_to(pos.x+1, pos.y+1);
+          buffer_gc->rel_line_to(tilesize-2, 0);
+          buffer_gc->rel_line_to(0, tilesize-2);
+          buffer_gc->rel_line_to(-tilesize +2, 0);
+          buffer_gc->rel_line_to(0, -tilesize+2);
+          buffer_gc->set_line_width(1.0);
+            {
+              std::vector<double> dashes;
+              dashes.push_back (tilesize / 7);
+              dashes.push_back (tilesize / 7);
+              buffer_gc->set_dash (dashes, 0);
+            }
+          buffer_gc->stroke();
+          break;
+
+        case MOVE:
+              
+          buffer_gc->unset_dash ();
+          if (moving_objects_from != Vector<int>(-1,-1))
+            {
+              Vector<int> tile = *i;
+              buffer_gc->set_source_rgb(moving_box_color.get_red(),
+                                        moving_box_color.get_green(),
+                                        moving_box_color.get_blue());
+              GameMap *gm = GameMap::getInstance();
+              Vector<int> from = moving_objects_from;
+              if (gm->getStack(from) != NULL)
+                {
+                  Stack *s = gm->getStack(from);
+                  if (!s)
+                    s = gm->getStack(from);
+                  std::vector<Stack *> enemy_stacks =
+                    gm->getEnemyStacks(tile, s->getOwner());
+                  if (gm->canPutStack(s->size(), s->getOwner(), tile) == true &&
+                      enemy_stacks.empty() == true)
+                    {
+                      Playerlist *plist = Playerlist::getInstance();
+                      pic = ImageCache::getInstance()->getArmyPic
+                        (s->getOwner()->getArmyset(), 0,
+                         s->getOwner (), NULL, true, 0);
+                      blit (pic, buffer, pos, armyset->get_scale ());
+                      Player *o = plist->getActiveplayer ();
+                      plist->setActiveplayer (s->getOwner());
+                      pic = ImageCache::getInstance()->getFlagPic
+                        (gm->countArmyUnits(s->getPos()),
+                         s->getOwner ());
+                      plist->setActiveplayer (o);
+                      blit (pic, buffer, pos, gm->getTileset()->get_scale ());
+                    }
+                }
+              else if (gm->getBackpack(from)->empty() == false)
+                {
+                  pic = ImageCache::getInstance()->getBagPic();
+                  blit (pic, buffer, pos, armyset->get_scale ());
+                }
+              else if (gm->getBuilding(from) != Maptile::NONE)
+                {
+                  guint32 s = gm->getBuildingSize(from);
+                  bool same = false;
+                  if (gm->getLocation(from)->contains(tile) ||
+                      LocationBox(tile, s).contains(from))
+                    same = true;
+                  if (gm->canPutBuilding
+                      (gm->getBuilding(from), s, tile, false) == true ||
+                      same)
+                    display_moving_building (from, pos);
+                }
+            }
+          else
+            buffer_gc->set_source_rgb(move_box_color.get_red(),
+                                      move_box_color.get_green(),
+                                      move_box_color.get_blue());
+          buffer_gc->move_to(pos.x+1, pos.y+1);
+          buffer_gc->rel_line_to(tilesize-2, 0);
+          buffer_gc->rel_line_to(0, tilesize-2);
+          buffer_gc->rel_line_to(-tilesize +2, 0);
+          buffer_gc->rel_line_to(0, -tilesize+2);
+          buffer_gc->set_line_width(1.0);
+          buffer_gc->stroke();
+          break;
+
+        case STACK:
+          pic = ImageCache::getInstance()->getArmyPic
+            (Playerlist::getInstance()->getActiveplayer()->getArmyset(), 0,
+             Playerlist::getInstance()->getActiveplayer(), NULL, true, 0);
+
+          blit (pic, buffer, pos, armyset->get_scale ());
+          break;
+
+        case CITY:
+          pic = ImageCache::getInstance()->getCityPic(0, Playerlist::getInstance()->getActiveplayer(), GameMap::getInstance()->getCitysetId());
+          blit (pic, buffer, pos, GameMap::getCityset()->get_scale ());
+          break;
+
+        case RUIN:
+          pic = ImageCache::getInstance()->getRuinPic(0, GameMap::getInstance()->getCitysetId());
+          blit (pic, buffer, pos, GameMap::getCityset()->get_scale ());
+          break;
+
+        case TEMPLE:
+          pic = ImageCache::getInstance()->getTemplePic(0, GameMap::getInstance()->getCitysetId());
+          blit (pic, buffer, pos, GameMap::getCityset()->get_scale ());
+          break;
+
+        case SIGNPOST:
+          pic = ImageCache::getInstance()->getSignpostPic();
+          blit (pic, buffer, pos, GameMap::getCityset()->get_scale ());
+          break;
+
+        case ROAD:
+            {
+              Road *r = GameMap::getRoad(*i);
+              if (r)
+                pic = ImageCache::getInstance()->getRoadPic(r->getType());
+              else
+                pic = ImageCache::getInstance()->getRoadPic(CreateScenario::calculateRoadType(*i));
+              blit (pic, buffer, pos, GameMap::getTileset()->get_scale ());
+            }
+          break;
+        case STONE:
+            {
+              Tileset *t = GameMap::getTileset();
+              Stone *s = GameMap::getStone(*i);
+              if (s)
+                pic = t->getStoneImage(s->getType());
+              else
+                {
+                  Road *r = GameMap::getRoad(*i);
+                  if (r)
+                    pic = t->getStoneImage(Stone::getRandomType
+                                           (Road::Type(r->getType())));
+                  else
+                    pic = t->getStoneImage
+                      (Stone::ROAD_ALL_DIRECTIONS_STONES_NW_NE_SW_SE);
+                }
+              if (pic)
+                blit (pic, buffer, pos, GameMap::getTileset()->get_scale ());
+            }
+          break;
+        case PORT:
+          pic = ImageCache::getInstance()->getPortPic();
+          blit (pic, buffer, pos, GameMap::getCityset()->get_scale ());
+          break;
+        case BRIDGE:
+          pic = ImageCache::getInstance()->getBridgePic(tile_to_bridge_type(*i));
+          blit (pic, buffer, pos, GameMap::getCityset()->get_scale ());
+          break;
+        case BAG:
+            {
+              pic = ImageCache::getInstance()->getBagPic();
+              //Vector<int> offset = Vector<int>(tilesize,tilesize) - 
+                //Vector<int>(pic->get_width(), pic->get_height());
+              blit (pic, buffer, pos, armyset->get_scale ());
+            }
+          break;
+        case FIGHT:
+            {
+              pic =
+                ImageCache::getInstance()->getCursorPic
+                (ImageCache::SWORD, FontSize::getInstance ()->get_height ());
+              PixMask *copy = pic->copy();
+              PixMask::scale (copy, tilesize * 0.66, tilesize * 0.66);
+              copy->blit_centered(buffer, pos +
+                         Vector<int>(tilesize /2, tilesize /2));
+              delete copy;
+            }
+          break;
+        }
+    }
+  return;
 }
