@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2014 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2014, 2017 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
+#pragma once
 #ifndef EDITORBIGMAP_H
 #define EDITORBIGMAP_H
 
@@ -30,12 +31,13 @@
 #include "Tile.h"
 #include "UniquelyIdentified.h"
 
+class MapBackpack;
 //! Scenario editor.  Specializatoin of the BigMap class for the editor.
 class EditorBigMap: public BigMap
 {
  public:
     EditorBigMap();
-    ~EditorBigMap() {};
+    ~EditorBigMap() {}
 
     enum Pointer {
 	POINTER = 0, 
@@ -50,7 +52,9 @@ class EditorBigMap: public BigMap
 	MOVE, 
 	PORT, 
 	BRIDGE,
-	BAG
+	BAG,
+	FIGHT,
+        STONE
     };
     void set_pointer(Pointer pointer, int size, Tile::Type terrain, 
 		     int tile_style_id);
@@ -59,8 +63,6 @@ class EditorBigMap: public BigMap
     void mouse_motion_event(MouseMotionEvent e);
     void mouse_leave_event();
 
-    void toggleViewStylesOrTypes() { show_tile_types_instead_of_tile_styles = 
-      !show_tile_types_instead_of_tile_styles;};
     // something was selected
     typedef std::vector<UniquelyIdentified*> map_selection_seq;
     sigc::signal<void, map_selection_seq> objects_selected;
@@ -76,6 +78,7 @@ class EditorBigMap: public BigMap
 
     // emitted when the water on the map is altered.
     sigc::signal<void, Vector<int> > bag_selected;
+    sigc::signal<void, Stack*> stack_selected_for_battle_calculator;
 
     void smooth_view();
 
@@ -93,6 +96,8 @@ class EditorBigMap: public BigMap
 	NONE, DRAGGING, MOVE_DRAGGING
     } mouse_state;
 
+    MapBackpack *moving_bag;
+
     virtual void after_draw();
     int tile_to_road_type(Vector<int> tile);
     int tile_to_bridge_type(Vector<int> tile);
@@ -100,9 +105,9 @@ class EditorBigMap: public BigMap
     std::vector<Vector<int> > get_cursor_tiles();
     Rectangle get_cursor_rectangle();
     std::vector<Vector<int> > get_screen_tiles();
-    bool show_tile_types_instead_of_tile_styles;
     void bring_up_details();
     void display_moving_building(Vector<int> src, Vector<int> dest);
+    void blit (PixMask *src, Cairo::RefPtr<Cairo::Surface> dest, Vector<int> pos, double scale);
 };
 
 #endif
