@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2010, 2012, 2014, 2015 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010, 2012, 2014, 2015, 2020 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
+#pragma once
 #ifndef GUI_MAIN_WINDOW_H
 #define GUI_MAIN_WINDOW_H
 
@@ -33,9 +34,11 @@ class SmallMap;
 class GameScenario;
 class CreateScenarioRandomize;
 class City;
+class Player;
 class Signpost;
 class Ruin;
 class RoadEditorTip;
+class NewMapDialog;
 
 #include "UniquelyIdentified.h"
 
@@ -43,16 +46,20 @@ class RoadEditorTip;
 class MainWindow: public sigc::trackable
 {
  public:
+    static double minimum_zoom_scale;
+    static double maximum_zoom_scale;
+
     MainWindow(Glib::ustring load_filename = "");
     ~MainWindow();
 
     void show();
-    void hide();
+    void hide() {window->hide();}
 
     void init();
     void show_initial_map();
     Gtk::Window &get_window() { return *window; }
 
+    sigc::signal<void> editor_quit;
 
  private:
     Gtk::Window* window;
@@ -69,8 +76,6 @@ class MainWindow: public sigc::trackable
     Gtk::MenuItem *save_map_menuitem;
     Gtk::MenuItem *save_map_as_menuitem;
     Gtk::MenuItem *import_map_from_sav_menuitem;
-    Gtk::MenuItem *export_as_bitmap_menuitem;
-    Gtk::MenuItem *export_as_bitmap_no_game_objects_menuitem;
     Gtk::MenuItem *validate_menuitem;
     Gtk::MenuItem *quit_menuitem;
     Gtk::MenuItem *edit_players_menuitem;
@@ -82,12 +87,17 @@ class MainWindow: public sigc::trackable
     Gtk::MenuItem *edit_fight_order_menuitem;
     Gtk::MenuItem *toggle_tile_graphics_menuitem;
     Gtk::MenuItem *toggle_grid_menuitem;
+    Gtk::MenuItem *zoom_in_menuitem;
+    Gtk::MenuItem *zoom_out_menuitem;
+    Gtk::MenuItem *best_fit_menuitem;
     Gtk::MenuItem *smooth_map_menuitem;
     Gtk::MenuItem *smooth_screen_menuitem;
     Gtk::MenuItem *switch_sets_menuitem;
     Gtk::MenuItem *edit_items_menuitem ;
     Gtk::MenuItem *edit_rewards_menuitem;
     Gtk::MenuItem *edit_smallmap_menuitem;
+    Gtk::MenuItem *edit_remove_all_stacks_menuitem;
+    Gtk::MenuItem *edit_scenario_media_menuitem;
     Gtk::MenuItem *random_all_cities_menuitem;
     Gtk::MenuItem *random_unnamed_cities_menuitem;
     Gtk::MenuItem *random_all_ruins_menuitem;
@@ -96,13 +106,15 @@ class MainWindow: public sigc::trackable
     Gtk::MenuItem *random_unnamed_temples_menuitem;
     Gtk::MenuItem *random_all_signs_menuitem;
     Gtk::MenuItem *random_unnamed_signs_menuitem;
+    Gtk::MenuItem *random_assign_capital_cities_menuitem;
+    Gtk::MenuItem *battle_calculator_menuitem;
     Gtk::MenuItem *help_about_menuitem;
     Gtk::Viewport *terrain_tile_style_viewport;
-    Gtk::Grid *terrain_tile_style_grid;
+    Gtk::FlowBox *terrain_tile_style_grid;
     Gtk::Image *smallmap_image;
     Glib::ustring current_save_filename;
     bool needs_saving;
-    Gtk::Table *terrain_type_table;
+    Gtk::FlowBox *terrain_type_table;
     Gtk::Label *mouse_position_label;
     Gtk::RadioToolButton *pointer_radiobutton;
     Gtk::Box *players_hbox;
@@ -124,8 +136,6 @@ class MainWindow: public sigc::trackable
     void on_load_map_activated();
     void on_save_map_activated();
     void on_import_map_activated();
-    void on_export_as_bitmap_activated();
-    void on_export_as_bitmap_no_game_objects_activated();
     void on_validate_activated();
     void on_save_map_as_activated();
     void on_quit_activated();
@@ -147,10 +157,14 @@ class MainWindow: public sigc::trackable
     void on_edit_items_activated();
     void on_edit_rewards_activated();
     void on_edit_fight_order_activated();
+    void on_remove_all_stacks_activated();
+    void on_edit_scenario_media_activated();
 
     void on_fullscreen_activated();
-    void on_tile_graphics_toggled();
     void on_grid_toggled();
+    void on_zoom_in_activated ();
+    void on_zoom_out_activated ();
+    void on_best_fit_activated ();
 
     void on_random_all_cities_activated();
     void on_random_unnamed_cities_activated();
@@ -160,6 +174,7 @@ class MainWindow: public sigc::trackable
     void on_random_unnamed_temples_activated();
     void on_random_all_signs_activated();
     void on_random_unnamed_signs_activated();
+    void on_random_assign_capital_cities_activated();
 
     void on_help_about_activated();
     
@@ -167,6 +182,7 @@ class MainWindow: public sigc::trackable
     {
 	Gtk::RadioToolButton *button;
 	EditorBigMap::Pointer pointer;
+        Glib::ustring image_file;
 	int size;
     };
 
@@ -207,11 +223,13 @@ class MainWindow: public sigc::trackable
 			Glib::ustring cityset, Glib::ustring armyset);
     void set_random_map(int width, int height,
 			int grass, int water, int swamp, int forest,
-			int hills, int mountains,
-			int cities, int ruins, int temples, int signposts,
-			Glib::ustring tileset, Glib::ustring shieldset,
-			Glib::ustring cityset, Glib::ustring armyset,
-                        bool generate_roads);
+			int hills, int mountains, int cities, int ruins,
+                        int temples, int signposts, int stones,
+                        int stone_road_chance,
+                        Glib::ustring tileset, Glib::ustring shieldset,
+                        Glib::ustring cityset, Glib::ustring armyset,
+                        bool generate_roads, bool random_names,
+                        NewMapDialog *d);
 
     void clear_map_state();
     void init_map_state();
@@ -227,18 +245,20 @@ class MainWindow: public sigc::trackable
     void on_smallmap_water_changed();
     void on_objects_selected(std::vector<UniquelyIdentified *> objects);
     void on_mouse_on_tile(Vector<int> tile);
+    void on_bigmap_tiles_changed (Rectangle r);
     
     void popup_dialog_for_object(UniquelyIdentified *object);
 
     void auto_select_appropriate_pointer();
 
-    bool on_bigmap_exposed();
-    bool on_smallmap_exposed();
     void on_bigmap_surface_changed(Gtk::Allocation box);
     void on_save_as_activated(Gtk::FileChooserDialog *chooser);
     bool on_bigmap_scrolled(GdkEventScroll* event);
-    void redraw();
+    void redraw(bool center = false);
     void fill_players();
+    void on_battle_calculator_activated();
+
+    Glib::ustring getDefaultMapFilename();
 
     struct PlayerItem
     {
@@ -250,11 +270,27 @@ class MainWindow: public sigc::trackable
     void update_window_title();
     void on_bag_selected(Vector<int> pos);
     void on_road_edited(Vector<int> pos, int new_type);
+    void on_stack_selected_for_battle_calculator(Stack *s);
+    void add_attacker_to_battle_calculator(Stack *s);
+    void add_defender_to_battle_calculator(Stack *s);
+    void append_defender_to_battle_calculator(Stack *s);
+    void change_city_ownership(City *city, Player *player);
+
+    bool on_window_state_event (GdkEventWindowState *e);
+    bool on_configure_event (GdkEventConfigure *e);
+    void zoom (double scale);
+    bool close_road_editor_tip ();
+    void update_menuitems ();
+    void set_default_bigmap_zoom ();
 
     int d_width;
     int d_height;
     Glib::ustring d_load_filename;// filename given on the command line.
     RoadEditorTip *road_editor_tip;
+    std::list<Army*> battle_calculator_attackers;
+    std::list<Army*> battle_calculator_defenders;
+    Gtk::Allocation unmaximized_box;
+
     
 };
 
