@@ -1,5 +1,6 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2010, 2012, 2014, 2015 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010, 2012, 2014, 2015, 2017,
+//  2020 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -33,14 +34,16 @@
 #include "ucompose.hpp"
 #include "GameMap.h"
 
+#define method(x) sigc::mem_fun(*this, &NewMapDialog::x)
 
 NewMapDialog::NewMapDialog(Gtk::Window &parent)
  : LwEditorDialog(parent, "new-map-dialog.ui")
 {
     map_set = false;
+    fill_style_combobox = NULL;
     
+    xml->get_widget("dialog-vbox", dialog_vbox);
     xml->get_widget("map_size_combobox", map_size_combobox);
-    xml->get_widget("custom_size_table", custom_size_table);
     xml->get_widget("width_spinbutton", width_spinbutton);
     xml->get_widget("height_spinbutton", height_spinbutton);
     xml->get_widget("random_map_container", random_map_container);
@@ -54,8 +57,15 @@ NewMapDialog::NewMapDialog(Gtk::Window &parent)
     xml->get_widget("ruins_scale", ruins_scale);
     xml->get_widget("temples_scale", temples_scale);
     xml->get_widget("signposts_scale", signposts_scale);
+    xml->get_widget("stones_scale", stones_scale);
     xml->get_widget("accept_button", accept_button);
-    xml->get_widget("roads_checkbutton", roads_checkbutton);
+    xml->get_widget("random_roads_switch", random_roads_switch);
+    random_roads_switch->property_active().signal_changed().connect
+      (method(on_random_roads_toggled));
+    xml->get_widget("random_names_switch", random_names_switch);
+    xml->get_widget("num_players_spinbutton", num_players_spinbutton);
+    xml->get_widget("stone_road_spinbutton", stone_road_spinbutton);
+    xml->get_widget ("notebook", notebook);
 
     // fill in tile themes combobox
     
@@ -81,8 +91,7 @@ NewMapDialog::NewMapDialog(Gtk::Window &parent)
     tile_size_combobox->set_active(default_id);
     xml->get_widget("tile_size_box", box);
     box->pack_start(*tile_size_combobox, Gtk::PACK_SHRINK);
-    tile_size_combobox->signal_changed().connect
-      (sigc::mem_fun(*this, &NewMapDialog::on_tile_size_changed));
+    tile_size_combobox->signal_changed().connect (method(on_tile_size_changed));
 
     // make new tile themes combobox
     tile_theme_combobox = manage(new Gtk::ComboBoxText);
@@ -137,25 +146,28 @@ NewMapDialog::NewMapDialog(Gtk::Window &parent)
     xml->get_widget("fill_style_alignment", alignment);
     alignment->add(*fill_style_combobox);
 
-    fill_style_combobox->signal_changed().connect(
-						  sigc::mem_fun(*this, &NewMapDialog::on_fill_style_changed));
+    fill_style_combobox->signal_changed().connect (method(on_fill_style_changed));
     fill_style_combobox->set_active(6);
 
     // map size
     map_size_combobox->set_active(MAP_SIZE_NORMAL);
-    map_size_combobox->signal_changed().connect(
-						sigc::mem_fun(*this, &NewMapDialog::on_map_size_changed));
+    map_size_combobox->signal_changed().connect(method(on_map_size_changed));
     grass_scale->set_value(78);
     water_scale->set_value(7);
     swamp_scale->set_value(2);
     forest_scale->set_value(3);
     hills_scale->set_value(5);
     signposts_scale->set_value(20);
+    stones_scale->set_value(40);
     mountains_scale->set_value(5);
     on_map_size_changed();
-	
-    width_spinbutton->set_value(MAP_SIZE_TINY_WIDTH);
-    height_spinbutton->set_value(MAP_SIZE_TINY_HEIGHT);
+    width_spinbutton->set_value (MAP_SIZE_NORMAL_WIDTH);
+    height_spinbutton->set_value (MAP_SIZE_NORMAL_HEIGHT);
+
+    random_names_switch->set_active(true);
+    num_players_spinbutton->set_value(8);
+    stone_road_spinbutton->set_value (ROAD_STONE_CHANCE);
+    stone_road_spinbutton->set_sensitive (false);
 }
 
 void NewMapDialog::run()
@@ -218,9 +230,12 @@ void NewMapDialog::run()
 	  map.ruins = int(ruins_scale->get_value());
 	  map.temples = int(temples_scale->get_value());
 	  map.signposts = int(signposts_scale->get_value());
-          map.generate_roads = roads_checkbutton->get_active();
+	  map.stones = int(stones_scale->get_value());
+          map.generate_roads = random_roads_switch->get_active();
+          map.random_names = random_names_switch->get_active();
+          map.stone_road_chance = int(stone_road_spinbutton->get_value());
 	}
-
+      map.num_players = int(num_players_spinbutton->get_value());
       map_set = true;
     }
   else
@@ -233,38 +248,52 @@ void NewMapDialog::on_fill_style_changed()
   assert(row >= 0 && row < int(fill_style.size()));
   bool random_selected = fill_style[row] == -1;
   random_map_container->set_sensitive(random_selected);
-  roads_checkbutton->set_sensitive(random_selected);
+  random_roads_switch->set_sensitive(random_selected);
+  random_names_switch->set_sensitive(random_selected);
+  update_button ();
 }
 
 void NewMapDialog::on_map_size_changed()
 {
   switch (map_size_combobox->get_active_row_number()) {
   case MAP_SIZE_SMALL:
+    width_spinbutton->set_value (MAP_SIZE_SMALL_WIDTH);
+    height_spinbutton->set_value (MAP_SIZE_SMALL_HEIGHT);
     cities_scale->set_value(15);
     ruins_scale->set_value(20);
     temples_scale->set_value(20);
-    custom_size_table->set_sensitive(false);
+    width_spinbutton->set_sensitive (false);
+    height_spinbutton->set_sensitive (false);
     break;
 
   case MAP_SIZE_TINY:
+    width_spinbutton->set_value (MAP_SIZE_TINY_WIDTH);
+    height_spinbutton->set_value (MAP_SIZE_TINY_HEIGHT);
     cities_scale->set_value(10);
     ruins_scale->set_value(15);
     temples_scale->set_value(15);
-    custom_size_table->set_sensitive(false);
+    width_spinbutton->set_sensitive (false);
+    height_spinbutton->set_sensitive (false);
     break;
 
   case MAP_SIZE_NORMAL:
   default:
+    width_spinbutton->set_value (MAP_SIZE_NORMAL_WIDTH);
+    height_spinbutton->set_value (MAP_SIZE_NORMAL_HEIGHT);
     cities_scale->set_value(20);
     ruins_scale->set_value(25);
     temples_scale->set_value(25);
-    custom_size_table->set_sensitive(false);
+    width_spinbutton->set_sensitive (false);
+    height_spinbutton->set_sensitive (false);
     break;
   case MAP_SIZE_CUSTOM:
+    width_spinbutton->set_value (MAP_SIZE_NORMAL_WIDTH);
+    height_spinbutton->set_value (MAP_SIZE_NORMAL_HEIGHT);
     cities_scale->set_value(20);
     ruins_scale->set_value(25);
     temples_scale->set_value(25);
-    custom_size_table->set_sensitive(true);
+    width_spinbutton->set_sensitive (true);
+    height_spinbutton->set_sensitive (true);
     break;
   }
 }
@@ -300,8 +329,6 @@ void NewMapDialog::on_tile_size_changed()
     }
 
   tile_theme_combobox->set_active(default_id);
-  if (tile_theme_combobox->get_children().size() == 0)
-    accept_button->set_sensitive(false);
 
   army_theme_combobox->remove_all();
   Armysetlist *al = Armysetlist::getInstance();
@@ -318,8 +345,6 @@ void NewMapDialog::on_tile_size_changed()
     }
 
   army_theme_combobox->set_active(default_id);
-  if (army_theme_combobox->get_children().size() == 0)
-    accept_button->set_sensitive(false);
 
   city_theme_combobox->remove_all();
   Citysetlist *cl = Citysetlist::getInstance();
@@ -336,7 +361,86 @@ void NewMapDialog::on_tile_size_changed()
     }
 
   city_theme_combobox->set_active(default_id);
-  if (city_theme_combobox->get_children().size() == 0)
-    accept_button->set_sensitive(false);
+  update_button ();
 }
 
+void NewMapDialog::update_button ()
+{
+  bool sens = true;
+  if (city_theme_combobox->get_model()->children().size() == 0 ||
+      army_theme_combobox->get_model()->children().size() == 0 ||
+      tile_theme_combobox->get_model()->children().size() == 0)
+    sens = false;
+
+  if (fill_style_combobox)
+    {
+      int row = fill_style_combobox->get_active_row_number();
+      assert(row >= 0 && row < int(fill_style.size()));
+      bool random_selected = fill_style[row] == -1;
+      if (random_selected)
+        accept_button->set_label (_("Create Random Map"));
+      else
+        {
+          switch (Tile::Type (fill_style[row]))
+            {
+            case Tile::GRASS:
+              accept_button->set_label (_("Create Grass Map"));
+              break;
+            case Tile::WATER:
+              accept_button->set_label (_("Create Water Map"));
+              break;
+            case Tile::FOREST:
+              accept_button->set_label (_("Create Forest Map"));
+              break;
+            case Tile::HILLS:
+              accept_button->set_label (_("Create Hills Map"));
+              break;
+            case Tile::MOUNTAIN:
+              accept_button->set_label (_("Create Mountains Map"));
+              break;
+            case Tile::SWAMP:
+              accept_button->set_label (_("Create Swamp Map"));
+              break;
+            }
+        }
+    }
+  accept_button->set_sensitive(sens);
+}
+
+void NewMapDialog::on_random_roads_toggled ()
+{
+  stone_road_spinbutton->set_sensitive (random_roads_switch->get_active ());
+}
+
+void NewMapDialog::setup_progress_bar ()
+{
+  progress_treeview = Gtk::manage (new Gtk::TreeView ());
+  progress_treeview->property_headers_visible () = false;
+  progress_liststore = Gtk::ListStore::create(progress_columns);
+  progress_treeview->set_model (progress_liststore);
+  progress_row = *(progress_liststore->append());
+  auto cell = Gtk::make_managed<Gtk::CellRendererProgress>();
+  cell->property_text () = "";
+  int cols_count = progress_treeview->append_column ("progress", *cell);
+  auto pColumn = progress_treeview->get_column(cols_count -1);
+  if (pColumn)
+    pColumn->add_attribute(cell->property_value (), progress_columns.perc);
+
+  dialog_vbox->pack_end (*progress_treeview, true, true);
+  dialog_vbox->show_all ();
+  while (g_main_context_iteration(NULL, FALSE)); //doEvents
+  dialog_vbox->set_sensitive(false);
+}
+
+void NewMapDialog::tick_progress (double p)
+{
+  if (!progress_treeview)
+    return;
+  progress_row[progress_columns.perc] = p * 100.0;
+  while (g_main_context_iteration(NULL, FALSE)); //doEvents
+}
+
+NewMapDialog::~NewMapDialog()
+{
+  notebook->property_show_tabs () = false;
+}
