@@ -15,6 +15,7 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
+#pragma once
 #ifndef TILESET_SMALLMAP_BUILDING_COLORS_DIALOG_H
 #define TILESET_SMALLMAP_BUILDING_COLORS_DIALOG_H
 
@@ -26,7 +27,7 @@ class TilesetSmallmapBuildingColorsDialog: public LwEditorDialog
 {
  public:
     TilesetSmallmapBuildingColorsDialog(Gtk::Window &parent, Tileset *tileset);
-    ~TilesetSmallmapBuildingColorsDialog() {};
+    ~TilesetSmallmapBuildingColorsDialog() {}
 
  private:
     Tileset *d_tileset;
