@@ -1,4 +1,4 @@
-// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2014, 2015 Ben Asselstine
+// Copyright (C) 2006-2010, 2014, 2015, 2016, 2017, 2020 Ben Asselstine
 // Copyright (C) 2007, 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -71,6 +71,8 @@
 #include "select-city-map.h"
 #include "Item.h"
 #include "rnd.h"
+#include "gui/main.h"
+#include "game-server.h"
 
 Game *Game::current_game = 0;
 
@@ -78,12 +80,8 @@ Game *Game::current_game = 0;
 #define debug(x)
 void Game::addPlayer(Player *p)
 {
-
-  //disconnect prior players' connections
-  for (std::list<sigc::connection>::iterator it = 
-       connections[p->getId()].begin(); 
-       it != connections[p->getId()].end(); it++) 
-    (*it).disconnect();
+  for (auto it: connections[p->getId()])
+    it.disconnect();
   connections[p->getId()].clear();
 
   //now setup the connections that are specific for human players
@@ -158,7 +156,10 @@ void Game::addPlayer(Player *p)
       
       
   //now do all of the common connections
-      
+  connections[p->getId()].push_back
+    (p->save_game.connect(sigc::mem_fun(this, &Game::on_save_game)));
+  connections[p->getId()].push_back
+    (p->get_round.connect(sigc::mem_fun(this, &Game::on_get_round)));
   connections[p->getId()].push_back
     (p->getStacklist()->snewpos.connect
      (sigc::mem_fun(stack_moves, &sigc::signal<void, Stack*, Vector<int> >::emit)));
@@ -176,6 +177,9 @@ void Game::addPlayer(Player *p)
   connections[p->getId()].push_back
     (p->getStacklist()->soldpos.connect
      (sigc::mem_fun(this, &Game::stack_leaves_tile)));
+  connections[p->getId()].push_back
+    (p->getStacklist()->sstackDied.connect
+     (sigc::mem_fun(this, &Game::on_stack_died)));
   connections[p->getId()].push_back
     (p->aborted_turn.connect (sigc::mem_fun
 	   (game_stopped, &sigc::signal<void>::emit)));
@@ -193,6 +197,8 @@ void Game::addPlayer(Player *p)
 				      &sigc::signal<void>::emit)));
   connections[p->getId()].push_back
     (p->supdatingStack.connect (sigc::mem_fun(this, &Game::stackUpdate)));
+  connections[p->getId()].push_back
+    (p->sbagdropped.connect (sigc::mem_fun(this, &Game::on_bag_dropped)));
   connections[p->getId()].push_back
     (p->sinvadingCity.connect(sigc::mem_fun(this, &Game::invading_city)));
   connections[p->getId()].push_back
@@ -242,9 +248,8 @@ void Game::on_stack_grouped(Stack *stack)
   //tell gamebigmap that we just grouped/ungrouped a stack.
   return;
 }
-#include "game-server.h"
 
-Game::Game(GameScenario* gameScenario, NextTurn *nextTurn)
+Game::Game(GameScenario* gameScenario, NextTurn *nextTurn, bool headless)
     : d_gameScenario(gameScenario), d_nextTurn(nextTurn)
 {
     current_game = this;
@@ -252,7 +257,7 @@ Game::Game(GameScenario* gameScenario, NextTurn *nextTurn)
 
     // init the bigmap
     bigmap.reset(new GameBigMap
-		 (GameScenario::s_intense_combat, 
+		 (headless, GameScenario::s_intense_combat, 
 		  GameScenario::s_see_opponents_production, 
 		  GameScenario::s_see_opponents_stacks, 
 		  GameScenario::s_military_advisor));
@@ -278,9 +283,15 @@ Game::Game(GameScenario* gameScenario, NextTurn *nextTurn)
 	sigc::mem_fun(this, &Game::on_stack_queried));
     bigmap->stack_unqueried.connect(
 	sigc::mem_fun(this, &Game::on_stack_unqueried));
+    bigmap->path_turns.connect(
+	sigc::mem_fun(this, &Game::on_show_path_turns));
+    bigmap->popup_stack_actions_menu.connect(
+	sigc::mem_fun(popup_stack_actions_menu, &sigc::signal<void, Stack*>::emit));
+    bigmap->pointing_at_new_tile.connect
+	(sigc::mem_fun(this, &Game::on_pointing_at_new_tile));
 
     // init the smallmap
-    smallmap.reset(new SmallMap);
+    smallmap.reset(new SmallMap(headless));
     // pass map changes directly through 
     smallmap->resize();
     smallmap->map_changed.connect(
@@ -297,21 +308,18 @@ Game::Game(GameScenario* gameScenario, NextTurn *nextTurn)
     smallmap->view_changed.connect(
 	sigc::mem_fun(bigmap.get(), &GameBigMap::set_view));
 
-    // get the maps up and running
-    Playerlist::getInstance()->setViewingplayer(Playerlist::getActiveplayer());
     bigmap->screen_size_changed(Gtk::Allocation(0,0,320,200));
 
     // connect player callbacks
-    Playerlist* pl = Playerlist::getInstance();
-    for (Playerlist::iterator i = pl->begin(); i != pl->end(); ++i)
-    {
-	Player *p = *i;
-	addPlayer(p);
-    }
+    for (auto p: *Playerlist::getInstance())
+      addPlayer(p);
+
     if (gameScenario->getPlayMode() == GameScenario::HOTSEAT ||
         gameScenario->getPlayMode() == GameScenario::NETWORKED)
-      pl->splayerDead.connect(sigc::mem_fun(this, &Game::on_player_died));
-    pl->ssurrender.connect(sigc::mem_fun(this, &Game::on_surrender_offered));
+      Playerlist::getInstance()->splayerDead.connect
+        (sigc::mem_fun(this, &Game::on_player_died));
+    Playerlist::getInstance()->ssurrender.connect
+      (sigc::mem_fun(this, &Game::on_surrender_offered));
 
     d_nextTurn->splayerStart.connect(
 	sigc::mem_fun(this, &Game::init_turn_for_player));
@@ -333,13 +341,12 @@ Game::~Game()
 {
   for (unsigned int i = 0; i < MAX_PLAYERS + 1; i++)
     {
-      for (std::list<sigc::connection>::iterator it = connections[i].begin(); 
-	   it != connections[i].end(); it++) 
-	(*it).disconnect();
+      for (auto it: connections[i])
+	it.disconnect();
       connections[i].clear();
     }
     delete d_gameScenario;
-    //delete d_nextTurn;
+    delete d_nextTurn;
     
     HeroTemplates::deleteInstance();
 }
@@ -402,15 +409,9 @@ void Game::update_sidebar_stats()
 void Game::redraw()
 {
     if (bigmap.get())
-      {
-	bigmap->draw(Playerlist::getViewingplayer());
-      }
+      bigmap->draw();
     if (smallmap.get())
-      {
-	//if (Playerlist::getActiveplayer()->getType() == Player::HUMAN ||
-	    //GameScenario::s_hidden_map == false)
-	smallmap->draw(Playerlist::getActiveplayer());
-      }
+      smallmap->draw();
 }
 
 void Game::select_next_movable_stack()
@@ -421,101 +422,6 @@ void Game::select_next_movable_stack()
   select_active_stack();
 }
 
-void Game::move_map_dir(int diffx, int diffy)
-{
-  smallmap->move_map_in_dir(Vector<int>(diffx, diffy));
-}
-
-void Game::move_selected_stack_dir(int diffx, int diffy)
-{
-  static bool currently_moving = false;
-  if (currently_moving == true)
-    return;
-  currently_moving = true;
-  Stack *stack = Playerlist::getActiveplayer()->getActivestack();
-  if (!stack)
-    return;
-  // Get rid of the old path if there is one
-  if (stack->getPath()->size())
-    stack->getPath()->clear();
-  //See if we can move there
-  Vector<int> dest = stack->getPos();
-  dest += Vector<int>(diffx, diffy);
-  int mp = stack->getPath()->calculate(stack, dest);
-  if (mp > 0)
-    move_selected_stack_along_path();
-  else
-    {
-      Playerlist::getActiveplayer()->setActivestack(0);
-      unselect_active_stack();
-    }
-  currently_moving = false;
-}
-
-void Game::move_selected_stack_northwest()
-{
-  if (Playerlist::getActiveplayer()->getActivestack() != NULL)
-    move_selected_stack_dir(-1, -1);
-  else
-    move_map_dir(-1,-1);
-}
-
-void Game::move_selected_stack_north()
-{
-  if (Playerlist::getActiveplayer()->getActivestack() != NULL)
-    move_selected_stack_dir(0, -1);
-  else
-    move_map_dir(0, -1);
-}
-
-void Game::move_selected_stack_northeast()
-{
-  if (Playerlist::getActiveplayer()->getActivestack() != NULL)
-    move_selected_stack_dir(1, -1);
-  else
-    move_map_dir(1, -1);
-}
-
-void Game::move_selected_stack_east()
-{
-  if (Playerlist::getActiveplayer()->getActivestack() != NULL)
-    move_selected_stack_dir(1, 0);
-  else
-    move_map_dir(1, 0);
-}
-
-void Game::move_selected_stack_west()
-{
-  if (Playerlist::getActiveplayer()->getActivestack() != NULL)
-    move_selected_stack_dir(-1, 0);
-  else
-    move_map_dir(-1, 0);
-}
-
-void Game::move_selected_stack_southwest()
-{
-  if (Playerlist::getActiveplayer()->getActivestack() != NULL)
-    move_selected_stack_dir(-1, 1);
-  else
-    move_map_dir(-1, 1);
-}
-
-void Game::move_selected_stack_south()
-{
-  if (Playerlist::getActiveplayer()->getActivestack() != NULL)
-    move_selected_stack_dir(0, 1);
-  else
-    move_map_dir(0, 1);
-}
-
-void Game::move_selected_stack_southeast()
-{
-  if (Playerlist::getActiveplayer()->getActivestack() != NULL)
-    move_selected_stack_dir(1, 1);
-  else
-    move_map_dir(1, 1);
-}
-
 void Game::move_selected_stack_along_path()
 {
   Stack *stack = Playerlist::getActiveplayer()->getActivestack();
@@ -615,30 +521,32 @@ void Game::search_stack(Stack *stack, bool &gotquest, bool &stackdied)
       ((ruin->isHidden() == true && ruin->getOwner() == player) ||
        ruin->isHidden() == false))
     {
-      Reward *reward;
-      reward = player->stackSearchRuin(stack, ruin, stackdied);
+      Reward *reward = player->stackSearchRuin(stack, ruin, stackdied);
       if (stackdied)
         return;
       if (ruin->hasSage() == true)
 	{
+          if (reward)
+            delete reward;
           Sage *sage = ruin->generateSage();
           if (player->isComputer() == false)
             reward = sage_visited.emit(ruin, sage, stack);
           else
             reward = player->chooseReward(ruin, sage, stack);
+          delete sage;
 	}
 	  
       if (reward)
 	{
           StackReflist *stacks = new StackReflist();
-	  player->giveReward(stack, reward, stacks);
+	  player->giveReward(stack, reward, stacks, false);
           delete stacks;
-	  //FIXME: delete this reward, but don't delete the item, or map
 	  redraw();
 	  update_stack_info();
 	  update_control_panel();
           if (player->isComputer() == false)
             ruin_searched.emit(ruin, stack, reward);
+          delete reward;
 	}
       else
 	{
@@ -745,9 +653,6 @@ void Game::stackUpdate(Stack* s)
   if (s)
     smallmap->center_view_on_tile(s->getPos(), true);
 
-  if (!s)
-    redraw();
-
   update_stack_info();
   update_control_panel();
 
@@ -827,7 +732,7 @@ void Game::on_ruin_queried (Ruin* r, bool brief)
 	    str += _("Unexplored");
 
 	  MapTipPosition mpos = bigmap->map_tip_position(r->getArea());
-	  map_tip_changed.emit(str, mpos);
+	  map_tip_changed.emit(str, mpos, false);
 	}
       else
 	{
@@ -835,7 +740,7 @@ void Game::on_ruin_queried (Ruin* r, bool brief)
 	}
     }
   else
-    map_tip_changed.emit("", MapTipPosition());
+    map_tip_changed.emit("", MapTipPosition(), false);
 }
 
 void Game::on_signpost_queried (Signpost* s)
@@ -847,10 +752,23 @@ void Game::on_signpost_queried (Signpost* s)
       str = s->getName();
 
       MapTipPosition mpos = bigmap->map_tip_position(s->getArea());
-      map_tip_changed.emit(str, mpos);
+      map_tip_changed.emit(str, mpos, false);
     }
   else
-    map_tip_changed.emit("", MapTipPosition());
+    map_tip_changed.emit("", MapTipPosition(), false);
+}
+
+void Game::on_show_path_turns (Vector<int> tile, guint32 turns)
+{
+  if (tile != Vector<int>(-1,-1))
+    {
+      //The number of turns is always going to be plural here.
+      Glib::ustring str = Glib::ustring::compose (_("%1 turns"), turns);
+      MapTipPosition mpos = bigmap->map_tip_position(tile);
+      map_tip_changed.emit (str, mpos, true);
+    }
+  else
+    map_tip_changed.emit("", MapTipPosition(), false);
 }
 
 void Game::on_stack_unqueried ()
@@ -874,7 +792,7 @@ void Game::on_temple_queried (Temple* t, bool brief)
 	  str = t->getName();
 
 	  MapTipPosition mpos = bigmap->map_tip_position(t->getArea());
-	  map_tip_changed.emit(str, mpos);
+	  map_tip_changed.emit(str, mpos, false);
 	}
       else
 	{
@@ -882,16 +800,15 @@ void Game::on_temple_queried (Temple* t, bool brief)
 	}
     }
   else
-    map_tip_changed.emit("", MapTipPosition());
+    map_tip_changed.emit("", MapTipPosition(), false);
 }
 
 void Game::looting_city(City* city, int &gold)
 {
-  Citylist *clist = Citylist::getInstance();
-  Playerlist *plist = Playerlist::getInstance();
-  Player *attacker = plist->getActiveplayer();
+  Player *attacker = Playerlist::getActiveplayer();
   Player *defender = city->getOwner();
-  int amt = (defender->getGold() / (2 * clist->countCities (defender)) * 2);
+  int amt = (defender->getGold() / 
+             (2 * Citylist::getInstance()->countCities (defender)) * 2);
   // give (Enemy-Gold/(2Enemy-Cities)) to the attacker 
   // and then take away twice that from the defender.
   // the idea here is that some money is taken in the invasion
@@ -999,8 +916,8 @@ void Game::update_control_panel()
 
   bool all_defending_or_parked = true;
   for (Stacklist::iterator i = sl->begin(); i != sl->end(); ++i)
-    if (!(*i)->getDefending() && !(*i)->getParked() 
-	&& *i != sl->getActivestack())
+    if (!(*i)->getDefending() && !(*i)->getParked() &&
+	*i != sl->getActivestack())
       {
 	all_defending_or_parked = false;
 	break;
@@ -1008,8 +925,8 @@ void Game::update_control_panel()
 
   bool all_immobile = true;
   for (Stacklist::iterator i = sl->begin(); i != sl->end(); ++i)
-    if (!(*i)->getDefending() && !(*i)->getParked() && (*i)->canMove()
-	&& *i != sl->getActivestack())
+    if (!(*i)->getDefending() && !(*i)->getParked() && (*i)->canMove() &&
+        *i != sl->getActivestack())
       {
 	all_immobile = false;
 	break;
@@ -1039,7 +956,7 @@ void Game::update_control_panel()
       can_plant_standard_selected_stack.emit(GameMap::can_plant_flag(stack));
 
       can_search_selected_stack.emit(GameMap::can_search(stack));
-          
+
       can_use_item.emit(player->hasUsableItem());
 
       if (GameMap::getSignpost(stack))
@@ -1059,6 +976,8 @@ void Game::update_control_panel()
       can_plant_standard_selected_stack.emit(false);
       can_search_selected_stack.emit(false);
       can_defend_selected_stack.emit(false);
+      can_change_signpost.emit(false);
+      can_use_item.emit(false);
     }
       
   if (d_gameScenario->getRound() > 1)
@@ -1152,8 +1071,6 @@ void Game::blank(bool on)
 
 void Game::init_turn_for_player(Player* p)
 {
-  Playerlist* pl = Playerlist::getInstance();
-
   blank(true);
 
   next_player_turn.emit(p, d_gameScenario->getRound());
@@ -1172,8 +1089,7 @@ void Game::init_turn_for_player(Player* p)
     {
       if (Commentator::getInstance()->hasComment() == true)
         {
-          std::vector<Glib::ustring> comments =
-            Commentator::getInstance()->getComments(p);
+          auto comments = Commentator::getInstance()->getComments(p);
           if (comments.size() > 0)
             commentator_comments.emit(comments[Rnd::rand() % comments.size()]);
         }
@@ -1192,15 +1108,15 @@ void Game::init_turn_for_player(Player* p)
 
       // update the diplomacy icon if we've received a proposal
       bool proposal_received = false;
-      for (Playerlist::iterator it = pl->begin(); it != pl->end(); ++it)
+      for (auto it: *Playerlist::getInstance())
 	{
-	  if ((*it) == pl->getNeutral())
+	  if (it == Playerlist::getInstance()->getNeutral())
 	    continue;
-	  if ((*it) == p)
+	  if (it == p)
 	    continue;
-	  if((*it)->isDead())
+	  if(it->isDead())
 	    continue;
-	  if ((*it)->getDiplomaticProposal(p) != Player::NO_PROPOSAL)
+	  if (it->getDiplomaticProposal(p) != Player::NO_PROPOSAL)
 	    {
 	      proposal_received = true;
 	      break;
@@ -1212,6 +1128,10 @@ void Game::init_turn_for_player(Player* p)
       if (p->countDestituteCitiesThisTurn() > 0)
         destitute = true;
       city_too_poor_to_produce.emit(destitute);
+
+      if (p->countEndTurnHistoryEntries() == 1 &&
+          Main::instance().own_all_on_round_two)
+        p->conquerAllCities();
     }
   else
     {
@@ -1221,14 +1141,12 @@ void Game::init_turn_for_player(Player* p)
 
 void Game::on_player_died(Player *player)
 {
-  const Playerlist* pl = Playerlist::getInstance();
-  if (pl->getNoOfPlayers() <= 1)
-    game_over.emit(pl->getFirstLiving());
+  if (Playerlist::getInstance()->getNoOfPlayers() <= 1)
+    game_over.emit(Playerlist::getInstance()->getFirstLiving());
   else
     player_died.emit(player);
 }
 
-      
 void Game::on_fight_started(Fight &fight)
 {
   
@@ -1372,10 +1290,9 @@ void Game::nextRound()
     
 void Game::on_surrender_offered(Player *recipient)
 {
-  Playerlist *plist = Playerlist::getInstance();
-  if (enemy_offers_surrender.emit(plist->countPlayersAlive() - 1))
+  if (enemy_offers_surrender(Playerlist::getInstance()->countPlayersAlive() - 1))
     {
-      plist->surrender();
+      Playerlist::getInstance()->surrender();
       surrender_answered.emit(true);
       game_over.emit(recipient);
     }
@@ -1452,8 +1369,7 @@ void Game::inhibitAutosaveRemoval(bool inhibit)
 
 void Game::endOfGameRoaming(Player *winner)
 {
-  Playerlist* pl = Playerlist::getInstance();
-  pl->setWinningPlayer(winner);
+  Playerlist::getInstance()->setWinningPlayer(winner);
   Playerlist::getActiveplayer()->immobilize();
   d_gameScenario->s_see_opponents_stacks = true;
   d_gameScenario->s_see_opponents_production = true;
@@ -1506,10 +1422,10 @@ bool Game::stack_searches_temple(Stack *stack)
   return hero_got_quest;
 }
 
-void Game::on_ruinfight_started(Stack *attacker, Stack *defender)
+void Game::on_ruinfight_started(Stack *attacker, Keeper *keeper)
 {
   if (Playerlist::getActiveplayer()->isComputer() == false)
-    ruinfight_started.emit(attacker, defender);
+    ruinfight_started.emit(attacker, keeper);
 }
 
 void Game::on_ruinfight_finished(Fight::Result result)
@@ -1517,3 +1433,32 @@ void Game::on_ruinfight_finished(Fight::Result result)
   if (Playerlist::getActiveplayer()->isComputer() == false)
     ruinfight_finished.emit(result);
 }
+
+void Game::on_save_game(Glib::ustring filename)
+{
+  if (getScenario())
+    getScenario()->saveGame(filename);
+}
+
+guint32 Game::on_get_round()
+{
+  if (getScenario())
+    return getScenario()->getRound();
+  else
+    return 0;
+}
+
+void Game::on_pointing_at_new_tile (Vector<int> tile)
+{
+  pointing_at_new_tile.emit(tile);
+}
+
+void Game::on_bag_dropped ()
+{
+  redraw ();
+}
+
+void Game::on_stack_died ()
+{
+  redraw ();
+}
