@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2010, 2011, 2014, 2015 Ben Asselstine
+//  Copyright (C) 2007-2011, 2014, 2015, 2017, 2020 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -47,9 +47,10 @@
 //#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<std::flush;}
 #define debug(x)
 
-GameBigMap::GameBigMap(bool intense_combat, bool see_opponents_production,
-		       bool see_opponents_stacks, bool military_advisor)
-:d_fighting(LocationBox(Vector<int>(-1,-1)))
+GameBigMap::GameBigMap(bool headless, bool intense_combat,
+                       bool see_opponents_production,
+                       bool see_opponents_stacks, bool military_advisor)
+ :BigMap(headless)
 {
   path_calculator = NULL;
   d_intense_combat = intense_combat;
@@ -62,13 +63,12 @@ GameBigMap::GameBigMap(bool intense_combat, bool see_opponents_production,
 
   prev_mouse_pos = Vector<int>(0, 0);
 
-  // setup timeout
-  selection_timeout_handler = Timing::instance().register_timer
-    (sigc::mem_fun(*this, &GameBigMap::on_selection_timeout),
-     TIMER_BIGMAP_SELECTOR);
+  if (!d_headless)
+    selection_timeout_handler = Timing::instance().register_timer
+      (sigc::mem_fun(*this, &GameBigMap::on_selection_timeout),
+       TIMER_BIGMAP_SELECTOR);
   shift_key_is_down = false;
   control_key_is_down = false;
-  magnification_factor = 1.0;
 }
 
 GameBigMap::~GameBigMap()
@@ -100,7 +100,7 @@ void GameBigMap::select_active_stack()
 void GameBigMap::unselect_active_stack()
 {
   Playerlist::getActiveplayer()->stackDeselect();
-  draw(Playerlist::getViewingplayer());
+  draw();
   stack_selected.emit(0);
   if (path_calculator)
     {
@@ -114,7 +114,7 @@ bool GameBigMap::on_selection_timeout()
 {
   // redraw to update the selection
   if (Playerlist::getActiveplayer()->getActivestack())
-    draw(Playerlist::getViewingplayer());
+    draw();
 
   return Timing::CONTINUE;
 }
@@ -157,7 +157,12 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
       Stack* stack = Playerlist::getActiveplayer()->getActivestack();
 
       if (d_cursor == ImageCache::HAND)
-	return;
+        {
+          d_cursor = ImageCache::CLOSED_HAND;
+          cursor_changed.emit (d_cursor);
+        }
+      if (d_cursor == ImageCache::CLOSED_HAND)
+        return;
 
       if (stack)
 	{
@@ -172,6 +177,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	    }
 	  else if (d_cursor == ImageCache::RUIN)
 	    {
+              // we're holding shift down here
 	      if (Ruin *r = GameMap::getRuin(tile))
 		{
 		  if ((r->isHidden() == true && 
@@ -179,11 +185,13 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		      r->isHidden() == false)
 		    {
 		      set_shift_key_down (false);
+                      cursor_changed.emit(ImageCache::POINTER);
 		      ruin_queried (r, false);
 		    }
 		}
 	      else if (Temple *t = GameMap::getTemple(tile))
 		{
+                  cursor_changed.emit(ImageCache::POINTER);
 		  temple_queried (t, false);
 		  set_shift_key_down (false);
 		}
@@ -199,6 +207,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		      set_control_key_down (false);
 		      if (d_see_opponents_production == true)
 			{
+                          cursor_changed.emit(ImageCache::POINTER);
 			  city_visited.emit (c);
 			  set_shift_key_down (false);
 			  return;
@@ -207,6 +216,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 			{
 			  if (c->getOwner() == Playerlist::getActiveplayer())
 			    {
+                              cursor_changed.emit(ImageCache::POINTER);
 			      city_visited.emit(c);
 			      set_shift_key_down (false);
 			      return;
@@ -220,7 +230,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	      //set in a course, mr crusher.
 	      stack->getPath()->calculate(stack, tile);
 	      path_set.emit();
-	      draw(Playerlist::getViewingplayer());
+	      draw();
 	      return;
 	    }
 	  Vector<int> p;
@@ -232,7 +242,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	      if (double_clicked == true && is_control_key_down() == false)
 		{
 		  StackTile *stile = GameMap::getStacks(stack->getPos());
-		  std::list<Stack *> stks= stile->getFriendlyStacks(active);
+		  std::vector<Stack *> stks= stile->getFriendlyStacks(active);
 		  if (stks.size() == 1)
 		    stile->ungroup(active);
 		  else
@@ -245,7 +255,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		    delete path_calculator;
                   stack = active->getActivestack();
 		  path_calculator = new PathCalculator(stack);
-		  draw(Playerlist::getViewingplayer());
+		  draw();
 		  stack_grouped_or_ungrouped.emit(stack);
 		  return;
 		}
@@ -270,7 +280,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		  // clear the path
 		  stack->getPath()->clear();
 		  path_set.emit();
-		  draw(Playerlist::getViewingplayer());
+		  draw();
 		  return;
 		}
 	    }
@@ -358,7 +368,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 
 	  path_set.emit();
 
-	  draw(Playerlist::getViewingplayer());
+	  draw();
 	}
       // Stack hasn't been active yet
       else
@@ -380,6 +390,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		      set_control_key_down (false);
 		      if (d_see_opponents_production == true)
 			{
+                          cursor_changed.emit(ImageCache::POINTER);
 			  city_visited.emit (c);
 			  set_shift_key_down (false);
 			}
@@ -387,6 +398,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 			{
 			  if (c->getOwner() == Playerlist::getActiveplayer())
 			    {
+                              cursor_changed.emit(ImageCache::POINTER);
 			      city_visited.emit (c);
 			      set_shift_key_down (false);
 			    }
@@ -397,10 +409,14 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		{
 		  if ((r->isHidden() == true && r->getOwner() == viewing) ||
 		      r->isHidden() == false)
-		    ruin_queried (r, false);
+                    {
+                      cursor_changed.emit(ImageCache::POINTER);
+                      ruin_queried (r, false);
+                    }
 		}
 	      else if (Temple *t = GameMap::getTemple(tile))
 		{
+                  cursor_changed.emit(ImageCache::POINTER);
 		  temple_queried (t, false);
 		}
 	    }
@@ -409,6 +425,11 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
   else if (e.button == MouseButtonEvent::LEFT_BUTTON
       && e.state == MouseButtonEvent::RELEASED)
     {
+      if (d_cursor == ImageCache::CLOSED_HAND)
+        {
+          d_cursor = ImageCache::HAND;
+	  cursor_changed.emit(d_cursor);
+        }
       if (mouse_state == DRAGGING_ENDPOINT)
 	{
 	  mouse_state = NONE;
@@ -452,7 +473,12 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	{
 	  if (viewing->getFogMap()->isCompletelyObscuredFogTile(tile) == true)
 	    return;
-	  if (City* c = GameMap::getCity(tile))
+          Stack *selected_stack = Playerlist::getActiveplayer()->getActivestack();
+          if (selected_stack != NULL && tile == selected_stack->getPos())
+            {
+              popup_stack_actions_menu.emit (selected_stack);
+            }
+          else if (City* c = GameMap::getCity(tile))
 	    {
 	      city_queried.emit (tile, c);
 	      mouse_state = SHOWING_CITY;
@@ -526,6 +552,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		{
 		  Playerlist::getActiveplayer()->setActivestack(0);
 		  unselect_active_stack();
+                  path_turns.emit (Vector<int>(-1,-1), 0);
 		  mouse_state = NONE;
 		  determine_mouse_cursor(NULL, current_tile);
 		}
@@ -547,7 +574,7 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
       d_cursor = ImageCache::HAND;
     }
   else if (mouse_state == DRAGGING_MAP)
-    d_cursor = ImageCache::HAND;
+    d_cursor = ImageCache::CLOSED_HAND;
   else if (stack && 
 	   (mouse_state == DRAGGING_STACK || mouse_state == DRAGGING_ENDPOINT))
     {
@@ -710,6 +737,8 @@ void GameBigMap::mouse_motion_event(MouseMotionEvent e)
   Player *viewing = Playerlist::getViewingplayer();
   Stack* stack = active->getActivestack();
   Vector<int> tile = mouse_pos_to_tile(e.pos);
+  if (current_tile != tile && d_grid_toggled)
+    pointing_at_new_tile.emit (tile);
   current_tile = tile;
   if (tile.x < 0)
     tile.x = 0;
@@ -723,7 +752,7 @@ void GameBigMap::mouse_motion_event(MouseMotionEvent e)
 
   if (e.pressed[MouseMotionEvent::LEFT_BUTTON]
       && (mouse_state == NONE || mouse_state == SHOWING_STACK) && 
-      stack && stack->getPos() == tile && viewing->getFogMap()->isCompletelyObscuredFogTile(tile) == false && d_cursor != ImageCache::HAND)
+      stack && stack->getPos() == tile && viewing->getFogMap()->isCompletelyObscuredFogTile(tile) == false && (d_cursor != ImageCache::HAND && d_cursor != ImageCache::CLOSED_HAND))
     {
       //initial dragging of stack from it's tile
       mouse_state = DRAGGING_STACK;
@@ -735,7 +764,7 @@ void GameBigMap::mouse_motion_event(MouseMotionEvent e)
       mouse_state = DRAGGING_ENDPOINT;
     }
   else if (e.pressed[MouseMotionEvent::LEFT_BUTTON]
-	   && (mouse_state == NONE || mouse_state == DRAGGING_MAP) && d_cursor == ImageCache::HAND)
+	   && (mouse_state == NONE || mouse_state == DRAGGING_MAP) && (d_cursor == ImageCache::HAND || d_cursor == ImageCache::CLOSED_HAND))
     {
       Vector<int> delta = -(e.pos - prev_mouse_pos);
 
@@ -748,10 +777,10 @@ void GameBigMap::mouse_motion_event(MouseMotionEvent e)
       Vector<int> screen_dim(image.get_width(), image.get_height());
       view_pos = clip(Vector<int>(0, 0),
 		      view_pos + delta,
-		      GameMap::get_dim() * ts *magnification_factor - screen_dim);
+		      GameMap::get_dim() * ts - screen_dim);
 
       // calculate new view position in tiles, rounding up
-      Vector<int> new_view = (view_pos + Vector<int>(ts * magnification_factor - 1, ts * magnification_factor - 1)) / (ts * magnification_factor);
+      Vector<int> new_view = (view_pos + Vector<int>(ts - 1, ts - 1)) / ts;
 
       if (new_view != view.pos)
 	{
@@ -761,10 +790,10 @@ void GameBigMap::mouse_motion_event(MouseMotionEvent e)
 	  view.x = new_view.x;
 	  view.y = new_view.y;
 	  view_changed.emit(view);
-	  draw(Playerlist::getViewingplayer(),true);
+	  draw(true);
 	}
       else
-	draw(Playerlist::getViewingplayer(), false);
+	draw(false);
       mouse_state = DRAGGING_MAP;
     }
 
@@ -817,7 +846,7 @@ void GameBigMap::mouse_motion_event(MouseMotionEvent e)
 	  delete new_path;
 	  //stack->getPath()->calculate(stack, tile);
 	  path_set.emit();
-	  draw(Playerlist::getViewingplayer());
+	  draw();
 	}
     }
 
@@ -839,7 +868,6 @@ void GameBigMap::after_draw()
   int tilesize = GameMap::getInstance()->getTileSize();
 
   Stack* stack = Playerlist::getActiveplayer()->getActivestack();
-
   // Draw Path
   if (stack && stack->getPath()->size() && 
       stack->getOwner()->getType() == Player::HUMAN)
@@ -862,10 +890,12 @@ void GameBigMap::after_draw()
 	  canMoveThere = (pathcount < stack->getPath()->getMovesExhaustedAtPoint());
           PixMask *waypoint;
 	  if (canMoveThere)
-            waypoint = gc->getWaypointImage(0);
+            waypoint = gc->getWaypointImage(0)->copy();
 	  else
-            waypoint = gc->getWaypointImage(1);
+            waypoint = gc->getWaypointImage(1)->copy();
+          waypoint->scale (waypoint, tilesize, tilesize);
           waypoint->blit_centered (buffer, pos + (Vector<int>(tilesize,tilesize)/2));
+          delete waypoint;
 	  pathcount++;
 
 	}
@@ -876,13 +906,15 @@ void GameBigMap::after_draw()
 	  Path::iterator it = stack->getPath()->end();
 	  it--;
 	  //this is where the ghosted army unit picture goes.
-	  PixMask *armypic = gc->getArmyPic(*stack->begin(), true);
+	  PixMask *armypic = gc->getArmyPic(*stack->begin(), true)->copy();
+          armypic->scale (armypic, tilesize, tilesize);
 	  pos = tile_to_buffer_pos(*it);
 	  armypic->blit_centered(buffer, pos + (Vector<int>(tilesize,tilesize)/2));
+          delete armypic;
 	}
     }
 
-  if (stack && d_fighting.getPos() == Vector<int>(-1,-1))
+  if (stack)
     {
       Player *viewer = Playerlist::getViewingplayer();
       // draw the selection
@@ -904,37 +936,54 @@ void GameBigMap::after_draw()
 
 	  p = tile_to_buffer_pos(p);
 
-	  draw_stack (stack, buffer);
+          draw_stack (stack, buffer);
 
 	  PixMask *tmp = NULL;
 	  if (stack->size() > 1)
-	    tmp = gc->getSelectorPic(0, bigframe, stack->getOwner());
+	    tmp = gc->getSelectorPic(0, bigframe, stack->getOwner())->copy();
 	  else
-	    tmp = gc->getSelectorPic(1, smallframe, stack->getOwner());
+	    tmp = gc->getSelectorPic(1, smallframe, stack->getOwner())->copy();
+          tmp->scale(tmp, tilesize, tilesize);
 	  tmp->blit(buffer, p);
+          delete tmp;
 	  //now re-fog it up because we just drew over the fog.
 	  if (viewer->getFogMap()->isFogged(stack->getPos()))
 	    {
 	      int fog_type_id = 
 		viewer->getFogMap()->getShadeTile(stack->getPos());
-	      PixMask *fog = gc->getFogPic(fog_type_id);
+	      PixMask *fog = gc->getFogPic(fog_type_id)->copy();
+              fog->scale(fog, tilesize, tilesize);
 	      fog->blit(buffer, p);
+              delete fog;
 	    }
 	}
+	  
+      if (current_tile != stack->getPos() &&
+          Playerlist::getActiveplayer()->getType() == Player::HUMAN)
+        {
+          //this is where the ghosted army unit picture goes.
+          PixMask *armypic = gc->getArmyPic(*stack->begin(), true)->copy();
+          armypic->scale (armypic, tilesize, tilesize);
+          Vector<int> pos = tile_to_buffer_pos(current_tile);
+          armypic->blit_centered(buffer, pos + (Vector<int>(tilesize,tilesize)/2));
+          delete armypic;
+          static Vector<int> prev_current_tile;
+          if (current_tile != prev_current_tile)
+            {
+              PathCalculator pc(stack);
+              guint32 moves, turns, left;
+              pc.calculate (current_tile, moves, turns, left);
+              if (turns >= 1 && left == stack->getMaxMoves ())
+                turns--;
+              if (turns > 0)
+                path_turns.emit (current_tile, turns+1);
+              else
+                path_turns.emit (Vector<int>(-1,-1), 0);
+              prev_current_tile = current_tile;
+            }
+        }
     }
 
-  if (d_fighting.getPos() != Vector<int>(-1,-1))
-    {
-      Vector<int> p = tile_to_buffer_pos(d_fighting.getPos());
-      PixMask *tmp = gc->getExplosionPic()->copy();
-      if (d_fighting.getSize() > 1)
-	{
-	  PixMask::scale(tmp, d_fighting.getSize() * tilesize,
-			 d_fighting.getSize() * tilesize);
-	}
-      tmp->blit(buffer, p);
-      delete tmp;
-    }
 }
 
 void GameBigMap::set_control_key_down (bool down)
