@@ -1,5 +1,6 @@
 //  Copyright (C) 2007, 2008, Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2010, 2012, 2014, 2015 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2014, 2015, 2016,
+//  2017, 2020 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -16,7 +17,6 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
-#include <assert.h>
 #include <sigc++/functors/mem_fun.h>
 #include <sigc++/bind.h>
 #include <errno.h>
@@ -71,25 +71,22 @@
 #include "citysetlist.h"
 #include "herotemplates.h"
 #include "new-network-game-dialog.h"
+#include "../editor/main-window.h"
+#include "../editor/editor-splash-window.h"
+#include "smallmap.h"
+#include "ScenarioMedia.h"
 
-Driver::Driver(Glib::ustring load_filename)
+#define method(x) sigc::mem_fun(*this, &Driver::x)
+
+Driver::Driver(bool start_editor, Glib::ustring load_filename)
 {
     game_window = NULL;
     game_lobby_dialog = NULL;
+    quick_help_window = NULL;
     splash_window = NULL;
     download_window = NULL;
-  game_scenario_downloaded = "";
-    splash_window = new SplashWindow;
-    splash_window->new_game_requested.connect(
-	sigc::mem_fun(*this, &Driver::on_new_game_requested));
-    splash_window->new_hosted_network_game_requested.connect(
-	sigc::mem_fun(*this, &Driver::on_new_hosted_network_game_requested));
-    splash_window->new_remote_network_game_requested.connect(
-	sigc::mem_fun(*this, &Driver::on_new_remote_network_game_requested));
-    splash_window->load_requested.connect(
-	sigc::mem_fun(*this, &Driver::on_load_requested));
-    splash_window->quit_requested.connect(
-	sigc::mem_fun(*this, &Driver::on_quit_requested));
+    editor_window = NULL;
+    game_scenario_downloaded = "";
 
     d_load_filename = load_filename;
 
@@ -141,7 +138,28 @@ Driver::Driver(Glib::ustring load_filename)
           serve (game_scenario);
         return;
       }
+
+    splash_window = new SplashWindow;
+    splash_window->new_game_requested.connect(method(on_new_game_requested));
+    splash_window->new_hosted_network_game_requested.connect
+      (method(on_new_hosted_network_game_requested));
+    splash_window->new_remote_network_game_requested.connect
+      (method(on_new_remote_network_game_requested));
+    splash_window->load_requested.connect(method(on_load_requested));
+    splash_window->editor_requested.connect(method(on_editor_requested));
+    splash_window->quit_requested.connect(method(on_quit_requested));
+
+    if (start_editor)
+      {
+        on_editor_requested(load_filename);
+        return;
+      }
     splash_window->show();
+    if (getenv ("LORDSAWAR_GUI_TEST") != NULL)
+      {
+        splash_window->get_window()->set_position(Gtk::WIN_POS_NONE);
+        splash_window->get_window()->move(0, 0);
+      }
     //here are the ones that do
     run();
 }
@@ -153,35 +171,33 @@ void Driver::serve (GameScenario *game_scenario)
   Playerlist::getInstance()->syncNeutral();
   //okay we're going to host a game, using this file as a scenario.
   GameServer *game_server = GameServer::getInstance();
-  guint32 port = LORDSAWAR_PORT;
-  if (Main::instance().port)
-    port = Main::instance().port;
-  game_server->port_in_use.connect(sigc::mem_fun(*this, &Driver::on_could_not_bind_to_port_for_headless_server));
+  game_server->port_in_use.connect(method(on_could_not_bind_to_port_for_headless_server));
   Glib::ustring id = "";
   if (Profilelist::getInstance()->empty() == false)
     id = Profilelist::getInstance()->front()->getId();
-  game_server->start(game_scenario, port, id, "admin");
+  game_server->start(game_scenario, get_port(), id, "admin");
   game_server = GameServer::getInstance();
   if (game_server->isListening() == false)
     {
       GameServer::deleteInstance();
       return;
     }
-  printf("Game Server is now listening on port %d\n", port);
-  NextTurnNetworked *next_turn = new NextTurnNetworked(game_scenario->getTurnmode(), game_scenario->s_random_turns);
+  printf("Game Server is now listening on port %d\n", get_port());
+  NextTurnNetworked *next_turn = new NextTurnNetworked();
   game_server->round_ends.connect(sigc::mem_fun(next_turn, &NextTurnNetworked::finishRound));
   game_server->start_player_turn.connect(sigc::mem_fun(next_turn, &NextTurnNetworked::start_player));
   next_turn->srequestAbort.connect(sigc::mem_fun(game_server, &GameServer::on_turn_aborted));
   game_server->get_next_player.connect(sigc::mem_fun(next_turn, &NextTurnNetworked::next));
-  game_server->round_ends.connect
-    (sigc::mem_fun(*this, &Driver::on_keep_network_play_going));
+  game_server->round_ends.connect (method(on_keep_network_play_going));
   Playerlist::getInstance()->splayerDead.connect
     (sigc::mem_fun(GameServer::getInstance(), &GameServer::sendKillPlayer));
 
   game_server->notifyClientsGameMayBeginNow();
-  Game *game = new Game(game_scenario, next_turn);
+  Configuration::s_autosave_policy = Configuration::NO_SAVING;
+  Game *game = new Game(game_scenario, next_turn, true);
+  game->game_over.connect (sigc::bind (sigc::mem_fun (this, &Driver::on_game_over_for_headless_server), game_scenario));
   if (game)
-    game_server->player_sits.connect(sigc::hide(sigc::hide(sigc::mem_fun(this, &Driver::on_client_sits_down_in_headless_server_game))));
+    game_server->player_sits.connect(sigc::hide(sigc::hide(method(on_client_sits_down_in_headless_server_game))));
 }
 
 void Driver::on_client_sits_down_in_headless_server_game()
@@ -264,18 +280,21 @@ void Driver::get_default(int num_players, GameParameters &g)
   g.tile_theme = "default";
   g.shield_theme = "default";
   g.city_theme = "default";
-  g.process_armies = GameParameters::PROCESS_ARMIES_AT_PLAYERS_TURN;
-  g.difficulty = GameScenario::calculate_difficulty_rating(g);
   g.cities_can_produce_allies = false;
   g.cusp_of_war = false;
   g.see_opponents_stacks = false;
   g.see_opponents_production = false;
+  g.vectoring_mode = GameParameters::VECTORING_ALWAYS_TWO_TURNS;
+  g.build_production_mode = GameParameters::BUILD_PRODUCTION_ALWAYS;
+  g.sacking_mode = GameParameters::SACKING_ALWAYS;
+  g.difficulty = GameScenario::calculate_difficulty_rating(g);
 }
 
 void Driver::run()
 {
   if (Main::instance().start_test_scenario) 
     {
+      SmallMap::s_quick = true;
       // quick load a test scenario
       GameParameters g;
       GameParameters::Player p;
@@ -294,43 +313,115 @@ void Driver::run()
 	  p.name = "";
 	  g.players.push_back(p);
 	}
+      //g.map.width = 50;
+      //g.map.height = 75;
+      //g.map.width = 70;
+      //g.map.height = 105;
+      //g.map.width = 112;
+      //g.map.height = 156;
       g.map.width = 125;
       g.map.height = 125;
+      //g.map.width = 255;
+      //g.map.height = 255;
+      //g.map.width = 500;
+      //g.map.height = 500;
       g.map.grass = 78;
       g.map.water = 7;
       g.map.swamp = 2;
       g.map.forest = 3;
       g.map.hills = 5;
       g.map.mountains = 5;
+      //g.map.cities = 3;
       g.map.cities = 40;
+      //g.map.cities = 80;
       g.map.ruins = 25;
       g.map.temples = 3;
+      g.map.signposts = 0;
+
       g.map_path = "";
-      g.play_with_quests = GameParameters::NO_QUESTING;
-      g.hidden_map = false;
-      g.neutral_cities = GameParameters::STRONG;
-      g.razing_cities = GameParameters::ALWAYS;
-      g.diplomacy = false;
-      g.random_turns = false;
-      g.quick_start = GameParameters::NO_QUICK_START;
-      g.intense_combat = false;
-      g.military_advisor = false;
-      g.army_theme = "default";
       g.tile_theme = "default";
+      g.army_theme = "default";
       g.shield_theme = "default";
       g.city_theme = "default";
-      g.process_armies = GameParameters::PROCESS_ARMIES_AT_PLAYERS_TURN;
-      g.difficulty = GameScenario::calculate_difficulty_rating(g);
-      g.cities_can_produce_allies = false;
-      g.cusp_of_war = false;
+
       g.see_opponents_stacks = true;
       g.see_opponents_production = true;
-      on_new_game_requested(g);
+      g.play_with_quests = GameParameters::NO_QUESTING;
+      g.hidden_map = false;
+      g.diplomacy = false;
+
+      g.neutral_cities = GameParameters::STRONG;
+      g.razing_cities = GameParameters::ALWAYS;
+      g.quick_start = GameParameters::NO_QUICK_START;
+      g.cusp_of_war = false;
+      g.intense_combat = false;
+      g.military_advisor = false;
+      g.random_turns = false;
+      g.cities_can_produce_allies = false;
+      g.name = "";
+
+      g.difficulty = GameScenario::calculate_difficulty_rating(g);
+      on_new_game_requested(g, NULL);
+    }
+  else if (Main::instance().start_net_test_scenario)
+    {
+      // quick load a test scenario
+      GameParameters g;
+      GameParameters::Player p;
+      for (unsigned int i = 0; i < MAX_PLAYERS; i++)
+	{
+	  p.type = GameParameters::Player::EASY;
+	  p.id = i;
+	  p.name = String::ucompose ("Player %1", i + 1);
+	  g.players.push_back(p);
+	}
+      g.map.width = 50;
+      g.map.height = 75;
+      g.map.grass = 78;
+      g.map.water = 7;
+      g.map.swamp = 2;
+      g.map.forest = 3;
+      g.map.hills = 5;
+      g.map.mountains = 5;
+      g.map.cities = 8;
+      g.map.ruins = 5;
+      g.map.temples = 3;
+      g.map.signposts = 0;
+
+      g.map_path = "";
+      g.tile_theme = "default";
+      g.army_theme = "default";
+      g.shield_theme = "default";
+      g.city_theme = "default";
+
+      g.see_opponents_stacks = true;
+      g.see_opponents_production = true;
+      g.play_with_quests = GameParameters::NO_QUESTING;
+      g.hidden_map = false;
+      g.diplomacy = false;
+
+      g.neutral_cities = GameParameters::STRONG;
+      g.razing_cities = GameParameters::ALWAYS;
+      g.quick_start = GameParameters::NO_QUICK_START;
+      g.cusp_of_war = false;
+      g.intense_combat = false;
+      g.military_advisor = false;
+      g.random_turns = false;
+      g.cities_can_produce_allies = false;
+      g.name = "";
+
+      g.difficulty = GameScenario::calculate_difficulty_rating(g);
+      if (Profilelist::getInstance()->empty() == true)
+        Profilelist::getInstance()->push_back(new Profile(Glib::get_user_name()));
+
+      on_new_hosted_network_game_requested(g, get_port(),
+                                           Profilelist::getInstance()->front(),
+                                           false, false);
     }
   else if (Main::instance().start_robots != 0) 
     {
       Snd::deleteInstance();
-      lordsawaromatic("127.0.0.1", LORDSAWAR_PORT, Player::AI_FAST,
+      lordsawaromatic("127.0.0.1", get_port(), Player::AI_FAST,
 		      Main::instance().start_robots);
     }
   else
@@ -344,8 +435,8 @@ void Driver::run()
 	  if (found != Glib::ustring::npos)
 	    {
 	      GamePreferencesDialog d(*splash_window->get_window(), d_load_filename, GameScenario::HOTSEAT);
-	      d.game_started.connect(sigc::mem_fun
-				     (*this, &Driver::on_new_game_requested));
+	      d.game_started.connect(sigc::bind(sigc::mem_fun
+				     (*this, &Driver::on_new_game_requested), &d));
 	      d.run();
 	    }
 	  else
@@ -354,7 +445,7 @@ void Driver::run()
 	      if (found != Glib::ustring::npos)
 		on_load_requested(d_load_filename);
 	      else
-		on_new_game_requested(g);
+		on_new_game_requested(g, NULL);
 	    }
 	}
     }
@@ -367,6 +458,8 @@ Driver::~Driver()
     delete game_window;
   if (game_lobby_dialog)
     delete game_lobby_dialog;
+  if (quick_help_window)
+    delete quick_help_window;
   if (splash_window)
     delete splash_window;
   if (download_window)
@@ -452,6 +545,7 @@ GameScenario *Driver::create_new_scenario(GameParameters &g, GameScenario::PlayM
       Glib::ustring path = 
         NewRandomMapDialog::create_and_dump_scenario("random.map", g, NULL);
       g.map_path = path;
+      start_game_progress_tick.emit ();
     }
   else
     update_uuid = true;
@@ -462,9 +556,13 @@ GameScenario *Driver::create_new_scenario(GameParameters &g, GameScenario::PlayM
   if (broken)
     return NULL;
 
+  start_game_progress_tick.emit ();
   GameScenarioOptions::s_see_opponents_stacks = g.see_opponents_stacks;
   GameScenarioOptions::s_see_opponents_production = g.see_opponents_production;
   GameScenarioOptions::s_play_with_quests = g.play_with_quests;
+  GameScenarioOptions::s_vectoring_mode = g.vectoring_mode;
+  GameScenarioOptions::s_sacking_mode = g.sacking_mode;
+  GameScenarioOptions::s_build_production_mode = g.build_production_mode;
   GameScenarioOptions::s_hidden_map = g.hidden_map;
   GameScenarioOptions::s_diplomacy = g.diplomacy;
   GameScenarioOptions::s_cusp_of_war = g.cusp_of_war;
@@ -484,6 +582,7 @@ GameScenario *Driver::create_new_scenario(GameParameters &g, GameScenario::PlayM
       Playerlist::getInstance()->syncPlayers(g.players);
       game_scenario->initialize(g);
     }
+  start_game_progress_tick.emit ();
   return game_scenario;
 }
 
@@ -492,7 +591,7 @@ void Driver::advertise_game(GameScenario *game_scenario, Profile *p)
   GamelistClient::deleteInstance();
   GamelistClient *gsc = GamelistClient ::getInstance();
   gsc->client_connected.connect
-    (sigc::bind(sigc::mem_fun(*this, &Driver::on_connected_to_gamelist_server_for_advertising), game_scenario, p));
+    (sigc::bind(method(on_connected_to_gamelist_server_for_advertising), game_scenario, p));
   gsc->start(Configuration::s_gamelist_server_hostname,
              Configuration::s_gamelist_server_port, p);
 }
@@ -505,7 +604,7 @@ void Driver::on_connected_to_gamelist_server_for_advertising(GameScenario *game_
     new RecentlyPlayedNetworkedGame(game_scenario, p);
   g->setNumberOfPlayers(Playerlist::getInstance()->countPlayersAlive());
   gsc->received_advertising_response.connect
-    (sigc::hide(sigc::mem_fun(*this, &Driver::on_advertising_response_received)));
+    (sigc::hide(method(on_advertising_response_received)));
   gsc->request_advertising(g);
 }
 
@@ -522,8 +621,8 @@ void Driver::remotely_serve (GameScenario *game_scenario, Profile *p)
     return;
   GamehostClient::deleteInstance();
   GamehostClient *ghc = GamehostClient::getInstance();
-  ghc->client_connected.connect(sigc::bind(sigc::mem_fun(*this, &Driver::on_connected_to_gamehost_server_for_hosting_request), game_scenario));
-  ghc->client_could_not_connect.connect(sigc::mem_fun(*this, &Driver::on_could_not_connect_to_gamehost_server));
+  ghc->client_connected.connect(sigc::bind(method(on_connected_to_gamehost_server_for_hosting_request), game_scenario));
+  ghc->client_could_not_connect.connect(method(on_could_not_connect_to_gamehost_server));
   ghc->start(Configuration::s_gamehost_server_hostname,
              Configuration::s_gamehost_server_port, p);
 }
@@ -545,7 +644,7 @@ void Driver::on_could_not_connect_to_gamehost_server()
 void Driver::on_connected_to_gamehost_server_for_hosting_request (GameScenario *game_scenario)
 {
   GamehostClient *ghc = GamehostClient::getInstance();
-  ghc->received_host_response.connect(sigc::bind(sigc::hide<0>(sigc::mem_fun(*this, &Driver::on_got_game_host_response)), game_scenario));
+  ghc->received_host_response.connect(sigc::bind(sigc::hide<0>(method(on_got_game_host_response)), game_scenario));
   ghc->request_game_host (game_scenario->getId());
 }
 
@@ -565,13 +664,12 @@ void Driver::on_got_game_host_response(Glib::ustring err, GameScenario *game_sce
       return;
     }
   GamehostClient *ghc = GamehostClient::getInstance();
-  ghc->received_map_response.connect(sigc::hide<0>(sigc::mem_fun(*this, &Driver::on_remote_game_hosted)));
+  ghc->received_map_response.connect(sigc::hide<0>(method(on_remote_game_hosted)));
   if (download_window)
     delete download_window;
   download_window = new NewNetworkGameDownloadWindow(_("Uploading."));
   download_window->pulse();
-  upload_heartbeat_conn = Glib::signal_timeout().connect
-    (sigc::mem_fun(*this, &Driver::upload_heartbeat), 1 * 1000);
+  upload_heartbeat_conn = Glib::signal_timeout().connect (method(upload_heartbeat), 1 * 1000);
   ghc->send_map(game_scenario);
 }
 
@@ -647,7 +745,7 @@ void Driver::on_load_hosted_network_game_requested(GameScenario *game_scenario,
     }
 
   GameServer *game_server = GameServer::getInstance();
-  game_server->port_in_use.connect(sigc::mem_fun(*this, &Driver::on_could_not_bind_to_port));
+  game_server->port_in_use.connect(method(on_could_not_bind_to_port));
   game_server->start(game_scenario, port, p->getId(), p->getNickname());
   game_server = GameServer::getInstance();
   if (game_server->isListening() == false)
@@ -657,12 +755,10 @@ void Driver::on_load_hosted_network_game_requested(GameScenario *game_scenario,
     }
   if (advertised)
     advertise_game(game_scenario, p);
-  NextTurnNetworked *next_turn = new NextTurnNetworked(game_scenario->getTurnmode(), game_scenario->s_random_turns);
+  NextTurnNetworked *next_turn = new NextTurnNetworked();
   game_server->round_ends.connect(sigc::mem_fun(next_turn, &NextTurnNetworked::finishRound));
   if (advertised)
-    game_server->round_ends.connect
-      (sigc::bind(sigc::mem_fun(*this, &Driver::on_advertised_game_round_ends),
-                  game_scenario, p));
+    game_server->round_ends.connect (sigc::bind(method(on_advertised_game_round_ends), game_scenario, p));
   game_server->start_player_turn.connect(sigc::mem_fun(next_turn, &NextTurnNetworked::start_player));
   next_turn->srequestAbort.connect(sigc::mem_fun(game_server, &GameServer::on_turn_aborted));
   if (game_lobby_dialog)
@@ -671,24 +767,20 @@ void Driver::on_load_hosted_network_game_requested(GameScenario *game_scenario,
                                           game_scenario, next_turn, 
                                           game_server, true);
   game_server->get_next_player.connect(sigc::mem_fun(next_turn, &NextTurnNetworked::next));
-  game_server->round_ends.connect
-    (sigc::mem_fun(*this, &Driver::on_keep_network_play_going));
+  game_server->round_ends.connect (method(on_keep_network_play_going));
   Playerlist::getInstance()->splayerDead.connect
     (sigc::mem_fun(GameServer::getInstance(), &GameServer::sendKillPlayer));
-  game_lobby_dialog->player_sat_down.connect
-    (sigc::mem_fun(this, &Driver::on_hosted_player_sat_down));
-  game_lobby_dialog->player_stood_up.connect
-    (sigc::mem_fun(this, &Driver::on_hosted_player_stood_up));
+  game_lobby_dialog->player_sat_down.connect (method(on_hosted_player_sat_down));
+  game_lobby_dialog->player_stood_up.connect (method(on_hosted_player_stood_up));
   game_lobby_dialog->player_changed_name.connect
-    (sigc::mem_fun(this, &Driver::on_hosted_player_changed_name));
+    (method(on_hosted_player_changed_name));
   game_lobby_dialog->player_changed_type.connect
-    (sigc::mem_fun(this, &Driver::on_hosted_player_changed_type));
-  game_lobby_dialog->message_sent.connect
-    (sigc::mem_fun(this, &Driver::on_hosted_player_chat));
+    (method(on_hosted_player_changed_type));
+  game_lobby_dialog->message_sent.connect (method(on_hosted_player_chat));
   game_lobby_dialog->game_may_begin.connect
-    (sigc::mem_fun(this, &Driver::on_hosted_player_says_game_may_begin));
+    (method(on_hosted_player_says_game_may_begin));
   game_lobby_dialog->start_network_game.connect
-    (sigc::mem_fun(this, &Driver::start_network_game_requested));
+    (method(start_network_game_requested));
   game_lobby_dialog->show();
   bool response = game_lobby_dialog->run();
   game_lobby_dialog->hide();
@@ -700,6 +792,13 @@ void Driver::on_load_hosted_network_game_requested(GameScenario *game_scenario,
     }
 }
 
+void Driver::on_server_went_away_text()
+{
+  /* same as on_server_went_away, but we're not doing graphics */
+  GameClient::deleteInstance();
+  on_quit_requested ();
+}
+
 void Driver::on_server_went_away()
 {
   upload_heartbeat_conn.disconnect();
@@ -747,23 +846,17 @@ void Driver::on_new_remote_network_game_requested(Glib::ustring host, unsigned s
   if (splash_window)
     splash_window->hide();
   GameClient *game_client = GameClient::getInstance();
-  game_client->game_scenario_received.connect
-    (sigc::mem_fun(this, &Driver::on_game_scenario_downloaded));
-  game_client->client_disconnected.connect
-    (sigc::mem_fun(this, &Driver::on_server_went_away));
-  game_client->client_forcibly_disconnected.connect
-    (sigc::mem_fun(this, &Driver::on_server_went_away));
-  game_client->client_could_not_connect.connect
-    (sigc::mem_fun(this, &Driver::on_client_could_not_connect));
-  recv_conn = game_scenario_received.connect
-    (sigc::bind(sigc::mem_fun(this, &Driver::on_game_scenario_received), p));
+  game_client->game_scenario_received.connect (method(on_game_scenario_downloaded));
+  game_client->client_disconnected.connect (method(on_server_went_away));
+  game_client->client_forcibly_disconnected.connect (method(on_server_went_away));
+  game_client->client_could_not_connect.connect (method(on_client_could_not_connect));
+  recv_conn = game_scenario_received.connect (sigc::bind(method(on_game_scenario_received), p));
   if (download_window)
     delete download_window;
   download_window = new NewNetworkGameDownloadWindow();
   download_window->pulse();
   game_client->start(host, port, p->getId(), p->getNickname());
-  heartbeat_conn = Glib::signal_timeout().connect
-    (sigc::mem_fun(*this, &Driver::heartbeat), 1 * 1000);
+  heartbeat_conn = Glib::signal_timeout().connect (method(heartbeat), 1 * 1000);
 
 }
 
@@ -800,25 +893,19 @@ void Driver::on_game_scenario_received(Glib::ustring path, Profile *p)
   RecentlyPlayedGameList::getInstance()->addNetworkedEntry(game_scenario, p, host, port);
   RecentlyPlayedGameList::getInstance()->save();
 
-  NextTurnNetworked *next_turn = new NextTurnNetworked(game_scenario->getTurnmode(), game_scenario->s_random_turns);
+  NextTurnNetworked *next_turn = new NextTurnNetworked();
   game_client->start_player_turn.connect(sigc::mem_fun(next_turn, &NextTurnNetworked::start_player));
   game_client->round_ends.connect(sigc::mem_fun(next_turn, &NextTurnNetworked::finishRound));
   if (game_lobby_dialog)
     delete game_lobby_dialog;
   game_lobby_dialog = new GameLobbyDialog(*splash_window->get_window(), game_scenario, next_turn, 
 					      GameClient::getInstance(), false);
-  game_lobby_dialog->player_sat_down.connect
-    (sigc::mem_fun(this, &Driver::on_client_player_sat_down));
-  game_lobby_dialog->player_stood_up.connect
-    (sigc::mem_fun(this, &Driver::on_client_player_stood_up));
-  game_lobby_dialog->player_changed_name.connect
-    (sigc::mem_fun(this, &Driver::on_client_player_changed_name));
-  game_lobby_dialog->player_changed_type.connect
-    (sigc::mem_fun(this, &Driver::on_client_player_changed_type));
-  game_lobby_dialog->message_sent.connect
-    (sigc::mem_fun(this, &Driver::on_client_player_chat));
-  game_lobby_dialog->start_network_game.connect
-    (sigc::mem_fun(this, &Driver::start_network_game_requested));
+  game_lobby_dialog->player_sat_down.connect (method(on_client_player_sat_down));
+  game_lobby_dialog->player_stood_up.connect (method(on_client_player_stood_up));
+  game_lobby_dialog->player_changed_name.connect (method(on_client_player_changed_name));
+  game_lobby_dialog->player_changed_type.connect (method(on_client_player_changed_type));
+  game_lobby_dialog->message_sent.connect (method(on_client_player_chat));
+  game_lobby_dialog->start_network_game.connect (method(start_network_game_requested));
   game_lobby_dialog->show();
   bool response = game_lobby_dialog->run();
 
@@ -839,8 +926,15 @@ void Driver::on_game_scenario_downloaded(Glib::ustring path)
   //...without stopping the game client from getting more messages
 }
 
-void Driver::on_new_game_requested(GameParameters g)
+void Driver::on_new_game_requested(GameParameters g, GamePreferencesDialog *gpd)
 {
+  if (gpd)
+    {
+      start_game_progress_tick.connect (sigc::mem_fun (gpd, &GamePreferencesDialog::tick_progress));
+      start_game_progress_finish.connect (sigc::mem_fun (gpd, &GamePreferencesDialog::finish_progress));
+    }
+
+    start_game_progress_tick.emit ();
     GameScenario *game_scenario = create_new_scenario(g, GameScenario::HOTSEAT);
 
     if (game_scenario == NULL)
@@ -868,12 +962,13 @@ void Driver::on_new_game_requested(GameParameters g)
 	splash_window->show();
 	return;
       }
+    start_game_progress_tick.emit ();
 
+    start_game_progress_finish.emit ();
     if (splash_window)
 	splash_window->hide();
 
-    NextTurn *next_turn = new NextTurnHotseat(game_scenario->getTurnmode(),
-					      game_scenario->s_random_turns);
+    NextTurn *next_turn = new NextTurnHotseat();
     init_game_window();
     
     game_window->show();
@@ -898,8 +993,7 @@ void Driver::on_load_requested(Glib::ustring filename)
 
         game_window->show();
         game_window->load_game
-          (game_scenario, new NextTurnHotseat(game_scenario->getTurnmode(),
-                                              game_scenario->s_random_turns));
+          (game_scenario, new NextTurnHotseat());
       }
     else if (game_scenario->getPlayMode() == GameScenario::NETWORKED)
       {
@@ -907,10 +1001,14 @@ void Driver::on_load_requested(Glib::ustring filename)
         bool retval = nngd.run();
         nngd.hide();
         if (retval == false)
-          splash_window->show();
+          {
+            delete game_scenario;
+            GameScenario::cleanup ();
+            splash_window->show();
+          }
         else
           {
-            on_load_hosted_network_game_requested(game_scenario, LORDSAWAR_PORT,
+            on_load_hosted_network_game_requested(game_scenario, get_port(),
 						  nngd.getProfile(), 
                                                   nngd.isAdvertised(),
                                                   nngd.isRemotelyHosted());
@@ -918,6 +1016,35 @@ void Driver::on_load_requested(Glib::ustring filename)
       }
 }
 
+void Driver::on_editor_requested(Glib::ustring filename)
+{
+  if (splash_window)
+    splash_window->hide();
+  if (editor_window)
+    delete editor_window;
+
+  Snd::getInstance()->halt();
+  Snd::getInstance()->disableBackground();
+  EditorSplashWindow d;
+  d.run();
+  d.hide();
+  editor_window = new MainWindow (filename);
+  editor_window->editor_quit.connect (method(on_editor_quit));
+  editor_window->show();
+  editor_window->init();
+}
+
+void Driver::on_editor_quit ()
+{
+  if (editor_window)
+    delete editor_window;
+  editor_window = NULL;
+  if (Main::instance().start_editor == false)
+    splash_window->show();
+  else
+    Main::instance().stop_main_loop();
+}
+
 void Driver::on_quit_requested()
 {
     if (splash_window)
@@ -926,6 +1053,7 @@ void Driver::on_quit_requested()
     if (game_window)
 	game_window->hide();
 
+    Snd::deleteInstance();
     Main::instance().stop_main_loop();
 }
 
@@ -936,6 +1064,12 @@ void Driver::on_game_ended()
       delete game_lobby_dialog;
       game_lobby_dialog = NULL;
     }
+  if (quick_help_window)
+    {
+      quick_help_window->hide ();
+      delete quick_help_window;
+      quick_help_window = NULL;
+    }
 
   if (game_window)
     {
@@ -967,6 +1101,7 @@ void Driver::on_game_ended()
   Tilesetlist::deleteInstance();
   Citysetlist::deleteInstance();
   HeroTemplates::deleteInstance();
+  ScenarioMedia::deleteInstance();
 
   splash_window->show();
 }
@@ -983,19 +1118,17 @@ void Driver::init_game_window()
     delete game_window;
   game_window = new GameWindow;
 
-  game_window->game_ended.connect
-    (sigc::mem_fun(*this, &Driver::on_game_ended));
-  game_window->game_ended_start_new.connect
-    (sigc::mem_fun(*this, &Driver::on_game_ended_and_start_new));
-  game_window->show_lobby.connect
-    (sigc::mem_fun(*this, &Driver::on_show_lobby_requested));
-  game_window->quit_requested.connect
-    (sigc::mem_fun(*this, &Driver::on_quit_requested));
+  game_window->game_ended.connect (method(on_game_ended));
+  game_window->game_ended_start_new.connect (method(on_game_ended_and_start_new));
+  game_window->show_lobby.connect (method(on_show_lobby_requested));
+  game_window->show_quick_help.connect (method(on_quick_help_requested));
+  game_window->quit_requested.connect (method(on_quit_requested));
   game_window->load_hosted_network_game.connect
-    (sigc::mem_fun(*this, &Driver::on_game_ended_and_load_network_game));
+    (method(on_game_ended_and_load_network_game));
 
   //make the width+height suitable for the screen size.
-  Glib::RefPtr<Gdk::Screen> screen = Gdk::Display::get_default()->get_default_screen();
+  Glib::RefPtr<Gdk::Screen> screen =
+    Gdk::Display::get_default()->get_default_screen();
   guint32 screen_height = screen->get_height();
   guint32 height = 450;
   if (screen_height <= 600)
@@ -1074,12 +1207,14 @@ void Driver::stress_test()
   g.tile_theme = "default";
   g.shield_theme = "default";
   g.city_theme = "default";
-  g.process_armies = GameParameters::PROCESS_ARMIES_AT_PLAYERS_TURN;
-  g.difficulty = GameScenario::calculate_difficulty_rating(g);
   g.cities_can_produce_allies = false;
   g.cusp_of_war = false;
   g.see_opponents_stacks = true;
   g.see_opponents_production = true;
+  g.vectoring_mode = GameParameters::VECTORING_ALWAYS_TWO_TURNS;
+  g.build_production_mode = GameParameters::BUILD_PRODUCTION_ALWAYS;
+  g.sacking_mode = GameParameters::SACKING_ALWAYS;
+  g.difficulty = GameScenario::calculate_difficulty_rating(g);
       
   bool broken = false;
   Glib::ustring path;
@@ -1091,12 +1226,11 @@ void Driver::stress_test()
   if (broken)
     return;
 
+  Configuration::s_autosave_policy = Configuration::NO_SAVING;
   NextTurnHotseat *nextTurn;
-  nextTurn = new NextTurnHotseat(game_scenario->getTurnmode(),
-				 game_scenario->s_random_turns);
+  nextTurn = new NextTurnHotseat();
     
-  nextTurn->snextRound.connect
-    (sigc::mem_fun(this, &Driver::stressTestNextRound));
+  nextTurn->snextRound.connect (method(stressTestNextRound));
   if (game_scenario->getRound() == 0)
     {
       Playerlist::getInstance()->syncPlayers(g.players);
@@ -1114,33 +1248,28 @@ void Driver::stress_test()
   fflush(stdout);
   printf("winner type: %s\n", Player::playerTypeToString(Player::Type(Playerlist::getInstance()->getFirstLiving()->getType())).c_str());
   Glib::ustring s = 
-    File::getTempFile(Glib::get_tmp_dir(), 
+    File::getTempFile(File::getCacheDir(),
                       String::ucompose ("run-seed-%1", 
                                         Main::instance().random_number_seed));
   game_scenario->saveGame(s + SAVE_EXT);
 
+  //nextTurn and game_scenario get deleted in game.
 }
-	
+
 void Driver::lordsawaromatic(Glib::ustring host, unsigned short port, Player::Type type, int num_players)
 {
   GameClient *game_client = GameClient::getInstance();
-  game_client->game_scenario_received.connect
-    (sigc::mem_fun(this, &Driver::on_game_scenario_downloaded));
-  game_client->client_disconnected.connect
-    (sigc::mem_fun(this, &Driver::on_server_went_away));
-  game_client->client_forcibly_disconnected.connect
-    (sigc::mem_fun(this, &Driver::on_server_went_away));
-  game_client->client_could_not_connect.connect
-    (sigc::mem_fun(this, &Driver::on_client_could_not_connect));
-  game_scenario_received.connect
-    (sigc::mem_fun(this, &Driver::on_game_scenario_received_for_robots));
+  game_client->game_scenario_received.connect(method(on_game_scenario_downloaded));
+  game_client->client_disconnected.connect (method(on_server_went_away_text));
+  game_client->client_forcibly_disconnected.connect (method(on_server_went_away_text));
+  game_client->client_could_not_connect.connect (method(on_client_could_not_connect));
+  game_scenario_received.connect (method(on_game_scenario_received_for_robots));
   game_client->setNickname("robot");
   Glib::ustring id = "";
   if (Profilelist::getInstance()->empty() == false)
     id = Profilelist::getInstance()->front()->getId();
   game_client->start(host, port, id, "robot");
-  heartbeat_conn = Glib::signal_timeout().connect
-    (sigc::mem_fun(*this, &Driver::heartbeat), 1 * 1000);
+  heartbeat_conn = Glib::signal_timeout().connect (method(heartbeat), 1 * 1000);
   robot_player_type = type;
   number_of_robots = num_players;
 }
@@ -1152,7 +1281,7 @@ void Driver::on_game_scenario_received_for_robots(Glib::ustring path)
   if (!game_scenario)
     return;
   GameClient *game_client = GameClient::getInstance();
-  NextTurnNetworked *next_turn = new NextTurnNetworked(game_scenario->getTurnmode(), game_scenario->s_random_turns);
+  NextTurnNetworked *next_turn = new NextTurnNetworked();
   game_client->start_player_turn.connect(sigc::mem_fun(next_turn, &NextTurnNetworked::start_player));
   game_client->round_ends.connect(sigc::mem_fun(next_turn, &NextTurnNetworked::finishRound));
 
@@ -1200,17 +1329,32 @@ void Driver::on_game_scenario_received_for_robots(Glib::ustring path)
     if (Player::Type((*it)->getType()) == robot_player_type)
       GameClient::getInstance()->listenForLocalEvents(*it);
 
-  Game *game = new Game(game_scenario, next_turn);
+  Configuration::s_autosave_policy = Configuration::NO_SAVING;
+  Game *game = new Game(game_scenario, next_turn, true);
   game->get_smallmap().set_slide_speed(0);
   game_client->request_seat_manifest();
 }
 
+void Driver::on_game_over_for_headless_server (Player *p, GameScenario *g)
+{
+  printf("all done!  player '%s' was the winner on turn %d\n", p->getName().c_str(), g->getRound());
+  GameServer::deleteInstance();
+  exit(0);
+}
+
 void Driver::on_show_lobby_requested()
 {
   if (game_lobby_dialog)
     game_lobby_dialog->show();
 }
     
+void Driver::on_quick_help_requested()
+{
+  if (!quick_help_window)
+    quick_help_window = new QuickHelpWindow ();
+  quick_help_window->show();
+}
+    
 void Driver::start_network_game_requested(GameScenario *game_scenario, NextTurnNetworked *next_turn)
 {
   if (game_window)
@@ -1250,7 +1394,6 @@ void Driver::on_keep_network_play_going()
           //we are waiting for a player to finish their turn
           //or they are unavailable.
           break;
-          ;
         }
     }
 }
@@ -1275,7 +1418,7 @@ void Driver::unadvertise_game(Glib::ustring scenario_id, Profile *p)
 {
   GamelistClient *gsc = GamelistClient ::getInstance();
   gsc->client_connected.connect
-    (sigc::bind(sigc::mem_fun(*this, &Driver::on_connected_to_gamelist_server_for_advertising_removal), scenario_id));
+    (sigc::bind(method(on_connected_to_gamelist_server_for_advertising_removal), scenario_id));
   gsc->start(Configuration::s_gamelist_server_hostname,
              Configuration::s_gamelist_server_port, p);
 }
@@ -1284,7 +1427,7 @@ void Driver::on_connected_to_gamelist_server_for_advertising_removal(Glib::ustri
 {
   GamelistClient *gsc = GamelistClient::getInstance();
   gsc->received_advertising_removal_response.connect
-    (sigc::hide(sigc::hide(sigc::mem_fun(*this, &Driver::on_advertising_removal_response_received))));
+    (sigc::hide(sigc::hide(method(on_advertising_removal_response_received))));
   gsc->request_advertising_removal(scenario_id);
 }
 
@@ -1298,3 +1441,11 @@ void Driver::on_advertised_game_round_ends(GameScenario *game_scenario, Profile
 {
   advertise_game(game_scenario, p);
 }
+
+guint32 Driver::get_port ()
+{
+  guint32 port = LORDSAWAR_PORT;
+  if (Main::instance().port)
+    port = Main::instance().port;
+  return port;
+}
