@@ -1,5 +1,6 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2011, 2014, 2015 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2011, 2014, 2015, 2016, 2017,
+//  2020  Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -16,6 +17,7 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
+#pragma once
 #ifndef GUI_DRIVER_H
 #define GUI_DRIVER_H
 
@@ -25,8 +27,11 @@
 #include "splash-window.h"
 #include "game-window.h"
 #include "game-lobby-dialog.h"
+#include "quick-help-window.h"
 #include "new-network-game-download-window.h"
 #include "game-parameters.h"
+#include "../editor/main-window.h"
+class GamePreferencesDialog;
 
 class Profile;
 // takes care of setting up the splash window and the game window, the
@@ -35,7 +40,7 @@ class Profile;
 class Driver: public sigc::trackable
 {
  public:
-    Driver(Glib::ustring load_filename);
+    Driver(bool start_editor, Glib::ustring load_filename);
     ~Driver();
 
     void run();
@@ -43,8 +48,10 @@ class Driver: public sigc::trackable
  private:
     GameWindow* game_window;
     GameLobbyDialog* game_lobby_dialog;
+    QuickHelpWindow* quick_help_window;
     SplashWindow* splash_window;
     NewNetworkGameDownloadWindow* download_window;
+    MainWindow *editor_window;
     Glib::ustring d_load_filename;
     sigc::connection heartbeat_conn;
     sigc::connection upload_heartbeat_conn;
@@ -55,8 +62,10 @@ class Driver: public sigc::trackable
     Glib::ustring game_scenario_downloaded;
     sigc::signal<void, Glib::ustring> game_scenario_received;
     sigc::signal<void, Player*> player_replaced;
+    sigc::signal<void> start_game_progress_tick;
+    sigc::signal<void> start_game_progress_finish;
 
-    void on_new_game_requested(GameParameters g);
+    void on_new_game_requested(GameParameters g, GamePreferencesDialog *gpd);
     void on_new_remote_network_game_requested(Glib::ustring host, unsigned short port, Profile *p);
     void on_new_hosted_network_game_requested(GameParameters g, int port, Profile *p, bool advertised, bool remotely_hosted);
     void on_load_hosted_network_game_requested(GameScenario *game_scenario, int port, Profile *p, bool advertised, bool remotely_hosted);
@@ -65,6 +74,8 @@ class Driver: public sigc::trackable
     void on_game_scenario_downloaded(Glib::ustring filename);
     void on_game_scenario_received(Glib::ustring path, Profile *p);
     void on_load_requested(Glib::ustring filename);
+    void on_editor_requested(Glib::ustring filename = "");
+    void on_editor_quit ();
     void on_quit_requested();
 
     void on_game_ended();
@@ -82,6 +93,7 @@ class Driver: public sigc::trackable
     void on_client_player_changed_name(Player *player, Glib::ustring name);
     void on_client_player_changed_type(Player *player, int type);
     void on_server_went_away();
+    void on_server_went_away_text();
     void on_client_could_not_connect();
 
     GameScenario *new_game(GameParameters g);
@@ -91,6 +103,7 @@ class Driver: public sigc::trackable
 
     void lordsawaromatic(Glib::ustring host, unsigned short port, Player::Type type, int num_players);
     void on_game_scenario_received_for_robots(Glib::ustring path);
+    void on_game_over_for_headless_server(Player *p, GameScenario *g);
   
 
     bool heartbeat();
@@ -101,6 +114,7 @@ class Driver: public sigc::trackable
     void on_hosted_player_says_game_may_begin();
 
     void on_show_lobby_requested();
+    void on_quick_help_requested();
 
     void start_network_game_requested(GameScenario *game_scenario,
 				      NextTurnNetworked *next_turn);
@@ -131,6 +145,7 @@ class Driver: public sigc::trackable
     void on_got_game_host_response(Glib::ustring err, GameScenario *game_scenario);
     void on_remote_game_hosted(guint32 port, Glib::ustring err);
     void on_could_not_connect_to_gamehost_server();
+    guint32 get_port ();
 };
 
 
