@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2011, 2012, 2014, 2015 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2011, 2012, 2014, 2015, 2020 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 
 #include <config.h>
 
-#include <assert.h>
 #include <sigc++/functors/mem_fun.h>
 #include <gtkmm.h>
 
@@ -32,37 +31,41 @@
 #include "player.h"
 #include "game-parameters.h"
 #include "Configuration.h"
+#include "font-size.h"
 
 static bool inhibit_difficulty_combobox = false;
 
+#define method(x) sigc::mem_fun(*this, &GamePreferencesDialog::x)
+
 void GamePreferencesDialog::init(Glib::ustring filename)
 {
   d_filename = filename;
+  d_max_players = 0;
+  d_inhibit_change = false;
   bool broken = false;
-    xml->get_widget("dialog-vbox1", dialog_vbox);
-    xml->get_widget("start_game_button", start_game_button);
-    xml->get_widget("difficulty_label", difficulty_label);
-    xml->get_widget("difficulty_combobox", difficulty_combobox);
+  xml->get_widget("dialog-vbox1", dialog_vbox);
+  xml->get_widget("start_game_button", start_game_button);
+  xml->get_widget("difficulty_label", difficulty_label);
+  xml->get_widget("difficulty_combobox", difficulty_combobox);
 
-    xml->get_widget("players_vbox", players_vbox);
-    xml->get_widget("game_name_label", game_name_label);
-    xml->get_widget("game_name_entry", game_name_entry);
+  xml->get_widget("players_vbox", players_vbox);
+  xml->get_widget("game_name_label", game_name_label);
+  xml->get_widget("game_name_entry", game_name_entry);
+  xml->get_widget("num_players_spinbutton", num_players_spinbutton);
+  num_players_spinbutton->signal_changed().connect(method(on_num_players_changed));
+  num_players_spinbutton->signal_insert_text().connect
+    (sigc::hide(sigc::hide(method(on_num_players_text_changed))));
 
-    difficulty_combobox->set_active(CUSTOM);
-    difficulty_combobox->signal_changed().connect(
-	sigc::mem_fun(*this, &GamePreferencesDialog::on_difficulty_changed));
+  difficulty_combobox->set_active(CUSTOM);
+  difficulty_combobox->signal_changed().connect(method(on_difficulty_changed));
 
-    start_game_button->signal_clicked().connect
-      (sigc::mem_fun(*this, &GamePreferencesDialog::on_start_game_clicked));
+  start_game_button->signal_clicked().connect (method(on_start_game_clicked));
 
-    xml->get_widget("edit_options_button", edit_options_button);
-    edit_options_button->signal_clicked().connect(
-	sigc::mem_fun(*this, &GamePreferencesDialog::on_edit_options_clicked));
+  xml->get_widget("edit_options_button", edit_options_button);
+  edit_options_button->signal_clicked().connect (method(on_edit_options_clicked));
 
   game_options_dialog = new GameOptionsDialog(*dialog, false);
-  game_options_dialog->difficulty_option_changed.connect(
-	sigc::mem_fun(*this, 
-		      &GamePreferencesDialog::update_difficulty_rating));
+  game_options_dialog->difficulty_option_changed.connect(method(update_difficulty_rating));
   GameParameters load_map_parameters;
   load_map_parameters = GameScenario::loadGameParameters(d_filename,
 							 broken);
@@ -99,17 +102,24 @@ void GamePreferencesDialog::init(Glib::ustring filename)
       (*c)->set_active((*i).type);
       (*e)->set_sensitive(true);
       (*e)->set_text((*i).name);
+      d_max_players++;
     }
+  num_players_spinbutton->set_range (0, d_max_players);
+  num_players_spinbutton->set_value (d_max_players);
   start_game_button->property_can_focus() = true;
   start_game_button->property_has_focus() = true;
   start_game_button->get_receives_default();
-      
+
   //load the game options from the config file.
   GameScenarioOptions::s_see_opponents_stacks = 
     Configuration::s_see_opponents_stacks;
   GameScenarioOptions::s_see_opponents_production = 
     Configuration::s_see_opponents_production;
   GameScenarioOptions::s_play_with_quests = Configuration::s_play_with_quests;
+  GameScenarioOptions::s_vectoring_mode = Configuration::s_vectoring_mode;
+  GameScenarioOptions::s_sacking_mode = Configuration::s_sacking_mode;
+  GameScenarioOptions::s_build_production_mode =
+    Configuration::s_build_production_mode;
   GameScenarioOptions::s_hidden_map = Configuration::s_hidden_map;
   GameScenarioOptions::s_neutral_cities = Configuration::s_neutral_cities;
   GameScenarioOptions::s_razing_cities = Configuration::s_razing_cities;
@@ -133,27 +143,14 @@ GamePreferencesDialog::GamePreferencesDialog(Gtk::Window &parent, Glib::ustring
       delete game_name_label;
       delete game_name_entry;
     }
-
-}
-
-GamePreferencesDialog::~GamePreferencesDialog()
-{
-  delete game_options_dialog;
-}
-
-void GamePreferencesDialog::hide()
-{
-  dialog->hide();
 }
 
 bool GamePreferencesDialog::run(Glib::ustring nickname)
 {
-
   dialog->show_all();
   if (mode == GameScenario::NETWORKED)
     {
-      Glib::ustring text = nickname;
-      text += "'s game";
+      Glib::ustring text = String::ucompose (_("%1's game"), nickname);
       game_name_entry->set_text(text);
     }
   update_shields();
@@ -169,17 +166,15 @@ void GamePreferencesDialog::add_player(GameParameters::Player::Type type,
 {
   //okay, add a new hbox, with a combo and an entry in it
   //add it to players_vbox
-  Gtk::HBox *player_hbox = new Gtk::HBox();
+  Gtk::Box *player_hbox = new Gtk::Box(Gtk::ORIENTATION_HORIZONTAL);
   Gtk::ComboBoxText *player_type = new Gtk::ComboBoxText();
   player_type->append(HUMAN_PLAYER_TYPE);
   player_type->append(EASY_PLAYER_TYPE);
   player_type->append(HARD_PLAYER_TYPE);
   player_type->append(NO_PLAYER_TYPE);
-  player_type->signal_changed().connect
-    (sigc::mem_fun(this, &GamePreferencesDialog::on_player_type_changed));
+  player_type->signal_changed().connect (method(on_player_type_changed));
   Gtk::Entry *player_name = new Gtk::Entry();
-  player_name->signal_changed().connect
-    (sigc::mem_fun(this, &GamePreferencesDialog::on_player_name_changed));
+  player_name->signal_changed().connect (method(on_player_name_changed));
   player_name->set_text(name);
 
   if (type == GameParameters::Player::HUMAN)
@@ -235,9 +230,11 @@ void GamePreferencesDialog::update_shields()
     {
       Gtk::Image *player_shield = new Gtk::Image ();
       player_shield->property_pixbuf() = 
-        ImageCache::getInstance()->getShieldPic(d_shieldset, 2, i)->to_pixbuf();
+        ImageCache::getInstance()->getShieldPic
+        (d_shieldset, 2, i, false,
+         FontSize::getInstance ()->get_height ())->to_pixbuf();
       player_shields.push_back(player_shield);
-      Gtk::HBox *player_hbox = static_cast<Gtk::HBox*>(list[i+1]);
+      Gtk::Box *player_hbox = static_cast<Gtk::Box*>(list[i+1]);
       player_hbox->pack_start(*manage(player_shield), Gtk::PACK_SHRINK, 10);
       player_hbox->reorder_child(*player_shield, 0);
       player_hbox->show_all();
@@ -248,6 +245,18 @@ void GamePreferencesDialog::update_shields()
 
 void GamePreferencesDialog::on_player_type_changed()
 {
+  std::list<Gtk::ComboBoxText*>::iterator c = player_types.begin();
+  int count = 0;
+  for (; c != player_types.end(); c++)
+    {
+      if ((*c)->property_sensitive () == false)
+        continue;
+      if ((*c)->get_active_row_number() != 3) //if not OFF
+        count++;
+    }
+  d_inhibit_change = true;
+  num_players_spinbutton->set_value (count);
+  d_inhibit_change = false;
   update_buttons();
   update_difficulty_rating();
 }
@@ -313,6 +322,9 @@ void GamePreferencesDialog::update_difficulty_rating()
   g.see_opponents_stacks = GameScenarioOptions::s_see_opponents_stacks;
   g.see_opponents_production = GameScenarioOptions::s_see_opponents_production;
   g.play_with_quests = GameScenarioOptions::s_play_with_quests;
+  g.vectoring_mode = GameScenarioOptions::s_vectoring_mode;
+  g.sacking_mode = GameScenarioOptions::s_sacking_mode;
+  g.build_production_mode = GameScenarioOptions::s_build_production_mode;
   g.hidden_map = GameScenarioOptions::s_hidden_map;
   g.neutral_cities = GameScenarioOptions::s_neutral_cities;
   g.razing_cities = GameScenarioOptions::s_razing_cities;
@@ -331,6 +343,21 @@ void GamePreferencesDialog::update_difficulty_rating()
 
 void GamePreferencesDialog::on_start_game_clicked()
 {
+  progress_treeview = Gtk::manage (new Gtk::TreeView ());
+  progress_treeview->property_headers_visible () = false;
+  progress_liststore = Gtk::ListStore::create(progress_columns);
+  progress_treeview->set_model (progress_liststore);
+  row = *(progress_liststore->append());
+  auto cell = Gtk::make_managed<Gtk::CellRendererProgress>();
+  cell->property_text () = "";
+  int cols_count = progress_treeview->append_column ("progress", *cell);
+  auto pColumn = progress_treeview->get_column(cols_count -1);
+  if (pColumn)
+    pColumn->add_attribute(cell->property_value (), progress_columns.perc);
+
+  dialog_vbox->pack_end (*progress_treeview, true, true);
+  dialog_vbox->show_all ();
+  while (g_main_context_iteration(NULL, FALSE)); //doEvents
   dialog_vbox->set_sensitive(false);
   // read out the values in the widgets
   GameParameters g;
@@ -350,11 +377,12 @@ void GamePreferencesDialog::on_start_game_clicked()
       g.players.push_back(p);
     }
 
-  g.process_armies = GameParameters::PROCESS_ARMIES_AT_PLAYERS_TURN;
-
   g.see_opponents_stacks = GameScenarioOptions::s_see_opponents_stacks;
   g.see_opponents_production = GameScenarioOptions::s_see_opponents_production;
   g.play_with_quests = GameScenarioOptions::s_play_with_quests;
+  g.vectoring_mode = GameScenarioOptions::s_vectoring_mode;
+  g.sacking_mode = GameScenarioOptions::s_sacking_mode;
+  g.build_production_mode = GameScenarioOptions::s_build_production_mode;
   g.hidden_map = GameScenarioOptions::s_hidden_map;
   g.neutral_cities = GameScenarioOptions::s_neutral_cities;
   g.razing_cities = GameScenarioOptions::s_razing_cities;
@@ -372,8 +400,8 @@ void GamePreferencesDialog::on_start_game_clicked()
     g.name = String::utrim(game_name_entry->get_text());
 
   // and call callback
-  dialog->hide();
   game_started.emit(g);
+  dialog->hide();
 }
 
 void GamePreferencesDialog::on_difficulty_changed()
@@ -439,7 +467,7 @@ void GamePreferencesDialog::on_difficulty_changed()
 	  std::list<Gtk::ComboBoxText*>::iterator c = player_types.begin();
 	  for (; c != player_types.end(); c++)
 	    {
-	      if ((*c)->get_active_row_number() != 3)
+	      if ((*c)->get_active_row_number() != 3) //if OFF
 		(*c)->set_active (type_num);
 	    }
 	}
@@ -498,3 +526,66 @@ bool GamePreferencesDialog::is_greatest()
 	  GameScenarioOptions::s_diplomacy == true &&
 	  GameScenarioOptions::s_cusp_of_war == true);
 }
+
+void GamePreferencesDialog::on_num_players_text_changed()
+{
+  num_players_spinbutton->set_value(atoi(num_players_spinbutton->get_text().c_str()));
+  on_num_players_changed();
+}
+
+void GamePreferencesDialog::on_num_players_changed()
+{
+  if (d_inhibit_change)
+    return;
+  if (num_players_spinbutton->get_value() < 0)
+    {
+      num_players_spinbutton->set_value(0);
+      return;
+    }
+  else if (num_players_spinbutton->get_value() > d_max_players)
+    {
+      num_players_spinbutton->set_value(d_max_players);
+      return;
+    }
+  std::list<Gtk::ComboBoxText*>::iterator c = player_types.begin();
+  int count = num_players_spinbutton->get_value ();
+  for (; c != player_types.end(); c++)
+    {
+      if ((*c)->property_sensitive () == false)
+        continue;
+      if (count)
+        {
+          if ((*c)->get_active_row_number() == 3) //if OFF
+            (*c)->set_active(GameParameters::Player::HUMAN);
+          count--;
+        }
+      else
+        (*c)->set_active(GameParameters::Player::OFF);
+    }
+}
+
+void GamePreferencesDialog::tick_progress ()
+{
+  if (!progress_treeview)
+    return;
+  if (row[progress_columns.perc] < 98)
+    {
+      row[progress_columns.perc] = row[progress_columns.perc] + 3;
+      while (g_main_context_iteration(NULL, FALSE)); //doEvents
+    }
+}
+
+void GamePreferencesDialog::finish_progress ()
+{
+  if (!progress_treeview)
+    return;
+  //finish off the progressbar
+  while (row[progress_columns.perc] < 100)
+    {
+      row[progress_columns.perc] = row[progress_columns.perc] + 1;
+      while (g_main_context_iteration(NULL, FALSE)); //doEvents
+      Glib::usleep (10000);
+    }
+  row[progress_columns.perc] = 100;
+  dialog->hide ();
+}
