@@ -1,5 +1,6 @@
 //  Copyright (C) 2007, 2008 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2011, 2012, 2014, 2015 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2011, 2012, 2014, 2015, 2016, 2017,
+//  2020 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -16,6 +17,7 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
+#pragma once
 #ifndef GUI_GAME_WINDOW_H
 #define GUI_GAME_WINDOW_H
 
@@ -57,6 +59,7 @@ class NextTurn;
 class LocationBox;
 class StatusBox;
 class Profile;
+class Keeper;
 
 /** The main window in which all the game action is going on.
   *
@@ -69,11 +72,13 @@ class Profile;
 class GameWindow: public sigc::trackable
 {
  public:
+     static double minimum_zoom_scale;
+     static double maximum_zoom_scale;
     GameWindow();
     ~GameWindow();
 
     void show();
-    void hide();
+    void hide() {window->hide();};
 
     // initialize the big map widget 
     void init(int width, int height);
@@ -96,6 +101,7 @@ class GameWindow: public sigc::trackable
     sigc::signal<void> game_ended_start_new;
     
     sigc::signal<void> show_lobby;
+    sigc::signal<void> show_quick_help;
 
     sigc::signal<void> quit_requested;
 
@@ -158,11 +164,15 @@ class GameWindow: public sigc::trackable
     Gtk::MenuItem *winning_report_menuitem;
     Gtk::MenuItem *quests_menuitem;
     Gtk::MenuItem *preferences_menuitem;
+    Gtk::MenuItem *zoom_in_menuitem;
+    Gtk::MenuItem *zoom_out_menuitem;
+    Gtk::MenuItem *best_fit_menuitem;
+    Gtk::MenuItem *quick_help_menuitem;
     Gtk::Image *smallmap_image;
     Gtk::Image *bigmap_image;
 
     Gtk::Label *turn_label;
-    Gtk::HBox *turn_hbox;
+    Gtk::Box *turn_hbox;
     Gtk::Image *shield_image[MAX_PLAYERS];
     
     // the button control panel
@@ -176,13 +186,16 @@ class GameWindow: public sigc::trackable
     stack_buttons_type stack_buttons;
     Gtk::EventBox *map_eventbox;
     Gtk::EventBox *bigmap_eventbox;
-    Gtk::HBox *status_box_container;
+    Gtk::Box *status_box_container;
     StatusBox *status_box;
 
     Glib::ustring current_save_filename;
 
     Game* game;
     GameButtonBox *game_button_box;
+    Gtk::Label *pos_label;
+
+    GdkEventButton *button_event;
     
     bool on_delete_event();
 
@@ -221,6 +234,10 @@ class GameWindow: public sigc::trackable
 
     void on_fullscreen_activated();
     void on_preferences_activated();
+    void on_zoom_in_activated();
+    void on_zoom_out_activated();
+    void on_best_fit_activated();
+    void zoom (double scale);
     void on_group_ungroup_activated();
     
     void on_fight_order_activated();
@@ -238,6 +255,7 @@ class GameWindow: public sigc::trackable
     void on_triumphs_activated();
     void on_help_about_activated();
     void on_online_help_activated();
+    void on_quick_help_activated();
 
     void on_message_requested(Glib::ustring msg);
     
@@ -253,7 +271,7 @@ class GameWindow: public sigc::trackable
     void on_bigmap_cursor_changed(ImageCache::CursorType cursor);
     void on_bigmap_changed(Cairo::RefPtr<Cairo::Surface> map);
     void on_stack_info_changed(Stack *s);
-    void on_bigmap_tip_changed(Glib::ustring tip, MapTipPosition pos);
+    void on_bigmap_tip_changed(Glib::ustring tip, MapTipPosition pos, bool timeout);
     void on_stack_tip_changed(StackTile *stile, MapTipPosition pos);
     void on_city_tip_changed(City *city, MapTipPosition pos);
     void on_ruin_searched(Ruin *ruin, Stack *s, Reward *reward);
@@ -261,7 +279,7 @@ class GameWindow: public sigc::trackable
     void on_ruin_rewarded(Reward_Ruin *reward);
     void on_fight_started(LocationBox box, Fight &fight);
     void on_abbreviated_fight_started(LocationBox box);
-    void on_ruinfight_started(Stack *attackers, Stack *defenders);
+    void on_ruinfight_started(Stack *attackers, Keeper *keeper);
     void on_ruinfight_finished(Fight::Result result);
     bool on_hero_offers_service(Player *player, HeroProto *hero, City *city, int gold);
     bool on_enemy_offers_surrender(int numEnemies);
@@ -303,15 +321,18 @@ class GameWindow: public sigc::trackable
     Player *on_select_item_victim_player();
     City *on_select_city_to_use_item_on(SelectCityMap::Type type);
     bool on_bigmap_scrolled(GdkEventScroll* event);
+    void on_popup_stack_menu (Stack *stack);
+    void on_pointing_at_new_tile(Vector<int> tile);
 
     // quest manager callbacks
     void on_quest_completed(Quest *quest, Reward *reward);
     void on_quest_expired(Quest *quest);
     
     // helpers
-    void show_map_tip(Glib::ustring msg, MapTipPosition pos);
+    void get_default_magnifying_glass_hotspot (int *x, int *y);
+    void show_map_tip(Glib::ustring msg, MapTipPosition pos, bool timeout = false);
     void on_city_looted(City *city, int gold);
-    void hide_map_tip();
+    bool hide_map_tip();
     void show_city_production_report (bool destitute);
 
     bool setup_game(GameScenario *game_scenario, NextTurn *nextTurn);
@@ -325,8 +346,10 @@ class GameWindow: public sigc::trackable
 
     Player *game_winner;
     void give_some_cheese(Player *game_winner);
-    void on_ui_form_factor_changed();
 
+    bool on_window_state_event (GdkEventWindowState *e);
+    bool on_configure_event (GdkEventConfigure *e);
+    void set_default_bigmap_zoom ();
 public:
     bool d_quick_fights; //do we speed up fights for this player's turn?
     Glib::ustring stop_action; //hackhackhack
@@ -335,6 +358,9 @@ public:
     std::list<Hero*> d_heroes;
     Glib::ustring d_player_name;
     Glib::ustring d_load_filename;
+    Gtk::Allocation last_box;
+    Gtk::Allocation unmaximized_box;
+    sigc::connection map_tip_timer;
 };
 
 #endif
