@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2012, 2014, 2015 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2012, 2014, 2015, 2020 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -26,43 +26,46 @@
 #include "ucompose.hpp"
 #include "defs.h"
 #include "File.h"
-#include "GameScenario.h"
+#include "scenario-list.h"
+#include "timed-message-dialog.h"
+
+#define method(x) sigc::mem_fun(*this, &LoadScenarioDialog::x)
 
 LoadScenarioDialog::LoadScenarioDialog(Gtk::Window &parent)
- : LwDialog(parent, "load-scenario-dialog.ui")
+ : LwDialog(parent, "load-scenario-dialog.ui"),
+    name_column("", name_renderer)
 {
   xml->get_widget("description_textview", description_textview);
   xml->get_widget("load_button", load_button);
   xml->get_widget("num_players_label", num_players_label);
   xml->get_widget("num_cities_label", num_cities_label);
+  xml->get_widget("dialog-vbox", dialog_vbox);
 
   scenarios_list = Gtk::ListStore::create(scenarios_columns);
   xml->get_widget("treeview", scenarios_treeview);
   scenarios_treeview->set_model(scenarios_list);
-  scenarios_treeview->append_column("", scenarios_columns.name);
 
   xml->get_widget("add_scenario_button", add_scenario_button);
   add_scenario_button->signal_clicked().connect
-    (sigc::mem_fun(*this, &LoadScenarioDialog::on_add_scenario_clicked));
+    (method(on_add_scenario_clicked));
   xml->get_widget("remove_scenario_button", remove_scenario_button);
   remove_scenario_button->signal_clicked().connect
-    (sigc::mem_fun(*this, &LoadScenarioDialog::on_remove_scenario_clicked));
+    (method(on_remove_scenario_clicked));
 
   scenarios_treeview->get_selection()->signal_changed()
-    .connect(sigc::mem_fun(this, &LoadScenarioDialog::on_selection_changed));
-  scenarios_treeview->signal_row_activated().connect(sigc::hide(sigc::hide(sigc::mem_fun(this, &LoadScenarioDialog::on_scenario_activated))));
+    .connect(method(on_selection_changed));
+  scenarios_treeview->signal_row_activated().connect(sigc::hide(sigc::hide(method(on_scenario_activated))));
   // add the scenarios
-  add_scenario("random.map");
-  std::list<Glib::ustring> lm = File::scanMaps();
-  for (std::list<Glib::ustring>::iterator i = lm.begin(), end = lm.end();
-       i != end; ++i)
-    add_scenario(File::getMapFile(*i));
-  lm.clear();
-  lm = File::scanUserMaps();
-  for (std::list<Glib::ustring>::iterator i = lm.begin(), end = lm.end();
-       i != end; ++i)
-    add_scenario(File::getUserMapFile(*i));
+  Gtk::TreeIter i = scenarios_list->append();
+  (*i)[scenarios_columns.filename] = "random.map";
+  (*i)[scenarios_columns.details] = NULL;
 
+  name_renderer.property_editable() = false;
+  name_column.set_cell_data_func(name_renderer, method(cell_data_name));
+  scenarios_treeview->append_column(name_column);
+
+  for (auto j : *ScenarioList::getInstance ())
+    add_scenario (j);
 
   Gtk::TreeModel::Row row;
   row = scenarios_treeview->get_model()->children()[0];
@@ -70,9 +73,23 @@ LoadScenarioDialog::LoadScenarioDialog(Gtk::Window &parent)
     scenarios_treeview->get_selection()->select(row);
 }
 
-void LoadScenarioDialog::hide()
+void LoadScenarioDialog::cell_data_name(Gtk::CellRenderer *renderer,
+                                        const Gtk::TreeIter& i)
 {
-  dialog->hide();
+  ScenarioDetails *details = (*i)[scenarios_columns.details];
+
+  if (details == NULL)
+    dynamic_cast<Gtk::CellRendererText*>(renderer)->property_text () =
+      _("Random Scenario");
+  else
+    {
+      if (details->getName () == "")
+        dynamic_cast<Gtk::CellRendererText*>(renderer)->property_markup () =
+          "<i>" + _("(untitled)") + "</i>";
+      else
+        dynamic_cast<Gtk::CellRendererText*>(renderer)->property_text() =
+          details->getName();
+    }
 }
 
 void LoadScenarioDialog::run()
@@ -88,6 +105,39 @@ void LoadScenarioDialog::run()
   if (response != Gtk::RESPONSE_ACCEPT)
     selected_filename = "";
 
+  if (selected_filename.empty () == false &&
+      selected_filename != "random.map")
+    {
+      bool valid = false;
+
+      Gtk::TreeIter i = scenarios_treeview->get_selection()->get_selected();
+      ScenarioDetails *d = (*i)[scenarios_columns.details];
+      if (d->getNumberOfCities () > 1 && d->getNumberOfPlayers() > 1)
+        {
+          setup_progress_bar ();
+          bool broken = false;
+
+          GameScenario::load_tick.connect (method (tick_progress));
+          GameScenario::load_finish.connect (method (finish_progress));
+          GameScenario *g = new GameScenario (selected_filename, broken);
+          if (g)
+            {
+              if (!broken)
+                {
+                  std::list<Glib::ustring> errors;
+                  std::list<Glib::ustring> warnings;
+                  valid = g->validate (errors, warnings);
+                }
+              delete g;
+            }
+        }
+      if (!valid)
+        {
+          selected_filename = "";
+          TimedMessageDialog di(*dialog, _("The scenario isn't valid."), 0);
+          di.run_and_hide();
+        }
+    }
   dialog->get_size(width, height);
 }
 
@@ -96,22 +146,11 @@ Glib::ustring LoadScenarioDialog::get_scenario_filename()
   return selected_filename;
 }
 
-void LoadScenarioDialog::add_scenario(Glib::ustring filename)
+void LoadScenarioDialog::add_scenario(ScenarioDetails *d)
 {
   Gtk::TreeIter i = scenarios_list->append();
-  (*i)[scenarios_columns.filename] = filename;
-  if (filename == "random.map") 
-    {
-      (*i)[scenarios_columns.name] = _("Random Scenario");
-      return;
-    }
-  bool broken = false;
-  Glib::ustring name = "", comment = "", id = "";
-  guint32 player_count = 0, city_count = 0;
-  selected_filename = Glib::ustring((*i)[scenarios_columns.filename]);
-  GameScenario::loadDetails(selected_filename, broken, player_count, city_count, name, comment, id);
-  if (broken == false)
-    (*i)[scenarios_columns.name] = name;
+  (*i)[scenarios_columns.filename] = d->getFilename ();
+  (*i)[scenarios_columns.details] = d;
 }
 
 void LoadScenarioDialog::on_selection_changed()
@@ -121,6 +160,7 @@ void LoadScenarioDialog::on_selection_changed()
   if (i)
     {
       Glib::ustring filename = (*i)[scenarios_columns.filename];
+      ScenarioDetails *details = (*i)[scenarios_columns.details];
       if (filename == "random.map")
 	{
 	  load_button->set_sensitive(true);
@@ -134,25 +174,13 @@ void LoadScenarioDialog::on_selection_changed()
 	}
 
       selected_filename = filename;
-      bool broken = false;
-      Glib::ustring name, comment, id;
-      guint32 player_count = 0, city_count = 0;
-      GameScenario::loadDetails(filename, broken, player_count, city_count, name, comment, id);
-
-      if (broken == true)
-	{
-	  std::cerr << "Error: Could not parse " << selected_filename << std::endl;
-	  load_button->set_sensitive(false);
-	  return;
-	}
-	  
       remove_scenario_button->set_sensitive(true);
       load_button->set_sensitive(true);
       num_players_label->set_markup 
-	("<b>" + String::ucompose("%1", player_count - 1) + "</b>");
+	("<b>" + String::ucompose("%1", details->getNumberOfPlayers () - 1) + "</b>");
       num_cities_label->set_markup 
-	("<b>" + String::ucompose("%1", city_count) + "</b>");
-      description_textview->get_buffer()->set_text(comment);
+	("<b>" + String::ucompose("%1", details->getNumberOfCities ()) + "</b>");
+      description_textview->get_buffer()->set_text(details->getDescription ());
     }
   else
     load_button->set_sensitive(false);
@@ -182,7 +210,9 @@ void LoadScenarioDialog::on_add_scenario_clicked()
       // copy it into our ~/.lordsawar/ dir.
       File::copy (filename, File::getUserMapFile(mapname));
       // add it to the list
-      add_scenario(File::getUserMapFile(mapname));
+      if (ScenarioList::getInstance ()->add_file
+          (File::getUserMapFile (mapname)))
+        add_scenario (ScenarioList::getInstance()->back ());
     }
   delete load_map_filechooser;
 }
@@ -201,11 +231,21 @@ void LoadScenarioDialog::on_remove_scenario_clicked()
       Glib::ustring filename = (*i)[scenarios_columns.filename];
       if (filename == "random.map")
         return;
-      File::erase(filename);
-      scenarios_list->erase(i);
-      description_textview->get_buffer()->set_text("");
-      num_players_label->set_text ("");
-      num_cities_label->set_text ("");
+      if (ScenarioList::getInstance ()->remove_file (filename))
+        {
+          scenarios_list->erase(i);
+          description_textview->get_buffer()->set_text("");
+          num_players_label->set_text ("");
+          num_cities_label->set_text ("");
+        }
+      else
+        {
+          Glib::ustring errmsg = Glib::strerror(errno);
+          Glib::ustring msg = _("Error!  Scenario could not be removed.");
+          msg += "\n" + filename + "\n" + errmsg;
+          TimedMessageDialog d(*dialog, msg, 0);
+          d.run_and_hide();
+        }
     }
   return;
 }
@@ -214,3 +254,51 @@ void LoadScenarioDialog::on_scenario_activated()
 {
   load_button->activate();
 }
+
+void LoadScenarioDialog::setup_progress_bar ()
+{
+  progress_treeview = Gtk::manage (new Gtk::TreeView ());
+  progress_treeview->property_headers_visible () = false;
+  progress_liststore = Gtk::ListStore::create(progress_columns);
+  progress_treeview->set_model (progress_liststore);
+  progressrow = *(progress_liststore->append());
+  auto cell = Gtk::make_managed<Gtk::CellRendererProgress>();
+  cell->property_text () = "";
+  int cols_count = progress_treeview->append_column ("progress", *cell);
+  auto pColumn = progress_treeview->get_column(cols_count -1);
+  if (pColumn)
+    pColumn->add_attribute(cell->property_value (), progress_columns.perc);
+
+  dialog_vbox->pack_end (*progress_treeview, true, true);
+  dialog_vbox->show_all ();
+  while (g_main_context_iteration(NULL, FALSE)); //doEvents
+  dialog_vbox->set_sensitive(false);
+}
+
+void LoadScenarioDialog::tick_progress ()
+{
+  if (!progress_treeview)
+    return;
+  if (progressrow[progress_columns.perc] < 88)
+    {
+      progressrow[progress_columns.perc] =
+        progressrow[progress_columns.perc] + 13;
+      while (g_main_context_iteration(NULL, FALSE)); //doEvents
+    }
+}
+
+void LoadScenarioDialog::finish_progress ()
+{
+  if (!progress_treeview)
+    return;
+  //finish off the progressbar
+  while (progressrow[progress_columns.perc] < 100)
+    {
+      progressrow[progress_columns.perc] =
+        progressrow[progress_columns.perc] + 1;
+      while (g_main_context_iteration(NULL, FALSE)); //doEvents
+      Glib::usleep (10000);
+    }
+  progressrow[progress_columns.perc] = 100;
+  dialog->hide ();
+}
