@@ -1,5 +1,5 @@
 // Copyright (C) 2006, 2007 Ulf Lorenz
-// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2014, 2015 Ben Asselstine
+// Copyright (C) 2006-2012, 2014, 2015, 2017, 2020 Ben Asselstine
 // Copyright (C) 2007 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 //  02110-1301, USA.
 
 #include <config.h>
@@ -37,18 +37,23 @@
 #include "Configuration.h"
 #include "shieldset.h"
 #include "tileset.h"
+#include "roadlist.h"
+#include "bridgelist.h"
+#include "bridge.h"
 #include "rnd.h"
+#include "gui/font-size.h"
+#include "bigmap.h"
 
-OverviewMap::OverviewMap()
+OverviewMap::OverviewMap(bool headless)
 {
   blank_screen = false;
+  d_headless = headless;
   map_tiles_per_tile = GameMap::calculateTilesPerOverviewMapTile();
   pixels_per_tile = 2.0;
 }
 
 bool OverviewMap::isShadowed(Tile::Type type, int i, int j)
 {
-  GameMap *gm = GameMap::getInstance();
   int x = int(i / pixels_per_tile);
   int y = int(j / pixels_per_tile);
   int x2;
@@ -59,51 +64,49 @@ bool OverviewMap::isShadowed(Tile::Type type, int i, int j)
   //if yes, then maybe
   //if the tile above us or beside us is land then this might be a shadow pixel
 
-  if (gm->getTile(x,y)->getType() != type)
+  if (GameMap::getInstance()->getTile(x,y)->getType() != type)
     return false;
-  if (x > 0 && gm->getTile(x-1,y)->getType() != type)
+  if (x > 0 && GameMap::getInstance()->getTile(x-1,y)->getType() != type)
     {
       x2 = int((i-1) / pixels_per_tile);
       y2 = int(j / pixels_per_tile);
-      if (gm->getTile(x-1,y) == gm->getTile(x2,y2))
+      if (GameMap::getInstance()->getTile(x-1,y) == GameMap::getInstance()->getTile(x2,y2))
         return true;
     }
-  if (y > 0 && gm->getTile(x,y-1)->getType() != type)
+  if (y > 0 && GameMap::getInstance()->getTile(x,y-1)->getType() != type)
     {
       x2 = int(i / pixels_per_tile);
       y2 = int((j-1) / pixels_per_tile);
-      if (gm->getTile(x,y-1) == gm->getTile(x2,y2))
+      if (GameMap::getInstance()->getTile(x,y-1) == GameMap::getInstance()->getTile(x2,y2))
         return true;
     }
-  if (y > 0 && x > 0 && gm->getTile(x-1,y-1)->getType() != type)
+  if (y > 0 && x > 0 && GameMap::getInstance()->getTile(x-1,y-1)->getType() != type)
     {
       x2 = int((i-1) / pixels_per_tile);
       y2 = int((j-1) / pixels_per_tile);
-      if (gm->getTile(x-1,y-1) == gm->getTile(x2,y2))
+      if (GameMap::getInstance()->getTile(x-1,y-1) == GameMap::getInstance()->getTile(x2,y2))
         return true;
     }
 
   return false;
 }
 
-static int 
+static int
 prand(int i, int j)
 {
-  if (i || j)
-    {
-      ;
-    }
+  (void)i;
+  (void)j;
   return (Rnd::rand () % 3);
 }
 
-static int 
+static int
 crand(int i, int j)
 {
   return (i + 1) ^ (j + 1);
   //return i + j + (i*j) + (i*100) + (j*43) / 43;
 }
 
-static int 
+static int
 drand(int i, int j)
 {
   float f = i / 43 * j / 43;
@@ -167,6 +170,7 @@ OverviewMap::draw_line(bool front, int src_x, int src_y, int dst_x, int dst_y, G
   choose_surface (front, surf, gc);
   gc->set_source_rgba(color.get_red(), color.get_green(), color.get_blue(), color.get_alpha());
   gc->move_to(src_x, src_y);
+  gc->set_line_width(pixels_per_tile / 1.5);
   gc->line_to(dst_x, dst_y);
 }
 
@@ -189,13 +193,13 @@ OverviewMap::draw_rect(bool front, int x, int y, int width, int height, const Gd
      gc->rel_line_to(0, height);
       gc->rel_line_to(-width, 0);
       gc->rel_line_to(0, -height);
-      gc->set_line_width(1.0);
+      gc->set_line_width(pixels_per_tile / 1.5);
       gc->stroke();
 }
 
 void OverviewMap::draw_terrain_tile(Cairo::RefPtr<Cairo::Context> gc,
 				    SmallTile::Pattern pattern,
-				    Gdk::RGBA first, 
+				    Gdk::RGBA first,
 				    Gdk::RGBA second,
 				    Gdk::RGBA third,
 				    int i, int j, bool shadowed)
@@ -314,34 +318,26 @@ void OverviewMap::draw_terrain_tile(Cairo::RefPtr<Cairo::Context> gc,
 void OverviewMap::draw_terrain_tile(Maptile *t, int i, int j)
 {
   bool shadowed = isShadowed(t->getType(), i, j);
-  draw_terrain_tile (static_surface_gc, t->getPattern(), 
-		     t->getColor(), 
-		     t->getSecondColor(), 
+  draw_terrain_tile (static_surface_gc, t->getPattern(),
+		     t->getColor(),
+		     t->getSecondColor(),
 		     t->getThirdColor(),
 		     i, j, shadowed);
 }
 
 int OverviewMap::calculatePixelsPerTile(int width, int height)
 {
-  int pixels = 2;
-  if (width <= (int)MAP_SIZE_TINY_WIDTH && 
-      height <= (int)MAP_SIZE_TINY_HEIGHT)
-    pixels = 4;
-  else if (width <= (int)MAP_SIZE_SMALL_WIDTH && 
-	   height <= (int)MAP_SIZE_SMALL_HEIGHT)
-    pixels = 3;
-  else
-    pixels = 2;
-  switch (Configuration::UiFormFactor(Configuration::s_ui_form_factor))
-    {
-    case Configuration::UI_FORM_FACTOR_DESKTOP:
-    case Configuration::UI_FORM_FACTOR_NETBOOK:
-      break;
-    case Configuration::UI_FORM_FACTOR_LARGE_SCREEN:
-      pixels++;
-      break;
-    }
-  return pixels;
+  int pixels = 1;
+
+  (void)width;
+  int h = height /
+    GameMap::calculateTilesPerOverviewMapTile();
+
+  double ratio = 28.36; //overviewmaps are 28.36 font heights tall
+  for (;h * pixels < FontSize::getInstance ()->get_height () * ratio; pixels ++)
+    ;
+
+  return pixels + 1;
 }
 
 int OverviewMap::calculatePixelsPerTile()
@@ -352,13 +348,14 @@ int OverviewMap::calculatePixelsPerTile()
 void OverviewMap::resize()
 {
   int factor = calculatePixelsPerTile();
-  resize(GameMap::get_dim() * factor, 
+  resize(GameMap::get_dim() * factor,
          GameMap::calculateTilesPerOverviewMapTile());
 }
 
 Vector<int> OverviewMap::calculate_smallmap_size()
 {
-  Vector<int> max_dimensions = GameMap::get_dim() * calculatePixelsPerTile();
+  int factor = calculatePixelsPerTile();
+  Vector<int> max_dimensions = GameMap::get_dim() * factor;
   Vector<int> d;
   double p = max_dimensions.x / double(GameMap::get_dim().x);
   d.x = max_dimensions.x;
@@ -369,6 +366,8 @@ Vector<int> OverviewMap::calculate_smallmap_size()
 	d.x = int(round(GameMap::get_dim().x * p));
 	d.y = max_dimensions.y;
     }
+  d.x /= GameMap::calculateTilesPerOverviewMapTile();
+  d.y /= GameMap::calculateTilesPerOverviewMapTile();
   return d;
 }
 
@@ -382,17 +381,23 @@ void OverviewMap::resize(Vector<int> max_dimensions, float scale)
 
     // first try scaling to horizontal size
     pixels_per_tile = max_dimensions.x / double(bigmap_dim.x);
+    if (scale > 1)
+      pixels_per_tile = calculatePixelsPerTile ();
     d.x = max_dimensions.x;
     d.y = int(round(bigmap_dim.y * pixels_per_tile));
-    
+
     if (d.y > max_dimensions.y)
     {
 	// if too big, scale to vertical
 	pixels_per_tile = max_dimensions.y / double(bigmap_dim.y);
+        if (scale > 1)
+          pixels_per_tile = calculatePixelsPerTile ();
 	d.x = int(round(bigmap_dim.x * pixels_per_tile));
 	d.y = max_dimensions.y;
     }
     map_tiles_per_tile = scale;
+    d.x /= map_tiles_per_tile;
+    d.y /= map_tiles_per_tile;
 
     Cairo::RefPtr<Cairo::Surface> empty = Cairo::ImageSurface::create (Cairo::FORMAT_ARGB32, d.x, d.y);
     static_surface = Cairo::Surface::create(empty, Cairo::CONTENT_COLOR_ALPHA, d.x, d.y);
@@ -416,7 +421,7 @@ void OverviewMap::redraw_tiles(Rectangle tiles)
 	tiles.pos -= Vector<int>(1, 1);
 	tiles.dim += Vector<int>(2, 2);
 	
-	// translate to pixel coordinates 
+	// translate to pixel coordinates
 	Vector<int> pos(int(round(tiles.x * pixels_per_tile)),
 			int(round(tiles.y * pixels_per_tile)));
 	Vector<int> dim(int(round(tiles.w * pixels_per_tile)),
@@ -436,14 +441,13 @@ void OverviewMap::redraw_tiles(Rectangle tiles)
 
 	draw_terrain_tiles(Rectangle(pos, dim));
     }
-    draw(Playerlist::getViewingplayer());
+    draw();
 }
 
 Maptile* OverviewMap::getTile(int x, int y)
 {
   //look for something interesting so we don't skip over important tiles.
-  GameMap *gm = GameMap::getInstance();
-  Maptile *favoured_tile = gm->getTile(x,y);
+  Maptile *favoured_tile = GameMap::getInstance()->getTile(x,y);
   int xmax = x + map_tiles_per_tile - 1;
   if (xmax >= GameMap::getWidth())
     xmax = GameMap::getWidth() - 1;
@@ -454,114 +458,133 @@ Maptile* OverviewMap::getTile(int x, int y)
     for (int j = y; j < ymax; j++)
       {
         Vector<int> pos(i, j);
-        if (gm->getBuilding(pos) == Maptile::TEMPLE)
-          favoured_tile = gm->getTile(pos);
-        else if (gm->getBuilding(pos) == Maptile::RUIN)
-          favoured_tile = gm->getTile(pos);
-        else if (gm->getTerrainType(pos) == Tile::WATER)
-          favoured_tile = gm->getTile(pos);
-        else if (gm->getTerrainType(pos) == Tile::MOUNTAIN)
-          favoured_tile = gm->getTile(pos);
+        if (GameMap::getInstance()->getBuilding(pos) == Maptile::TEMPLE)
+          favoured_tile = GameMap::getInstance()->getTile(pos);
+        else if (GameMap::getInstance()->getBuilding(pos) == Maptile::RUIN)
+          favoured_tile = GameMap::getInstance()->getTile(pos);
+        else if (GameMap::getInstance()->getTerrainType(pos) == Tile::WATER)
+          favoured_tile = GameMap::getInstance()->getTile(pos);
+        else if (GameMap::getInstance()->getTerrainType(pos) == Tile::MOUNTAIN)
+          favoured_tile = GameMap::getInstance()->getTile(pos);
       }
   return favoured_tile;
 }
 
 void OverviewMap::draw_terrain_tiles(Rectangle r)
 {
-    Tileset *ts = GameMap::getTileset();
-    Gdk::RGBA rd = ts->getRoadColor();
-    for (int i = r.x; i < r.x + r.w; i+=int(map_tiles_per_tile))
-      for (int j = r.y; j < r.y + r.h; j+=int(map_tiles_per_tile))
-        {
-          int x = int(i / pixels_per_tile);
-          int y = int(j / pixels_per_tile);
-          Maptile *mtile = getTile(x,y);
+  Gdk::RGBA rd = GameMap::getTileset()->getRoadColor();
+  r.w *= map_tiles_per_tile;
+  r.h *= map_tiles_per_tile;
+  for (int i = r.x; i < r.x + r.w; i += int(map_tiles_per_tile))
+    for (int j = r.y; j < r.y + r.h; j += int(map_tiles_per_tile))
+      {
+        int x = int(i / pixels_per_tile);
+        int y = int(j / pixels_per_tile);
+        Maptile *mtile = getTile(x,y);
+        draw_terrain_tile (mtile, i /map_tiles_per_tile, j /map_tiles_per_tile);
+      }
 
-          if (mtile->isRoadTerrain())
-            draw_pixel(static_surface_gc, i, j, rd);
-          else
-            draw_terrain_tile (mtile, i, j);
-        }
+  int size = int(pixels_per_tile) > 1 ? int(pixels_per_tile) : 1;
+  for (auto it : *Roadlist::getInstance())
+      {
+        Vector<int> pos = it->getPos();
+        pos = mapToSurface(pos);
+        pos -= Vector<int>(size,size) / 2;
+	draw_filled_rect(false, pos.x, pos.y, size, size, rd);
+      }
+  for (auto it : *Bridgelist::getInstance())
+      {
+        Vector<int> pos = it->getPos();
+        pos = mapToSurface(pos);
+        pos -= Vector<int>(size,size) / 2;
+	draw_filled_rect(false, pos.x, pos.y, size, size, rd);
+      }
 }
 
 void OverviewMap::after_draw()
 {
 }
 
-void OverviewMap::draw(Player *player)
+void OverviewMap::draw()
 {
-    Tileset *ts = GameMap::getTileset();
-    Playerlist::getInstance()->setViewingplayer(player);
-    int size = int(pixels_per_tile) > 1 ? int(pixels_per_tile) : 1;
-    assert(surface);
+  if (d_headless)
+    return;
+  Tileset *ts = GameMap::getTileset();
+  //Playerlist::getInstance()->setViewingplayer(player);
+  int size = int(pixels_per_tile) > 1 ? int(pixels_per_tile) : 1;
+  assert(surface);
 
 
-    // During the whole drawing stuff, ALWAYS consider that 
-    // there is an offset of 1 between map coordinates and coordinates
-    // of the surface when drawing. I will implcitely assume this during this
-    // function.
-    
-    //put the static surface on the surface
-    surface_gc->set_source(static_surface, 0, 0);
-    surface_gc->paint();
+  // During the whole drawing stuff, ALWAYS consider that
+  // there is an offset of 1 between map coordinates and coordinates
+  // of the surface when drawing. I will implcitely assume this during this
+  // function.
 
-    // Draw ruins as a white dot
-	
-    Gdk::RGBA ruindotcolor = ts->getRuinColor();
-    for (Ruinlist::iterator it = Ruinlist::getInstance()->begin();
-        it != Ruinlist::getInstance()->end(); it++)
+  //put the static surface on the surface
+  surface_gc->set_source(static_surface, 0, 0);
+  surface_gc->paint();
+
+  // Draw ruins as a white dot
+
+  Gdk::RGBA ruindotcolor = ts->getRuinColor();
+  for (Ruinlist::iterator it = Ruinlist::getInstance()->begin();
+       it != Ruinlist::getInstance()->end(); it++)
     {
-        Ruin *r = *it;
-        if (r->isHidden() == true && 
-	    r->getOwner() != Playerlist::getViewingplayer())
-          continue;
-        if (r->isVisible(Playerlist::getViewingplayer()) == false)
-          continue;
-        Vector<int> pos = r->getPos();
-        pos = mapToSurface(pos);
+      Ruin *r = *it;
+      if (BigMap::s_show_hidden_ruins == false)
+        {
+          if (r->isHidden() == true &&
+              r->getOwner() != Playerlist::getViewingplayer())
+            continue;
+          if (r->isVisible(Playerlist::getViewingplayer()) == false)
+            continue;
+        }
+      Vector<int> pos = r->getPos();
+      pos = mapToSurface(pos);
 
-	draw_filled_rect(true, pos.x, pos.y, size, size, ruindotcolor);
+      draw_filled_rect(true, pos.x, pos.y, size, size, ruindotcolor);
     }
 
-    // Draw temples as a white dot
-    Gdk::RGBA templedotcolor = ts->getTempleColor();
-    for (Templelist::iterator it = Templelist::getInstance()->begin();
-        it != Templelist::getInstance()->end(); it++)
+  // Draw temples as a white dot
+  Gdk::RGBA templedotcolor = ts->getTempleColor();
+  for (Templelist::iterator it = Templelist::getInstance()->begin();
+       it != Templelist::getInstance()->end(); it++)
     {
       Temple *t = *it;
-        if (t->isVisible(Playerlist::getViewingplayer()) == false)
-          continue;
-        Vector<int> pos = t->getPos();
-        pos = mapToSurface(pos);
+      if (t->isVisible(Playerlist::getViewingplayer()) == false)
+        continue;
+      Vector<int> pos = t->getPos();
+      pos = mapToSurface(pos);
 
-	draw_filled_rect(true, pos.x, pos.y, size, size, templedotcolor);
+      draw_filled_rect(true, pos.x, pos.y, size, size, templedotcolor);
     }
 
-    //fog it up
-    if (GameScenarioOptions::s_hidden_map == true)
-      {
-        for (int i = 0; i < GameMap::getWidth(); i++)
-          for (int j = 0; j < GameMap::getHeight(); j++)
-            {
-              Vector <int> pos = Vector<int>(i, j);
-              if (Playerlist::getViewingplayer()->getFogMap()->isFogged(pos) == true)
-                {
-                  pos = mapToSurface(pos);
-                  draw_filled_rect(true, pos.x, pos.y, size, size, FOG_COLOUR);
-                }
-            }
-      }
+  //fog it up
+  if (GameScenarioOptions::s_hidden_map == true)
+    {
+      for (int i = 0; i < GameMap::getWidth(); i++)
+        for (int j = 0; j < GameMap::getHeight(); j++)
+          {
+            Vector <int> pos = Vector<int>(i, j);
+            if (Playerlist::getViewingplayer()->getFogMap()->isFogged(pos) == true)
+              {
+                pos = mapToSurface(pos);
+                draw_filled_rect(true, pos.x, pos.y, size, size, FOG_COLOUR);
+                if (i == 0)
+                  draw_filled_rect(true, pos.x-size, pos.y, size, size, FOG_COLOUR);
+                if (j == 0)
+                  draw_filled_rect(true, pos.x, pos.y-size, size, size, FOG_COLOUR);
+              }
+          }
+    }
 
-    //the idea here is that we want to show what happens when an AI-owned
-    //stack moves through our area.  so we block the view of computer
-    //players after the fact.
-    if (Playerlist::getViewingplayer()->getType() != Player::HUMAN &&
-       	GameScenarioOptions::s_hidden_map == true)
-      {
-	int width = get_width();
-	int height = get_height();
-	draw_filled_rect(true, 0, 0, width, height, FOG_COLOUR);
-      }
+  if (Playerlist::getActiveplayer()->getType() != Player::HUMAN &&
+      GameScenarioOptions::s_hidden_map == true)
+    {
+      int width = get_width();
+      int height = get_height();
+      draw_filled_rect(true, 0, 0, width, height, FOG_COLOUR);
+    }
 
   if (blank_screen)
     {
@@ -571,8 +594,8 @@ void OverviewMap::draw(Player *player)
       surface_gc->rectangle(0,0,width, height);
       surface_gc->fill();
     }
-    // let derived classes do their job
-    after_draw();
+  // let derived classes do their job
+  after_draw();
 }
 
 Cairo::RefPtr<Cairo::Surface> OverviewMap::get_surface()
@@ -582,9 +605,9 @@ Cairo::RefPtr<Cairo::Surface> OverviewMap::get_surface()
 
 Vector<int> OverviewMap::mapFromScreen(Vector<int> pos)
 {
-    int x = int(pos.x / pixels_per_tile / map_tiles_per_tile);
-    int y = int(pos.y / pixels_per_tile / map_tiles_per_tile);
-    
+    int x = int(pos.x / pixels_per_tile * map_tiles_per_tile);
+    int y = int(pos.y / pixels_per_tile * map_tiles_per_tile);
+
     if (x >= GameMap::getWidth())
         x = GameMap::getWidth() - 1;
 
@@ -593,10 +616,10 @@ Vector<int> OverviewMap::mapFromScreen(Vector<int> pos)
 
     if (x < 0)
       x = 0;
-    
+
     if (y < 0)
       y = 0;
-    
+
     return Vector<int>(x,y);
 }
 
@@ -621,23 +644,13 @@ Vector<int> OverviewMap::mapToSurface(Vector<int> pos)
 
     if (pixels_per_tile > 2)
         y += int(0.5 * pixels_per_tile);
-    
+
     return Vector<int>(x, y);
 }
 
 void OverviewMap::draw_cities (bool all_razed)
 {
   int csize = 0;
-  switch (Configuration::UiFormFactor(Configuration::s_ui_form_factor))
-    {
-    case Configuration::UI_FORM_FACTOR_DESKTOP:
-    case Configuration::UI_FORM_FACTOR_NETBOOK:
-      break;
-    case Configuration::UI_FORM_FACTOR_LARGE_SCREEN:
-      csize = 1;
-      break;
-    }
-  ImageCache *gc = ImageCache::getInstance();
 
   // Draw all cities as shields over the city location, in the colors of
   // the players.
@@ -649,47 +662,73 @@ void OverviewMap::draw_cities (bool all_razed)
       if (c->isVisible(Playerlist::getViewingplayer()) == false)
         continue;
       if (c->isBurnt() == true || all_razed == true)
-        tmp = gc->getSmallRuinedCityImage();
+        {
+          tmp = ImageCache::getInstance()->getSmallRuinedCityImage()->copy ();
+
+          //we need 78 of these per map height
+          double new_height = get_height () / 78.0;
+          int new_width = tmp->get_width () * (new_height / tmp->get_height ());
+
+          PixMask::scale (tmp, new_width, new_height);
+        }
       else
-        tmp = gc->getShieldPic(csize, c->getOwner());
-  
+        {
+          tmp = ImageCache::getInstance()->getShieldPic(csize, c->getOwner(),
+                                                        true, 0)->copy ();
+          //we need 39 of these per map height
+          double new_height = get_height () / 39.0;
+          int new_width = tmp->get_width () * (new_height / tmp->get_height ());
+
+          PixMask::scale (tmp, new_width, new_height);
+        }
+
       Vector<int> pos = c->getPos();
       pos = mapToSurface(pos);
       tmp->blit_centered(surface, pos);
+      delete tmp;
   }
 }
 
 void OverviewMap::blank(bool on)
 {
   blank_screen = on;
-  draw(Playerlist::getViewingplayer());
+  draw();
 }
 
 void OverviewMap::draw_hero(Vector<int> pos, bool white)
 {
-    ImageCache *gc = ImageCache::getInstance();
     // draw the hero picture over top of the host city
 
     Vector<int> start = mapToSurface(pos);
 
     start += Vector<int>(int(pixels_per_tile/2), int(pixels_per_tile/2));
 
-    PixMask *heropic = gc->getSmallHeroImage(white);
+    PixMask *heropic =
+      ImageCache::getInstance()->getSmallHeroImage(white)->copy ();
+
+    //we need 20.8 of these per map height
+    double new_height = get_height () / 20.8;
+    int new_width = heropic->get_width () * (new_height / heropic->get_height ());
+    PixMask::scale (heropic, new_width, new_height);
     heropic->blit_centered(surface, start);
+    delete heropic;
 }
 
 void OverviewMap::draw_target_box(Vector<int> pos, const Gdk::RGBA c)
 {
   Vector<int> start = mapToSurface(pos);
   start += Vector<int>(int(pixels_per_tile/2), int(pixels_per_tile/2));
-  int xsize = 8;
-  int ysize = 8;
+  double newsize = get_height () / 39.0; // 312 / 8
+  int xsize = int(newsize);
+  int ysize = int(newsize);
   //draw an 8 by 8 box, with a smaller box inside of it
-  draw_rect(start.x - (xsize / 2), start.y - (ysize / 2), 
+  draw_rect(start.x - (xsize / 2), start.y - (ysize / 2),
 	    xsize, ysize, c);
-  xsize = 5;
-  ysize = 5;
-  draw_filled_rect(start.x - (xsize / 2), start.y - (ysize / 2), 
+  //draw a 5 by 5 smaller box
+  newsize = get_height () / 62.4; // 312 / 5
+  xsize = int(newsize);
+  ysize = int(newsize);
+  draw_filled_rect(start.x - (xsize / 2), start.y - (ysize / 2),
 		   xsize, ysize, c);
 }
 
@@ -697,12 +736,12 @@ void OverviewMap::draw_square_around_city(City *c, Gdk::RGBA colour)
 {
   Vector<int> start = c->getPos();
   start = mapToSurface(start);
-  Shieldset *ss = GameMap::getShieldset();
-  guint32 width = ss->getSmallWidth();
-  guint32 height = ss->getSmallHeight();
+  //we need 22.29 of these per map height
+  double height = get_height () / 22.29;
+  int width = 14.0 * (height / 14.0);
   start -= Vector<int>(width,height)/2;
   Vector<int> end = start + Vector<int>(width,height);
-  draw_rect (start.x-3, start.y-3, end.x-start.x+4, end.y-start.y+4, colour);
+  draw_rect (start.x-0, start.y-0, end.x-start.x+0, end.y-start.y+0, colour);
 }
 
 void OverviewMap::draw_radial_gradient(Cairo::RefPtr<Cairo::Surface> surface, Gdk::RGBA inner, Gdk::RGBA outer, int width, int height)
@@ -719,7 +758,7 @@ void OverviewMap::draw_radial_gradient(Cairo::RefPtr<Cairo::Surface> surface, Gd
     max = (double)width;
   double xcenter = (double)width / 2.0;
   double ycenter = (double)height / 2.0;
-  Cairo::RefPtr<Cairo::RadialGradient> gradient = 
+  Cairo::RefPtr<Cairo::RadialGradient> gradient =
     Cairo::RadialGradient::create(xcenter, ycenter, 1, xcenter, ycenter, max);
   gradient->add_color_stop_rgb(0, ired, igreen, iblue);
   gradient->add_color_stop_rgb(1.0, ored, ogreen, oblue);
