@@ -1,5 +1,5 @@
 // Copyright (C) 2006 Ulf Lorenz
-// Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2014, 2015 Ben Asselstine
+// Copyright (C) 2007-2012, 2014, 2015, 2017 Ben Asselstine
 // Copyright (C) 2007 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -17,6 +17,7 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
+#pragma once
 #ifndef OVERVIEWMAP_H
 #define OVERVIEWMAP_H
 
@@ -55,7 +56,7 @@ class OverviewMap : public sigc::trackable
       * This constructor is not normally called by itself, usually it is
       * called by a derived class.
       */
-     OverviewMap();
+     OverviewMap(bool headless = false);
 
      //! Destructor.
      virtual ~OverviewMap() {};
@@ -93,15 +94,12 @@ class OverviewMap : public sigc::trackable
      * Draws the roads, cities, ruins, temples and roads onto the map in the
      * correct aspect ratio.
      *
-     * @param player draw the map from the given player's perspective.
-     *
-     *
      * The aspect ratio for the terrain features is governed by the dimensions 
      * passed to the OverviewMap::resize method.
      *
      * This method calls the after_draw method from the derived classes.
      */
-    void draw(Player *player);
+    void draw();
 
     //! Redraw a portion of the map graphic.
     /**
@@ -151,6 +149,7 @@ class OverviewMap : public sigc::trackable
 
     int get_width();
     int get_height();
+
  private:
 
     //! Returns whether or not the given pixel appears sunken (Tile::SUNKEN).
@@ -231,6 +230,9 @@ class OverviewMap : public sigc::trackable
      */
     void draw_cities(bool all_razed);
 
+    //! Draw a hero icon at the given location.  white or black.
+    void draw_hero(Vector<int> pos, bool white);
+
     //! Redraw the specified region.
     void draw_terrain_tiles(Rectangle r);
 
@@ -249,14 +251,12 @@ class OverviewMap : public sigc::trackable
     Cairo::RefPtr<Cairo::Surface> surface;
     Cairo::RefPtr<Cairo::Context> surface_gc;
 
-    //! Draw a hero icon at the given location.  white or black.
-    void draw_hero(Vector<int> pos, bool white);
-
     void draw_target_box(Vector<int> pos, const Gdk::RGBA colour);
     void draw_square_around_city(City *c, const Gdk::RGBA colour);
     void draw_radial_gradient(Gdk::RGBA inner, Gdk::RGBA outer, int width, int height);
 
     bool blank_screen;
+    bool d_headless;
 
 };
 
