@@ -2,7 +2,7 @@
 // Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
 // Copyright (C) 2004 John Farrell
 // Copyright (C) 2005 Andrea Paternesi
-// Copyright (C) 2007, 2008, 2009, 2010, 2014, 2015 Ben Asselstine
+// Copyright (C) 2007, 2008, 2009, 2010, 2014, 2015, 2017 Ben Asselstine
 // Copyright (C) 2007 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -181,7 +181,7 @@ void Playerlist::nextPlayer()
     }
 
     d_activeplayer = (*it);
-    viewingplayer = (*it);
+    updateViewingPlayer();
     debug("got player: " <<d_activeplayer->getName())
 }
 
@@ -271,11 +271,11 @@ bool Playerlist::load(Glib::ustring tag, XML_Helper* helper)
 
     //set neutral and active
     if (p->getId() == neutral)
-        d_neutral = p;
+      d_neutral = p;
     if (p->getId() == active)
       d_activeplayer = p;
 
-    viewingplayer = d_activeplayer;
+    updateViewingPlayer();
 
     return true;
 }
@@ -422,9 +422,8 @@ void Playerlist::calculateWinners()
 	if ((*it)->isDead() == true)
 	  continue;
 
-	Citylist *clist = Citylist::getInstance();
 	float city_component = (float)
-	  ((float) clist->countCities(*it)/ (float)total_cities) * 70.0;
+	  ((float) Citylist::getInstance()->countCities(*it)/ (float)total_cities) * 70.0;
 	float gold_component = (float)
 	  ((float) (*it)->getGold() / (float)total_gold) * 10.0;
 	float army_component = (float)
@@ -541,12 +540,8 @@ void Playerlist::swap(Player *old_player, Player *new_player)
 
 bool Playerlist::randomly(const Player *lhs, const Player *rhs)  
 {
-  Playerlist *pl = Playerlist::getInstance();
-  if (rhs)
-    {
-      ;
-    }
-  if (lhs == pl->getNeutral())
+  (void) rhs;
+  if (lhs == Playerlist::getInstance()->getNeutral())
     return false;
   if (Rnd::rand() % 2 == 0)
     return true;
@@ -590,9 +585,8 @@ void Playerlist::nextRound(bool diplomacy, bool *surrender_already_offered)
 	{
 	  if ((*it)->getType() == Player::HUMAN)
 	    {
-	      Citylist *cl = Citylist::getInstance();
-	      int target_level = cl->size() / 2;
-	      if (cl->countCities(*it) > target_level)
+	      int target_level = Citylist::getInstance()->size() / 2;
+	      if (Citylist::getInstance()->countCities(*it) > target_level)
 		{
 		  *surrender_already_offered = 1;
 		  ssurrender.emit(*it);
@@ -646,7 +640,7 @@ void Playerlist::syncPlayer(GameParameters::Player player)
 
 	  sort(inOrderOfId);
 	  d_activeplayer = getFirstLiving();
-	  viewingplayer = d_activeplayer;
+          updateViewingPlayer();
 	}
       return;
     }
@@ -701,7 +695,7 @@ void Playerlist::syncPlayer(GameParameters::Player player)
 
   sort(inOrderOfId);
   d_activeplayer = getFirstLiving();
-  viewingplayer = d_activeplayer;
+  updateViewingPlayer();
   return;
 }
 
@@ -825,7 +819,7 @@ void Playerlist::reorder(std::list<guint32> order)
   sort(inGivenOrder);
   given_turn_order.clear();
   d_activeplayer = getFirstLiving();
-  viewingplayer = d_activeplayer;
+  updateViewingPlayer();
 }
 
 std::list<History *>Playerlist::getHistoryForHeroId(guint32 id) const
@@ -940,3 +934,62 @@ Stack *Playerlist::getStackById(guint32 id) const
     }
   return NULL;
 }
+
+void Playerlist::updateViewingPlayer ()
+{
+  /*
+   * the idea here is that the viewing player is related to the
+   * hidden map.
+   *
+   * imagine a computer player moving and then it walks into the area of
+   * your map that you have uncovered.
+   *
+   * i think the smallmap happens to be getting blanked right now,
+   * but this funciton is about retaining the last human player as the
+   * player who can see movements through his or her territory.
+   *
+   * the blanking of the smallmap/bigmap is because:
+   * it's not really fair to show one player some enemy units going through
+   * his or her territory, and not all players.  why should one player be
+   * rewarded in this way just by fluke?
+   *
+   * this is the story for hotseat anyway.  i'm not sure how it relates to
+   * network play.
+   */
+  if (d_activeplayer && d_activeplayer->getType() == Player::HUMAN)
+    viewingplayer = d_activeplayer;
+  else
+    viewingplayer = getNeutral();
+}
+
+guint32 Playerlist::getTurnOrderNumber(const Player *p)
+{
+  guint32 count = 1;
+  for (const_iterator i = begin(); i != end(); i++)
+    {
+      if ((*i) == p)
+        break;
+      count++;
+    }
+  return count;
+}
+
+guint32 Playerlist::countAllStacks () const
+{
+  guint32 count = 0;
+  for (const_iterator i = begin(); i != end(); i++)
+    count += (*i)->getStacklist ()->size ();
+  return count;
+}
+
+bool Playerlist::playerHasNoCapitalCity () const
+{
+  for (const_iterator i = begin (); i != end (); i++)
+    {
+      if (*i != d_neutral &&
+          Citylist::getInstance ()->countCities (*i) > 0 &&
+          Citylist::getInstance ()->getCapitalCity (*i) == NULL)
+        return true;
+    }
+  return false;
+}
