@@ -1,6 +1,6 @@
 // Copyright (C) 2000, 2001, 2002, 2003 Michael Bartl
 // Copyright (C) 2001, 2002, 2003, 2004, 2005 Ulf Lorenz
-// Copyright (C) 2007, 2008, 2009, 2014, 2015 Ben Asselstine
+// Copyright (C) 2007, 2008, 2009, 2014, 2015, 2017, 2020 Ben Asselstine
 // Copyright (C) 2007 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -18,6 +18,7 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
+#pragma once
 #ifndef PLAYERLIST_H
 #define PLAYERLIST_H
 
@@ -58,7 +59,7 @@ class Playerlist : public std::list<Player*>, public sigc::trackable
 	void setActiveplayer(Player *p) {d_activeplayer = p;};
 
 	//! set the player who is looking at the bigmap and smallmap.
-	void setViewingplayer(Player *p) {viewingplayer = p;};
+	//void setViewingplayer(Player *p) {viewingplayer = p;};
 
 	//! Get Methods
 
@@ -236,6 +237,12 @@ class Playerlist : public std::list<Player*>, public sigc::trackable
 	//! Return the number of players left alive, not including neutral.
 	guint32 countPlayersAlive() const;
 
+        //! Return the total number of stacks on the map.
+        guint32 countAllStacks () const;
+
+        //! Return true if one non-neutral player doesn't have a capital city.
+        bool playerHasNoCapitalCity () const;
+
 	//! Return the list of activities that the given hero has accomplished.
 	std::list<History *>getHistoryForHeroId(guint32 id) const;
 
@@ -247,10 +254,11 @@ class Playerlist : public std::list<Player*>, public sigc::trackable
 	  */
 	bool isEndOfRound() const;
 
-
         //! Return true if any of the players use the given armyset.
         bool hasArmyset(guint32 id) const;
 
+        //! Get which numeric sequence the player is in this round
+        guint32 getTurnOrderNumber(const Player *p);
 	// Signals
 
 	/**
@@ -323,6 +331,7 @@ class Playerlist : public std::list<Player*>, public sigc::trackable
 	//! Calculate new diplomatic states for all players.
 	void negotiateDiplomacy();
 
+        void updateViewingPlayer ();
 
         Glib::ustring get_title(int rank);
         // DATA
