@@ -2,7 +2,7 @@
 // Copyright (C) 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
 // Copyright (C) 2004, 2005, 2006 Andrea Paternesi
 // Copyright (C) 2004 Thomas Plonka
-// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2014, 2015 Ben Asselstine
+// Copyright (C) 2006-2010, 2014, 2015, 2017 Ben Asselstine
 // Copyright (C) 2007 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -29,13 +29,19 @@
 #include "GameMap.h"
 #include "playerlist.h"
 
-SmallMap::SmallMap()
+bool SmallMap::s_quick = false;
+
+SmallMap::SmallMap(bool headless)
+ : OverviewMap(headless)
 {
     input_locked = false;
     sliding = false;
     view.pos = Vector<int>(0, 0);
     view.dim = Vector<int>(3, 3);
-    sleep_interval = TIMER_SMALLMAP_REFRESH;
+    if (s_quick)
+      sleep_interval = 0;
+    else
+      sleep_interval = TIMER_SMALLMAP_REFRESH;
 }
 
 void SmallMap::set_view(Rectangle new_view)
@@ -43,7 +49,7 @@ void SmallMap::set_view(Rectangle new_view)
     if (view != new_view)
     {
 	view = new_view;
-	draw(Playerlist::getViewingplayer());
+	draw();
     }
 }
 
@@ -52,8 +58,8 @@ void SmallMap::draw_selection()
     // draw the selection rectangle that shows the viewed part of the map
     Vector<int> pos = mapToSurface(view.pos);
 
-    int w = int(view.w * pixels_per_tile);
-    int h = int(view.h * pixels_per_tile);
+    int w = int(view.w * pixels_per_tile / map_tiles_per_tile);
+    int h = int(view.h * pixels_per_tile / map_tiles_per_tile);
 
     int width = get_width();
     int height = get_height();
@@ -69,7 +75,7 @@ void SmallMap::draw_selection()
 	   pos.y >= 0 && pos.y + h < height);
     
     draw_rect(pos.x, pos.y, w, h, SELECTOR_BOX_COLOUR);
-    draw_rect(pos.x-1, pos.y-1, w+2,  h+2, SELECTOR_BOX_COLOUR);
+    //draw_rect(pos.x-1, pos.y-1, w+2,  h+2, SELECTOR_BOX_COLOUR);
 }
 
 void SmallMap::center_view_on_tile(Vector<int> pos, bool slide_me)
@@ -78,7 +84,7 @@ void SmallMap::center_view_on_tile(Vector<int> pos, bool slide_me)
              GameMap::get_dim() - view.dim);
 
   sliding = false;
-  if (slide_me)
+  if (slide_me && sleep_interval > 0)
     slide_view(Rectangle(pos.x, pos.y, view.w, view.h));
   else
     set_view(Rectangle(pos.x, pos.y, view.w, view.h));
@@ -88,14 +94,20 @@ void SmallMap::center_view_on_tile(Vector<int> pos, bool slide_me)
 
 void SmallMap::center_view_on_pixel(Vector<int> pos, bool slide_me)
 {
-  pos.x = int(round(pos.x / pixels_per_tile));
-  pos.y = int(round(pos.y / pixels_per_tile));
+  pos.x = int(round(pos.x / pixels_per_tile * map_tiles_per_tile));
+  pos.y = int(round(pos.y / pixels_per_tile * map_tiles_per_tile));
 
-  pos -= view.dim / pixels_per_tile;
+  /* FIXME: i have no idea why 1.65 is the number i need here.
+   it controls the centeredness of the white box on where we clicked.
+   this works well for the 3 standard sizes of maps.
+   */
+
+  pos.x -= (view.w / 1.65);
+  pos.y -= (view.h / 1.65);
 
   pos = clip(Vector<int>(0, 0), pos, GameMap::get_dim() - view.dim);
 
-  if (slide_me)
+  if (slide_me && sleep_interval > 0)
     slide_view(Rectangle(pos.x, pos.y, view.w, view.h));
   else
     set_view(Rectangle(pos.x, pos.y, view.w, view.h));
@@ -151,20 +163,24 @@ void SmallMap::mouse_motion_event(MouseMotionEvent e)
 
 int SmallMap::slide (int x, int y)
 {
-  int skip = 2;
+  int skip = map_tiles_per_tile + 1;
   if (x < y)
     {
       if (x + skip < y)
 	x += skip;
+      else if (x + map_tiles_per_tile < y)
+	x += map_tiles_per_tile ;
       else
-	x++;
+	x += 1;
     }
   else if (x > y)
     {
       if (x - skip > y)
 	x -= skip;
+      else if (x - map_tiles_per_tile > y)
+	x -= map_tiles_per_tile;
       else
-	x--;
+	x -= 1;
     }
   return x;
 }
@@ -182,7 +198,7 @@ void SmallMap::slide_view(Rectangle new_view)
 	  tmp_view.y = slide(tmp_view.y, new_view.y);
 
 	  view = tmp_view;
-	  draw(Playerlist::getViewingplayer());
+	  draw();
           view_slid.emit(view);
           if (sliding_to != new_view)
             break;
@@ -207,3 +223,9 @@ void SmallMap::move_map_in_dir(Vector<int> dir)
   set_view(new_view);
   view_changed.emit(view);
 }
+
+void SmallMap::center_view ()
+{
+  set_view (Rectangle ((GameMap::get_dim () / 2) - (view.dim / 2), view.dim));
+  view_changed.emit(view);
+}
