@@ -2,7 +2,7 @@
 // Copyright (C) 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
 // Copyright (C) 2004, 2005 Andrea Paternesi
 // Copyright (C) 2004 Thomas Plonka
-// Copyright (C) 2007, 2008, 2009, 2014 Ben Asselstine
+// Copyright (C) 2007, 2008, 2009, 2014, 2017 Ben Asselstine
 // Copyright (C) 2007 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,7 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
+#pragma once
 #ifndef SMALLMAP_H
 #define SMALLMAP_H
 
@@ -44,8 +45,10 @@
 class SmallMap: public OverviewMap
 {
 public:
+    static bool s_quick;
+
     //! Default constructor.  Make a new SmallMap.
-    SmallMap();
+    SmallMap(bool headless);
 
     //! Destructor.
     ~SmallMap() {};
@@ -110,6 +113,9 @@ public:
      */
     void center_view_on_pixel(Vector<int> pos, bool slide);
 
+    //! Center the little white box in the center of the map graphic.
+    void center_view ();
+
     //! Move the view one tile in the given direction.
     void move_map_in_dir(Vector<int> dir);
 
