@@ -125,10 +125,8 @@ void Stack::moveOneStep(bool skipping)
 bool Stack::isMovingToOrFromAShip(Vector<int> dest, bool &on_ship) const
 {
   Vector<int> pos = getPos();
-  GameMap *gm = GameMap::getInstance();
-
-  Maptile::Building src_building = gm->getBuilding(pos);
-  Maptile::Building dst_building = gm->getBuilding(dest);
+  Maptile::Building src_building = GameMap::getInstance()->getBuilding(pos);
+  Maptile::Building dst_building = GameMap::getInstance()->getBuilding(dest);
 
   bool to_city = dst_building == Maptile::CITY;
   bool on_city = src_building == Maptile::CITY;
@@ -136,8 +134,8 @@ bool Stack::isMovingToOrFromAShip(Vector<int> dest, bool &on_ship) const
   bool on_port = src_building == Maptile::PORT;
   bool on_bridge = src_building == Maptile::BRIDGE;
   bool to_bridge = dst_building == Maptile::BRIDGE;
-  bool on_water = (gm->getTerrainType(pos) == Tile::WATER);
-  bool to_water = (gm->getTerrainType(dest) == Tile::WATER);
+  bool on_water = (GameMap::getInstance()->getTerrainType(pos) == Tile::WATER);
+  bool to_water = (GameMap::getInstance()->getTerrainType(dest) == Tile::WATER);
   //here we mark the armies as being on or off a boat
   /* skipping refers to when we have to move over another friendly stack
    * of a size that's too big to join with. */
@@ -247,8 +245,7 @@ guint32 Stack::getMoves() const
 
 int Stack::getMinTileMoves() const
 {
-  GameMap *map = GameMap::getInstance();
-  Rectangle bounds = map->get_boundary();
+  Rectangle bounds = GameMap::getInstance()->get_boundary();
 
   std::vector<Vector<int> > tiles;
   tiles.push_back(Vector<int>(getPos().x + 1, getPos().y - 1));
@@ -262,11 +259,10 @@ int Stack::getMinTileMoves() const
 
   int min = -1;
 
-  for (std::vector<Vector<int> >::iterator i = tiles.begin(); i != tiles.end();
-       ++i)
-    if (is_inside(bounds, *i))
+  for (auto tile: tiles)
+    if (is_inside(bounds, tile))
       {
-	int v = map->getTile(i->x, i->y)->getMoves();
+	int v = GameMap::getInstance()->getTile(tile)->getMoves();
 	if (min == -1)
 	  min = v;
 	else
@@ -310,31 +306,15 @@ Army* Stack::getStrongestHero() const
 {
   Army *strongest = 0;
   guint32 highest_strength = 0;
-  bool water = 
-    GameMap::getInstance()->getTile(getPos())->getType() == Tile::WATER;
-  if (GameMap::getBridge(getPos()))
-    water = false;
   for (const_iterator it = begin(); it != end(); ++it)
     {
       if ((*it)->isHero())
         {
-          if (!water)
-            {
-              if ((*it)->getStat(Army::STRENGTH) > highest_strength)
+          if ((*it)->getStat(Army::STRENGTH) > highest_strength)
 
-                {
-                  highest_strength = (*it)->getStat(Army::STRENGTH);
-                  strongest = *it;
-                }
-            }
-          else
             {
-              if ((*it)->getStat(Army::SHIP) &&
-                  (*it)->getStat(Army::BOAT_STRENGTH) > highest_strength)
-                {
-                  highest_strength = (*it)->getStat(Army::STRENGTH);
-                  strongest = *it;
-                }
+              highest_strength = (*it)->getStat(Army::STRENGTH);
+              strongest = *it;
             }
         }
     }
@@ -592,13 +572,29 @@ bool Stack::load(Glib::ustring tag, XML_Helper* helper)
 void Stack::flClear()
 {
   for (iterator it = begin(); it != end(); it++)
-    delete (*it);
+    {
+      Army *a = *it;
+      if (a->isHero())
+        {
+          Hero *h = dynamic_cast<Hero*>(a);
+          delete h;
+        }
+      else
+        delete a;
+    }
   clear();
 }
 
 Stack::iterator Stack::flErase(Stack::iterator object)
 {
-  delete (*object);
+  Army *a = *object;
+  if (a->isHero())
+    {
+      Hero *h = dynamic_cast<Hero*>(a);
+      delete h;
+    }
+  else
+    delete (a);
   return erase(object);
 }
 
@@ -645,7 +641,7 @@ guint32 Stack::calculateMoveBonus() const
       if ((*it)->isHero())
 	{
 	  Hero *h = dynamic_cast<Hero*>(*it);
-	  if (h->getBackpack()->countStackFlightGivers() > 0)
+	  if (h->isFlyer())
 	    {
 	      d_bonus = Tile::isFlying();
 	      return d_bonus;
@@ -774,12 +770,8 @@ bool Stack::canJoin(const Stack *stack) const
 }
 
 //take the weakest units where their strengths add up to strength.
-std::list<guint32> Stack::determineArmiesByStrength(bool strongest, float strength) const
+std::list<guint32> Stack::determineArmiesByStrength(float strength) const
 {
-  if (strongest)
-    {
-      ;
-    }
   std::list<guint32> armies;
   float remaining = strength; 
   Stack *stack = new Stack(*this);
@@ -801,12 +793,12 @@ std::list<guint32> Stack::determineArmiesByStrength(bool strongest, float streng
 
 std::list<guint32> Stack::determineStrongArmies(float strength) const
 {
-  return determineArmiesByStrength(true, strength);
+  return determineArmiesByStrength(strength);
 }
 
 std::list<guint32> Stack::determineWeakArmies(float strength) const
 {
-  return determineArmiesByStrength(false, strength);
+  return determineArmiesByStrength(strength);
 }
 
 std::list<guint32> Stack::determineReachableArmies(Vector<int> dest) const
@@ -876,8 +868,7 @@ Stack* Stack::createNonUniqueStack(Player *player, Vector<int> pos)
 
 guint32 Stack::getMaxMoves() const
 {
-  GameMap *gm = GameMap::getInstance();
-  if (gm->getTile(getPos())->getType() != Tile::WATER)
+  if (GameMap::getInstance()->getTile(getPos())->getType() != Tile::WATER)
     return getMaxLandMoves();
   else
     return getMaxBoatMoves();
@@ -1183,9 +1174,8 @@ void Stack::getUsableItems(std::list<Item*> &items) const
       for (std::list<Item*>::iterator i = backpack_items.begin(); 
            i !=backpack_items.end(); i++)
         {
-          GameMap *gm = GameMap::getInstance();
-          Maptile::Building b = gm->getBuilding(getPos());
-          Ruin *ruin = gm->getRuin(getPos());
+          Maptile::Building b = GameMap::getInstance()->getBuilding(getPos());
+          Ruin *ruin = GameMap::getInstance()->getRuin(getPos());
           bool ruin_has_occupant = false;
           if (ruin)
             {
@@ -1193,7 +1183,7 @@ void Stack::getUsableItems(std::list<Item*> &items) const
                 ruin_has_occupant = true;
             }
           bool victims = Playerlist::getInstance()->countPlayersAlive() > 1;
-          if ((*i)->isCurrentlyUsable(b, gm->getBackpacks().empty() == false, 
+          if ((*i)->isCurrentlyUsable(b, !GameMap::getInstance()->getBackpacks().empty(),
                                       victims, ruin_has_occupant,
                                       GameMap::friendlyCitiesPresent(),
                                       GameMap::enemyCitiesPresent(),
@@ -1201,7 +1191,8 @@ void Stack::getUsableItems(std::list<Item*> &items) const
             i = backpack_items.erase(i);
         }
       if (backpack_items.size() > 0)
-        items.merge(backpack_items);
+        items.insert(std::end(items),
+                     std::begin(backpack_items), std::end(backpack_items));
     }
   return;
 }
@@ -1230,8 +1221,7 @@ void Stack::kill()
 bool Stack::killArmyUnitsInBoats()
 {
   bool retval = false;
-  GameMap *gm = GameMap::getInstance();
-  if (gm->getTile(getPos())->getType() != Tile::WATER)
+  if (GameMap::getInstance()->getTile(getPos())->getType() != Tile::WATER)
     return retval;
   if (isFlying())
     return retval;
@@ -1324,8 +1314,7 @@ void Stack::sortByIds(std::list<guint32> ids)
 
 void Stack::updateShipStatus(Vector<int> dest)
 {
-  GameMap *gm = GameMap::getInstance();
-  bool to_water = (gm->getTile(dest.x,dest.y)->getType() == Tile::WATER);
+  bool to_water = (GameMap::getInstance()->getTile(dest)->getType() == Tile::WATER);
   bool to_bridge = (GameMap::getBridge(dest) != NULL);
   for (Stack::iterator it = begin(); it != end(); it++)
     {
@@ -1363,4 +1352,35 @@ bool Stack::removeArmiesWithoutArmyType(guint32 armyset)
     }
   return removedArmy;
 }
+
+/*
+ * the idea here is that a hero has a stack flight item, and we don't want
+ * to split the stack when it is in the precarious position of stranding
+ * army units in water or mountains.
+ *
+ * this routine just detects when splitting a stack would strand an army unit.
+ *
+ * we could do a better job detecting the non-flyers in the stack.
+ */
+bool Stack::fliesWithItemAndNonFlyersOverWaterOrMountains() const
+{
+  Maptile *mtile = GameMap::getInstance()->getTile(getPos());
+  bool on_water = mtile->getType() == Tile::WATER &&
+    mtile->getBuilding() != Maptile::BRIDGE;
+  bool on_mountains = mtile->getType() == Tile::MOUNTAIN &&
+    mtile->getBuilding() != Maptile::ROAD;
+  if (!on_water && !on_mountains)
+    return false;
+  bool flies_with_item = false;
+  std::vector<guint32> ids;
+  getHeroes(ids);
+  for (auto id : ids)
+    {
+      Army *a = getArmyById(id);
+      Hero *h = dynamic_cast<Hero*>(a);
+      if (h->getBackpack()->countStackFlightGivers() > 0)
+        flies_with_item = true;
+    }
+  return flies_with_item && size() > 1 && (on_water || on_mountains);
+}
 // End of file
